@@ -1,3 +1,11 @@
+// Copyright © 2024 Mikhail Hogrefe
+//
+// This file is part of Malachite.
+//
+// Malachite is free software: you can redistribute it and/or modify it under the terms of the GNU
+// Lesser General Public License (LGPL) as published by the Free Software Foundation; either version
+// 3 of the License, or (at your option) any later version. See <https://www.gnu.org/licenses/>.
+
 use malachite_base::named::Named;
 use malachite_base::num::arithmetic::root::{
     cbrt_chebyshev_approx_u32, cbrt_chebyshev_approx_u64, fast_ceiling_root_u32,
@@ -334,6 +342,7 @@ fn demo_root_assign_rem<T: PrimitiveUnsigned>(gm: GenMode, config: &GenConfig, l
 macro_rules! benchmark_floor_cbrt_algorithms_unsigned {
     ($t:ident, $f:ident) => {
         fn $f(gm: GenMode, config: &GenConfig, limit: usize, file_name: &str) {
+            #[allow(clippy::cast_lossless)]
             run_benchmark(
                 &format!("{}.floor_root(3)", $t::NAME),
                 BenchmarkType::Algorithms,
@@ -575,6 +584,7 @@ fn benchmark_cbrt_assign_rem<T: PrimitiveUnsigned>(
 macro_rules! benchmark_floor_root_algorithms_unsigned {
     ($t:ident, $f:ident) => {
         fn $f(gm: GenMode, config: &GenConfig, limit: usize, file_name: &str) {
+            #[allow(clippy::cast_lossless)]
             run_benchmark(
                 &format!("{}.floor_root(u64)", $t::NAME),
                 BenchmarkType::Algorithms,
