@@ -1,3 +1,11 @@
+// Copyright © 2024 Mikhail Hogrefe
+//
+// This file is part of Malachite.
+//
+// Malachite is free software: you can redistribute it and/or modify it under the terms of the GNU
+// Lesser General Public License (LGPL) as published by the Free Software Foundation; either version
+// 3 of the License, or (at your option) any later version. See <https://www.gnu.org/licenses/>.
+
 use crate::num::logic::traits::NotAssign;
 
 impl NotAssign for bool {
@@ -22,6 +30,6 @@ impl NotAssign for bool {
     /// ```
     #[inline]
     fn not_assign(&mut self) {
-        *self = !*self
+        *self = !*self;
     }
 }
