@@ -1,3 +1,11 @@
+// Copyright © 2024 Mikhail Hogrefe
+//
+// This file is part of Malachite.
+//
+// Malachite is free software: you can redistribute it and/or modify it under the terms of the GNU
+// Lesser General Public License (LGPL) as published by the Free Software Foundation; either version
+// 3 of the License, or (at your option) any later version. See <https://www.gnu.org/licenses/>.
+
 use crate::num::arithmetic::traits::{Ceiling, CeilingAssign};
 
 macro_rules! impl_ceiling {
@@ -5,11 +13,10 @@ macro_rules! impl_ceiling {
         impl Ceiling for $f {
             type Output = $f;
 
-            /// This is a wrapper over the `ceil` functions in the standard library, for example
-            /// [this one](f32::ceil).
+            /// This is a wrapper over the `ceil` functions in [`libm`]
             #[inline]
             fn ceiling(self) -> $f {
-                $f::ceil(self)
+                libm::Libm::<$f>::ceil(self)
             }
         }
 
