@@ -1,3 +1,11 @@
+// Copyright © 2024 Mikhail Hogrefe
+//
+// This file is part of Malachite.
+//
+// Malachite is free software: you can redistribute it and/or modify it under the terms of the GNU
+// Lesser General Public License (LGPL) as published by the Free Software Foundation; either version
+// 3 of the License, or (at your option) any later version. See <https://www.gnu.org/licenses/>.
+
 use crate::num::arithmetic::traits::{Floor, FloorAssign};
 
 macro_rules! impl_floor {
@@ -5,11 +13,10 @@ macro_rules! impl_floor {
         impl Floor for $f {
             type Output = $f;
 
-            /// This is a wrapper over the `floor` functions in the standard library, for example
-            /// [this one](f32::floor).
+            /// This is a wrapper over the `floor` functions in [`libm`].
             #[inline]
             fn floor(self) -> $f {
-                $f::floor(self)
+                libm::Libm::<$f>::floor(self)
             }
         }
 
