@@ -1,3 +1,24 @@
+// Copyright © 2024 Mikhail Hogrefe
+//
+// Uses code adopted from the FLINT Library.
+//
+//      Copyright © 1991, 1992, 1993, 1994, 1996, 1997, 1999, 2000, 2001, 2002, 2003, 2004, 2005
+//      Free Software Foundation, Inc.
+//
+//      Copyright © 2006, 2007, 2008, 2009, 2010, 2015, 2016 William Hart
+//
+//      Copyright © 2010, 2011, 2021 Fredrik Johansson
+//
+//      Copyright © 2008, Peter Shrimpton
+//
+//      Copyright © 2009, Tom Boothby
+//
+// This file is part of Malachite.
+//
+// Malachite is free software: you can redistribute it and/or modify it under the terms of the GNU
+// Lesser General Public License (LGPL) as published by the Free Software Foundation; either version
+// 3 of the License, or (at your option) any later version. See <https://www.gnu.org/licenses/>.
+
 use crate::num::arithmetic::traits::{
     ModMul, ModMulAssign, ModMulPrecomputed, ModMulPrecomputedAssign, Parity, PowerOf2,
     WrappingSubAssign,
@@ -8,6 +29,8 @@ use crate::num::conversion::traits::{ExactFrom, HasHalf, JoinHalves, SplitInHalf
 use crate::num::logic::traits::LeadingZeros;
 
 pub_test! {naive_mod_mul<T: PrimitiveUnsigned>(x: T, y: T, m: T) -> T {
+    assert!(x < m, "x must be reduced mod m, but {x} >= {m}");
+    assert!(y < m, "y must be reduced mod m, but {y} >= {m}");
     let (product_1, product_0) = T::x_mul_y_to_zz(x, y);
     T::xx_div_mod_y_to_qr(product_1, product_0, m).1
 }}
@@ -248,7 +271,8 @@ pub_test! {fast_mod_mul<
     m: T,
     inv: T,
 ) -> T {
-    assert_ne!(m, T::ZERO);
+    assert!(x < m, "x must be reduced mod m, but {x} >= {m}");
+    assert!(y < m, "y must be reduced mod m, but {y} >= {m}");
     let (product_1, product_0) = (DT::from(x) * DT::from(y)).split_in_half();
     limbs_mod_preinverted::<T, DT>(product_1, product_0, m, inv)
 }}
@@ -270,8 +294,8 @@ macro_rules! impl_mod_mul_precomputed_fast {
                 $invert_limb(m << LeadingZeros::leading_zeros(m))
             }
 
-            /// Multiplies two numbers modulo a third number $m$. Assumes the inputs are already
-            /// reduced modulo $m$.
+            /// Multiplies two numbers modulo a third number $m$. The inputs must be already reduced
+            /// modulo $m$.
             ///
             /// Some precomputed data is provided; this speeds up computations involving several
             /// modular multiplications with the same modulus. The precomputed data should be
@@ -281,6 +305,9 @@ macro_rules! impl_mod_mul_precomputed_fast {
             /// # Worst-case complexity
             /// Constant time and additional memory.
             ///
+            /// # Panics
+            /// Panics if `self` or `other` are greater than or equal to `m`.
+            ///
             /// # Examples
             /// See [here](super::mod_mul#mod_mul_precomputed).
             ///
@@ -311,8 +338,8 @@ macro_rules! impl_mod_mul_precomputed_promoted {
                 u32::precompute_mod_mul_data(&u32::from(m))
             }
 
-            /// Multiplies two numbers modulo a third number $m$. Assumes the inputs are already
-            /// reduced modulo $m$.
+            /// Multiplies two numbers modulo a third number $m$. The inputs must be already reduced
+            /// modulo $m$.
             ///
             /// Some precomputed data is provided; this speeds up computations involving several
             /// modular multiplications with the same modulus. The precomputed data should be
@@ -322,6 +349,9 @@ macro_rules! impl_mod_mul_precomputed_promoted {
             /// # Worst-case complexity
             /// Constant time and additional memory.
             ///
+            /// # Panics
+            /// Panics if `self` or `other` are greater than or equal to `m`.
+            ///
             /// # Examples
             /// See [here](super::mod_mul#mod_mul_precomputed).
             ///
@@ -350,8 +380,8 @@ impl ModMulPrecomputed<u128, u128> for u128 {
     /// Constant time and additional memory.
     fn precompute_mod_mul_data(_m: &u128) {}
 
-    /// Multiplies two numbers modulo a third number $m$. Assumes the inputs are already reduced
-    /// modulo $m$.
+    /// Multiplies two numbers modulo a third number $m$. The inputs must be already reduced modulo
+    /// $m$.
     ///
     /// Some precomputed data is provided; this speeds up computations involving several modular
     /// multiplications with the same modulus. The precomputed data should be obtained using
@@ -360,6 +390,9 @@ impl ModMulPrecomputed<u128, u128> for u128 {
     /// # Worst-case complexity
     /// Constant time and additional memory.
     ///
+    /// # Panics
+    /// Panics if `self` or `other` are greater than or equal to `m`.
+    ///
     /// # Examples
     /// See [here](super::mod_mul#mod_mul_precomputed).
     ///
@@ -389,8 +422,8 @@ impl ModMulPrecomputed<usize, usize> for usize {
         }
     }
 
-    /// Multiplies two numbers modulo a third number $m$. Assumes the inputs are already reduced
-    /// modulo $m$.
+    /// Multiplies two numbers modulo a third number $m$. The inputs must be already reduced modulo
+    /// $m$.
     ///
     /// Some precomputed data is provided; this speeds up computations involving several modular
     /// multiplications with the same modulus. The precomputed data should be obtained using
@@ -399,6 +432,9 @@ impl ModMulPrecomputed<usize, usize> for usize {
     /// # Worst-case complexity
     /// Constant time and additional memory.
     ///
+    /// # Panics
+    /// Panics if `self` or `other` are greater than or equal to `m`.
+    ///
     /// # Examples
     /// See [here](super::mod_mul#mod_mul_precomputed).
     ///
@@ -423,7 +459,7 @@ impl ModMulPrecomputed<usize, usize> for usize {
 macro_rules! impl_mod_mul {
     ($t:ident) => {
         impl ModMulPrecomputedAssign<$t, $t> for $t {
-            /// Multiplies two numbers modulo a third number $m$, in place. Assumes the inputs are
+            /// Multiplies two numbers modulo a third number $m$, in place. The inputs must be
             /// already reduced modulo $m$.
             ///
             /// Some precomputed data is provided; this speeds up computations involving several
@@ -434,6 +470,9 @@ macro_rules! impl_mod_mul {
             /// # Worst-case complexity
             /// Constant time and additional memory.
             ///
+            /// # Panics
+            /// Panics if `self` or `other` are greater than or equal to `m`.
+            ///
             /// # Examples
             /// See [here](super::mod_mul#mod_mul_precomputed_assign).
             ///
@@ -448,18 +487,21 @@ macro_rules! impl_mod_mul {
         impl ModMul<$t> for $t {
             type Output = $t;
 
-            /// Multiplies two numbers modulo a third number $m$. Assumes the inputs are already
-            /// reduced modulo $m$.
+            /// Multiplies two numbers modulo a third number $m$. The inputs must be already reduced
+            /// modulo $m$.
             ///
             /// $f(x, y, m) = z$, where $x, y, z < m$ and $xy \equiv z \mod m$.
             ///
             /// # Worst-case complexity
             /// Constant time and additional memory.
             ///
+            /// # Panics
+            /// Panics if `self` or `other` are greater than or equal to `m`.
+            ///
             /// # Examples
             /// See [here](super::mod_mul#mod_mul).
             ///
-            /// This is equivalent to `nmod_mul` from `nmod_vec.h`, FLINT 2.7.1.
+            /// This is equivalent to `nmod_mul` from `nmod.h`, FLINT 2.7.1.
             #[inline]
             fn mod_mul(self, other: $t, m: $t) -> $t {
                 naive_mod_mul(self, other, m)
@@ -467,7 +509,7 @@ macro_rules! impl_mod_mul {
         }
 
         impl ModMulAssign<$t> for $t {
-            /// Multiplies two numbers modulo a third number $m$, in place. Assumes the inputs are
+            /// Multiplies two numbers modulo a third number $m$, in place. The inputs must be
             /// already reduced modulo $m$.
             ///
             /// $x \gets z$, where $x, y, z < m$ and $xy \equiv z \mod m$.
@@ -478,8 +520,11 @@ macro_rules! impl_mod_mul {
             /// # Examples
             /// See [here](super::mod_mul#mod_mul_assign).
             ///
-            /// This is equivalent to `nmod_mul` from `nmod_vec.h`, FLINT 2.7.1, where the result
-            /// is assigned to `a`.
+            /// # Panics
+            /// Panics if `self` or `other` are greater than or equal to `m`.
+            ///
+            /// This is equivalent to `nmod_mul` from `nmod.h`, FLINT 2.7.1, where the result is
+            /// assigned to `a`.
             #[inline]
             fn mod_mul_assign(&mut self, other: $t, m: $t) {
                 *self = naive_mod_mul(*self, other, m);
