@@ -1,3 +1,19 @@
+// Copyright © 2024 Mikhail Hogrefe
+//
+// Uses code adopted from the FLINT Library.
+//
+//      Copyright © 2015 William Hart
+//
+//      Copyright © 2015 Fredrik Johansson
+//
+//      Copyright © 2015 Kushagra Singh
+//
+// This file is part of Malachite.
+//
+// Malachite is free software: you can redistribute it and/or modify it under the terms of the GNU
+// Lesser General Public License (LGPL) as published by the Free Software Foundation; either version
+// 3 of the License, or (at your option) any later version. See <https://www.gnu.org/licenses/>.
+
 #[cfg(feature = "test_build")]
 use crate::num::arithmetic::sqrt::floor_inverse_checked_binary;
 #[cfg(feature = "test_build")]
@@ -13,8 +29,8 @@ use crate::num::conversion::traits::{
     RawMantissaAndExponent, RoundingFrom, SaturatingFrom, WrappingFrom,
 };
 use crate::num::logic::traits::{LowMask, SignificantBits};
-use crate::rounding_modes::RoundingMode;
-use std::cmp::Ordering;
+use crate::rounding_modes::RoundingMode::*;
+use core::cmp::Ordering::*;
 
 const U8_CUBES: [u8; 7] = [0, 1, 8, 27, 64, 125, 216];
 
@@ -480,17 +496,17 @@ pub_test! {floor_root_approx_and_refine<T: PrimitiveUnsigned, F: Fn(T) -> f64, G
         max_pow
     };
     match pow.cmp(&x) {
-        Ordering::Equal => root,
-        Ordering::Less => loop {
+        Equal => root,
+        Less => loop {
             root += T::ONE;
             pow = root.pow(exp);
             match pow.cmp(&x) {
-                Ordering::Equal => return root,
-                Ordering::Less => {}
-                Ordering::Greater => return root - T::ONE,
+                Equal => return root,
+                Less => {}
+                Greater => return root - T::ONE,
             }
         },
-        Ordering::Greater => loop {
+        Greater => loop {
             root -= T::ONE;
             pow = root.pow(exp);
             if pow <= x {
@@ -500,15 +516,14 @@ pub_test! {floor_root_approx_and_refine<T: PrimitiveUnsigned, F: Fn(T) -> f64, G
     }
 }}
 
-// Coefficients of Chebyshev's approximation polynomial (deg 2) {c0, c1, c2}
-// splitting 0.5 to 1 into 8 equal intervals
+// Coefficients of Chebyshev's approximation polynomial (deg 2) {c0, c1, c2} splitting 0.5 to 1 into
+// 8 equal intervals
 //
-// Values of these coefficients of Chebyshev's approximation polynomial have been
-// calculated from the python module, "mpmath" - http://mpmath.org/
-// function call:
-// mpmath.chebyfit(lambda x: mpmath.root(x,3), [i, j], 3, error=True)
-// where (i, j) is the  range.
+// Values of these coefficients of Chebyshev's approximation polynomial have been calculated from
+// the python module, "mpmath" - http://mpmath.org/ function call: mpmath.chebyfit(lambda x:
+// mpmath.root(x,3), [i, j], 3, error=True) where (i, j) is the  range.
 //
+// ```
 //          c0          c1           c2        range
 // 0.445434042 0.864136635 -0.335205926 [0.50000, 0.53125]
 // 0.454263239 0.830878907 -0.303884962 [0.53125, 0.56250]
@@ -526,7 +541,8 @@ pub_test! {floor_root_approx_and_refine<T: PrimitiveUnsigned, F: Fn(T) -> f64, G
 // 0.540672371 0.586548233 -0.127254189 [0.90625, 0.93750]
 // 0.546715310 0.573654340 -0.120376066 [0.93750, 0.96875]
 // 0.552627494 0.561446514 -0.114074068 [0.96875, 1.00000]
-
+// ```
+//
 // 1^(1/3), 2^(1/3), 4^(1/3)
 const FACTOR_TABLE: [f32; 3] = [1.000000, 1.259921, 1.587401];
 
@@ -566,6 +582,7 @@ pub_test! {cbrt_chebyshev_approx_u32(n: u32) -> u32 {
     let (exponent_over_3, exponent_rem) = (u32::wrapping_from(exponent) - BIAS).div_mod(3);
 
     // Calculating cube root of dec using Chebyshev approximation polynomial
+    //
     // Evaluating approx polynomial at (dec) by Estrin's scheme
     let x = f32::from_bits(mantissa);
     let row = COEFF[table_index];
@@ -606,6 +623,7 @@ pub_test! {cbrt_chebyshev_approx_u64(n: u64) -> u64 {
     let (exponent_over_3, exponent_rem) = (exponent - BIAS).div_mod(3);
 
     // Calculating cube root of dec using Chebyshev approximation polynomial
+    //
     // Evaluating approx polynomial at x by Estrin's scheme
     let x = f64::from_bits(mantissa);
     let row = COEFF[table_index];
@@ -631,8 +649,8 @@ pub_test! {cbrt_chebyshev_approx_u64(n: u64) -> u64 {
     cbrt
 }}
 
-// This is equivalent to `n_cbrt_estimate` from `ulong_extras/n_cbrt_estimate.c`, FLINT 2.7.1,
-// where `FLINT64` is `true`.
+// This is equivalent to `n_cbrt_estimate` from `ulong_extras/n_cbrt_estimate.c`, FLINT 2.7.1, where
+// `FLINT64` is `true`.
 #[cfg(feature = "test_build")]
 fn cbrt_estimate_f64(a: f64) -> f64 {
     const S: u64 = 4607182418800017408; // ((1 << 10) - 1) << 52
@@ -716,7 +734,7 @@ pub fn fast_floor_cbrt_u32(n: u32) -> u32 {
     ret
 }
 
-//TODO tune
+// TODO tune
 #[cfg(feature = "test_build")]
 const CBRT_CHEBYSHEV_THRESHOLD: u64 = 10;
 
@@ -1009,7 +1027,7 @@ pub_test! {fast_floor_root_u32(n: u32, exp: u64) -> u32 {
     // one round of Newton iteration
     let mut root = u32::rounding_from(
         (f64::from(n / x.pow(exp - 1)) - f64::from(x)) * INV_TABLE[exp_usize],
-        RoundingMode::Down,
+        Down,
     ).0;
     if root >= upper_limit {
         root = upper_limit - 1;
@@ -1032,8 +1050,8 @@ pub_test! {fast_floor_root_u32(n: u32, exp: u64) -> u32 {
     root
 }}
 
-// This is equivalent to `n_root` from `ulong_extras/root.c`, FLINT 2.7.1, where `FLINT64` is
-// `true` and `root` is nonzero.
+// This is equivalent to `n_root` from `ulong_extras/root.c`, FLINT 2.7.1, where `FLINT64` is `true`
+// and `root` is nonzero.
 pub_test! {fast_floor_root_u64(n: u64, exp: u64) -> u64 {
     assert_ne!(exp, 0);
     if n < 2 || exp == 1 {
@@ -1055,7 +1073,7 @@ pub_test! {fast_floor_root_u64(n: u64, exp: u64) -> u64 {
     // one round of Newton iteration
     let mut root = u64::rounding_from(
         (((n / x.saturating_pow(exp - 1)) as f64) - x as f64) * INV_TABLE[exp_usize],
-        RoundingMode::Down,
+        Down,
     ).0;
     if root >= upper_limit {
         root = upper_limit - 1;
@@ -1100,7 +1118,7 @@ pub_test! {fast_ceiling_root_u32(n: u32, exp: u64) -> u32 {
     // one round of Newton iteration
     let mut root = u32::rounding_from(
         (f64::from(n / x.pow(exp - 1)) - f64::from(x)) * INV_TABLE[exp_usize],
-        RoundingMode::Down,
+        Down,
     ).0;
     if root >= upper_limit {
         root = upper_limit - 1;
@@ -1149,7 +1167,7 @@ pub_test! {fast_ceiling_root_u64(n: u64, exp: u64) -> u64 {
     // one round of Newton iteration
     let mut root = u64::rounding_from(
         (((n / x.pow(exp - 1)) as f64) - x as f64) * INV_TABLE[exp_usize],
-        RoundingMode::Down,
+        Down,
     ).0;
     if root >= upper_limit {
         root = upper_limit - 1;
@@ -1198,7 +1216,7 @@ pub_test! {fast_checked_root_u32(n: u32, exp: u64) -> Option<u32> {
     // one round of Newton iteration
     let mut root = u32::rounding_from(
         (f64::from(n / x.pow(exp - 1)) - f64::from(x)) * INV_TABLE[exp_usize],
-        RoundingMode::Down,
+        Down,
     ).0;
     if root >= upper_limit {
         root = upper_limit - 1;
@@ -1247,7 +1265,7 @@ pub_test! {fast_checked_root_u64(n: u64, exp: u64) -> Option<u64> {
     // one round of Newton iteration
     let mut root = u64::rounding_from(
         (((n / x.pow(exp - 1)) as f64) - x as f64) * INV_TABLE[exp_usize],
-        RoundingMode::Down,
+        Down,
     ).0;
     if root >= upper_limit {
         root = upper_limit - 1;
@@ -1297,7 +1315,7 @@ pub_test! {fast_root_rem_u32(n: u32, exp: u64) -> (u32, u32) {
     // one round of Newton iteration
     let mut root = u32::rounding_from(
         (f64::from(n / x.pow(exp - 1)) - f64::from(x)) * INV_TABLE[exp_usize],
-        RoundingMode::Down,
+        Down,
     ).0;
     if root >= upper_limit {
         root = upper_limit - 1;
@@ -1343,7 +1361,7 @@ pub_test! {fast_root_rem_u64(n: u64, exp: u64) -> (u64, u64) {
     // one round of Newton iteration
     let mut root = u64::rounding_from(
         (((n / x.pow(exp - 1)) as f64) - x as f64) * INV_TABLE[exp_usize],
-        RoundingMode::Down,
+        Down,
     ).0;
     if root >= upper_limit {
         root = upper_limit - 1;
@@ -1377,7 +1395,7 @@ pub fn floor_root_binary<T: PrimitiveUnsigned>(x: T, exp: u64) -> T {
         if bits <= exp {
             T::ONE
         } else {
-            let p = T::power_of_2(bits.div_round(exp, RoundingMode::Ceiling).0);
+            let p = T::power_of_2(bits.div_round(exp, Ceiling).0);
             floor_inverse_checked_binary(|i| i.checked_pow(exp), x, p >> 1, p)
         }
     }
@@ -1431,7 +1449,7 @@ impl FloorRoot<u64> for u8 {
     fn floor_root(self, exp: u64) -> u8 {
         match (self, exp) {
             (_, 0) => panic!(),
-            (0, _) | (1, _) | (_, 1) => self,
+            (0 | 1, _) | (_, 1) => self,
             (_, 8..=u64::MAX) => 1,
             (x, 2) => x.floor_sqrt(),
             (x, 3) => u8::wrapping_from(match U8_CUBES.binary_search(&x) {
@@ -1442,10 +1460,10 @@ impl FloorRoot<u64> for u8 {
             (x, 4) if x < 81 => 2,
             (x, 5) if x < 32 => 1,
             (x, 5) if x < 243 => 2,
-            (_, 4) | (_, 5) => 3,
+            (_, 4 | 5) => 3,
             (x, 6) if x < 64 => 1,
             (x, 7) if x < 128 => 1,
-            (_, 6) | (_, 7) => 2,
+            (_, 6 | 7) => 2,
         }
     }
 }
@@ -1471,7 +1489,7 @@ impl CeilingRoot<u64> for u8 {
     fn ceiling_root(self, exp: u64) -> u8 {
         match (self, exp) {
             (_, 0) => panic!(),
-            (0, _) | (1, _) | (_, 1) => self,
+            (0 | 1, _) | (_, 1) => self,
             (_, 8..=u64::MAX) => 2,
             (x, 2) => x.ceiling_sqrt(),
             (x, 3) => u8::wrapping_from(match U8_CUBES.binary_search(&x) {
@@ -1481,10 +1499,10 @@ impl CeilingRoot<u64> for u8 {
             (x, 4) if x <= 81 => 3,
             (x, 5) if x <= 32 => 2,
             (x, 5) if x <= 243 => 3,
-            (_, 4) | (_, 5) => 4,
+            (_, 4 | 5) => 4,
             (x, 6) if x <= 64 => 2,
             (x, 7) if x <= 128 => 2,
-            (_, 6) | (_, 7) => 3,
+            (_, 6 | 7) => 3,
         }
     }
 }
@@ -1516,7 +1534,7 @@ impl CheckedRoot<u64> for u8 {
     fn checked_root(self, exp: u64) -> Option<u8> {
         match (self, exp) {
             (_, 0) => panic!(),
-            (0, _) | (1, _) | (_, 1) => Some(self),
+            (0 | 1, _) | (_, 1) => Some(self),
             (x, 2) => x.checked_sqrt(),
             (x, 3) => U8_CUBES.binary_search(&x).ok().map(u8::wrapping_from),
             (16, 4) | (32, 5) | (64, 6) | (128, 7) => Some(2),
@@ -1549,7 +1567,7 @@ impl RootRem<u64> for u8 {
     fn root_rem(self, exp: u64) -> (u8, u8) {
         match (self, exp) {
             (_, 0) => panic!(),
-            (0, _) | (1, _) | (_, 1) => (self, 0),
+            (0 | 1, _) | (_, 1) => (self, 0),
             (x, 8..=u64::MAX) => (1, x - 1),
             (x, 2) => x.sqrt_rem(),
             (x, 3) => match U8_CUBES.binary_search(&x) {
@@ -1958,8 +1976,8 @@ impl CeilingRoot<u64> for usize {
 impl CheckedRoot<u64> for usize {
     type Output = usize;
 
-    /// Returns the the $n$th root of a [`usize`], or `None` if the [`usize`] is not a perfect
-    /// $n$th power.
+    /// Returns the the $n$th root of a [`usize`], or `None` if the [`usize`] is not a perfect $n$th
+    /// power.
     ///
     /// $$
     /// f(x, n) = \\begin{cases}
@@ -2130,8 +2148,8 @@ impl RootRem<u64> for u128 {
     type RootOutput = u128;
     type RemOutput = u128;
 
-    /// Returns the floor of the $n$th root of a [`u128`], and the remainder (the difference
-    /// between the [`u128`] and the $n$th power of the floor).
+    /// Returns the floor of the $n$th root of a [`u128`], and the remainder (the difference between
+    /// the [`u128`] and the $n$th power of the floor).
     ///
     /// $f(x, n) = (\lfloor\sqrt\[n\]{x}\rfloor, x - \lfloor\sqrt\[n\]{x}\rfloor^n)$.
     ///
@@ -2161,8 +2179,8 @@ macro_rules! impl_root_assign_rem {
         impl RootAssignRem<u64> for $t {
             type RemOutput = $t;
 
-            /// Replaces an integer with the floor of its $n$th root, and returns the remainder
-            /// (the difference between the original integer and the $n$th power of the floor).
+            /// Replaces an integer with the floor of its $n$th root, and returns the remainder (the
+            /// difference between the original integer and the $n$th power of the floor).
             ///
             /// $f(x, n) = x - \lfloor\sqrt\[n\]{x}\rfloor^n$,
             ///
