@@ -1,11 +1,19 @@
+// Copyright © 2024 Mikhail Hogrefe
+//
+// This file is part of Malachite.
+//
+// Malachite is free software: you can redistribute it and/or modify it under the terms of the GNU
+// Lesser General Public License (LGPL) as published by the Free Software Foundation; either version
+// 3 of the License, or (at your option) any later version. See <https://www.gnu.org/licenses/>.
+
 use crate::num::arithmetic::traits::{ShrRound, ShrRoundAssign, UnsignedAbs};
 use crate::num::basic::integers::PrimitiveInt;
 use crate::num::basic::signeds::PrimitiveSigned;
 use crate::num::basic::unsigneds::PrimitiveUnsigned;
 use crate::num::conversion::traits::WrappingFrom;
-use crate::rounding_modes::RoundingMode;
-use std::cmp::Ordering;
-use std::ops::{Shl, ShlAssign, Shr, ShrAssign};
+use crate::rounding_modes::RoundingMode::{self, *};
+use core::cmp::Ordering::{self, *};
+use core::ops::{Shl, ShlAssign, Shr, ShrAssign};
 
 fn shr_round_unsigned_unsigned<
     T: PrimitiveUnsigned + Shl<U, Output = T> + Shr<U, Output = T>,
@@ -16,71 +24,56 @@ fn shr_round_unsigned_unsigned<
     rm: RoundingMode,
 ) -> (T, Ordering) {
     if bits == U::ZERO || x == T::ZERO {
-        return (x, Ordering::Equal);
+        return (x, Equal);
     }
     let width = U::wrapping_from(T::WIDTH);
     match rm {
-        RoundingMode::Down | RoundingMode::Floor if bits >= width => (T::ZERO, Ordering::Less),
-        RoundingMode::Down | RoundingMode::Floor => {
+        Down | Floor if bits >= width => (T::ZERO, Less),
+        Down | Floor => {
             let shifted = x >> bits;
-            (
-                shifted,
-                if shifted << bits == x {
-                    Ordering::Equal
-                } else {
-                    Ordering::Less
-                },
-            )
+            (shifted, if shifted << bits == x { Equal } else { Less })
         }
-        RoundingMode::Up | RoundingMode::Ceiling if bits >= width => (T::ONE, Ordering::Greater),
-        RoundingMode::Up | RoundingMode::Ceiling => {
+        Up | Ceiling if bits >= width => (T::ONE, Greater),
+        Up | Ceiling => {
             let shifted = x >> bits;
             if shifted << bits == x {
-                (shifted, Ordering::Equal)
+                (shifted, Equal)
             } else {
-                (shifted + T::ONE, Ordering::Greater)
+                (shifted + T::ONE, Greater)
             }
         }
-        RoundingMode::Nearest if bits == width && x > T::power_of_2(T::WIDTH - 1) => {
-            (T::ONE, Ordering::Greater)
-        }
-        RoundingMode::Nearest if bits >= width => (T::ZERO, Ordering::Less),
-        RoundingMode::Nearest => {
+        Nearest if bits == width && x > T::power_of_2(T::WIDTH - 1) => (T::ONE, Greater),
+        Nearest if bits >= width => (T::ZERO, Less),
+        Nearest => {
             let bm1 = bits - U::ONE;
             let mostly_shifted = x >> bm1;
             let bm1_zeros = mostly_shifted << bm1 == x;
             if mostly_shifted.even() {
                 // round down
-                (
-                    mostly_shifted >> 1,
-                    if bm1_zeros {
-                        Ordering::Equal
-                    } else {
-                        Ordering::Less
-                    },
-                )
+                (mostly_shifted >> 1, if bm1_zeros { Equal } else { Less })
             } else if !bm1_zeros {
                 // round up
-                ((mostly_shifted >> 1) + T::ONE, Ordering::Greater)
+                ((mostly_shifted >> 1) + T::ONE, Greater)
             } else {
                 // result is half-integer; round to even
                 let shifted: T = mostly_shifted >> 1;
                 if shifted.even() {
-                    (shifted, Ordering::Less)
+                    (shifted, Less)
                 } else {
-                    (shifted + T::ONE, Ordering::Greater)
+                    (shifted + T::ONE, Greater)
                 }
             }
         }
-        RoundingMode::Exact if bits >= width => {
+        Exact if bits >= width => {
             panic!("Right shift is not exact: {x} >> {bits}");
         }
-        RoundingMode::Exact => {
+        Exact => {
             let shifted = x >> bits;
-            if shifted << bits != x {
-                panic!("Right shift is not exact: {x} >> {bits}");
-            }
-            (shifted, Ordering::Equal)
+            assert!(
+                shifted << bits == x,
+                "Right shift is not exact: {x} >> {bits}"
+            );
+            (shifted, Equal)
         }
     }
 }
@@ -94,46 +87,46 @@ fn shr_round_assign_unsigned_unsigned<
     rm: RoundingMode,
 ) -> Ordering {
     if bits == U::ZERO || *x == T::ZERO {
-        return Ordering::Equal;
+        return Equal;
     }
     let width = U::wrapping_from(T::WIDTH);
     match rm {
-        RoundingMode::Down | RoundingMode::Floor if bits >= width => {
+        Down | Floor if bits >= width => {
             *x = T::ZERO;
-            Ordering::Less
+            Less
         }
-        RoundingMode::Down | RoundingMode::Floor => {
+        Down | Floor => {
             let original = *x;
             *x >>= bits;
             if *x << bits == original {
-                Ordering::Equal
+                Equal
             } else {
-                Ordering::Less
+                Less
             }
         }
-        RoundingMode::Up | RoundingMode::Ceiling if bits >= width => {
+        Up | Ceiling if bits >= width => {
             *x = T::ONE;
-            Ordering::Greater
+            Greater
         }
-        RoundingMode::Up | RoundingMode::Ceiling => {
+        Up | Ceiling => {
             let original = *x;
             *x >>= bits;
             if *x << bits == original {
-                Ordering::Equal
+                Equal
             } else {
                 *x += T::ONE;
-                Ordering::Greater
+                Greater
             }
         }
-        RoundingMode::Nearest if bits == width && *x > T::power_of_2(T::WIDTH - 1) => {
+        Nearest if bits == width && *x > T::power_of_2(T::WIDTH - 1) => {
             *x = T::ONE;
-            Ordering::Greater
+            Greater
         }
-        RoundingMode::Nearest if bits >= width => {
+        Nearest if bits >= width => {
             *x = T::ZERO;
-            Ordering::Less
+            Less
         }
-        RoundingMode::Nearest => {
+        Nearest => {
             let original = *x;
             let bm1 = bits - U::ONE;
             *x >>= bm1;
@@ -143,34 +136,35 @@ fn shr_round_assign_unsigned_unsigned<
             if old_x.even() {
                 // round down
                 if bm1_zeros {
-                    Ordering::Equal
+                    Equal
                 } else {
-                    Ordering::Less
+                    Less
                 }
             } else if !bm1_zeros {
                 // round up
                 *x += T::ONE;
-                Ordering::Greater
+                Greater
             } else {
                 // result is half-integer; round to even
                 if x.even() {
-                    Ordering::Less
+                    Less
                 } else {
                     *x += T::ONE;
-                    Ordering::Greater
+                    Greater
                 }
             }
         }
-        RoundingMode::Exact if bits >= width => {
+        Exact if bits >= width => {
             panic!("Right shift is not exact: {} >>= {}", *x, bits);
         }
-        RoundingMode::Exact => {
+        Exact => {
             let original = *x;
             *x >>= bits;
-            if *x << bits != original {
-                panic!("Right shift is not exact: {original} >>= {bits}");
-            }
-            Ordering::Equal
+            assert!(
+                *x << bits == original,
+                "Right shift is not exact: {original} >>= {bits}"
+            );
+            Equal
         }
     }
 }
@@ -184,12 +178,11 @@ macro_rules! impl_shr_round_unsigned_unsigned {
 
                     /// Shifts a number right (divides it by a power of 2) and rounds according to
                     /// the specified rounding mode. An [`Ordering`] is also returned, indicating
-                    /// whether the returned value is less than, equal to, or greater than the
-                    /// exact value.
+                    /// whether the returned value is less than, equal to, or greater than the exact
+                    /// value.
                     ///
-                    /// Passing `RoundingMode::Floor` or `RoundingMode::Down` is equivalent to using
-                    /// `>>`. To test whether `RoundingMode::Exact` can be passed, use
-                    /// `self.divisible_by_power_of_2(bits)`.
+                    /// Passing `Floor` or `Down` is equivalent to using `>>`. To test whether
+                    /// `Exact` can be passed, use `self.divisible_by_power_of_2(bits)`.
                     ///
                     /// Let $q = \frac{x}{2^k}$, and let $g$ be the function that just returns the
                     /// first element of the pair, without the [`Ordering`]:
@@ -223,8 +216,7 @@ macro_rules! impl_shr_round_unsigned_unsigned {
                     /// Constant time and additional memory.
                     ///
                     /// # Panics
-                    /// Panics if `rm` is `RoundingMode::Exact` but `self` is not divisible by
-                    /// $2^b$.
+                    /// Panics if `rm` is `Exact` but `self` is not divisible by $2^b$.
                     ///
                     /// # Examples
                     /// See [here](super::shr_round#shr_round).
@@ -240,18 +232,16 @@ macro_rules! impl_shr_round_unsigned_unsigned {
                     /// indicating whether the assigned value is less than, equal to, or greater
                     /// than the exact value.
                     ///
-                    /// Passing `RoundingMode::Floor` or `RoundingMode::Down` is equivalent to
-                    /// using `>>`. To test whether `RoundingMode::Exact` can be passed, use
-                    /// `self.divisible_by_power_of_2(bits)`.
+                    /// Passing `Floor` or `Down` is equivalent to using `>>`. To test whether
+                    /// `Exact` can be passed, use `self.divisible_by_power_of_2(bits)`.
                     ///
-                    /// See the [`ShrRound`](super::traits::ShrRound) documentation for details.
+                    /// See the [`ShrRound`] documentation for details.
                     ///
                     /// # Worst-case complexity
                     /// Constant time and additional memory.
                     ///
                     /// # Panics
-                    /// Panics if `rm` is `RoundingMode::Exact` but `self` is not divisible by
-                    /// $2^b$.
+                    /// Panics if `rm` is `Exact` but `self` is not divisible by $2^b$.
                     ///
                     /// # Examples
                     /// See [here](super::shr_round#shr_round_assign).
@@ -304,12 +294,11 @@ macro_rules! impl_shr_round_signed_unsigned {
 
                     /// Shifts a number right (divides it by a power of 2) and rounds according to
                     /// the specified rounding mode. An [`Ordering`] is also returned, indicating
-                    /// whether the returned value is less than, equal to, or greater than the
-                    /// exact value.
+                    /// whether the returned value is less than, equal to, or greater than the exact
+                    /// value.
                     ///
-                    /// Passing `RoundingMode::Floor` or `RoundingMode::Down` is equivalent to
-                    /// using `>>`. To test whether `RoundingMode::Exact` can be passed, use
-                    /// `self.divisible_by_power_of_2(bits)`.
+                    /// Passing `Floor` or `Down` is equivalent to using `>>`. To test whether
+                    /// `Exact` can be passed, use `self.divisible_by_power_of_2(bits)`.
                     ///
                     /// Let $q = \frac{x}{2^k}$, and let $g$ be the function that just returns the
                     /// first element of the pair, without the [`Ordering`]:
@@ -343,8 +332,7 @@ macro_rules! impl_shr_round_signed_unsigned {
                     /// Constant time and additional memory.
                     ///
                     /// # Panics
-                    /// Panics if `rm` is `RoundingMode::Exact` but `self` is not divisible by
-                    /// $2^b$.
+                    /// Panics if `rm` is `Exact` but `self` is not divisible by $2^b$.
                     ///
                     /// # Examples
                     /// See [here](super::shr_round#shr_round).
@@ -360,16 +348,14 @@ macro_rules! impl_shr_round_signed_unsigned {
                     /// indicating whether the assigned value is less than, equal to, or greater
                     /// than the exact value.
                     ///
-                    /// Passing `RoundingMode::Floor` or `RoundingMode::Down` is equivalent to
-                    /// using `>>`. To test whether `RoundingMode::Exact` can be passed, use
-                    /// `self.divisible_by_power_of_2(bits)`.
+                    /// Passing `Floor` or `Down` is equivalent to using `>>`. To test whether
+                    /// `Exact` can be passed, use `self.divisible_by_power_of_2(bits)`.
                     ///
                     /// # Worst-case complexity
                     /// Constant time and additional memory.
                     ///
                     /// # Panics
-                    /// Panics if `rm` is `RoundingMode::Exact` but `self` is not divisible by
-                    /// $2^b$.
+                    /// Panics if `rm` is `Exact` but `self` is not divisible by $2^b$.
                     ///
                     /// # Examples
                     /// See [here](super::shr_round#shr_round_assign).
@@ -406,7 +392,7 @@ fn shr_round_primitive_signed<
             } else {
                 x << bits.unsigned_abs()
             },
-            Ordering::Equal,
+            Equal,
         )
     }
 }
@@ -429,7 +415,7 @@ fn shr_round_assign_primitive_signed<
         } else {
             *x <<= bits.unsigned_abs();
         }
-        Ordering::Equal
+        Equal
     }
 }
 
@@ -442,14 +428,13 @@ macro_rules! impl_shr_round_primitive_signed {
 
                     /// Shifts a number right (divides it by a power of 2) and rounds according to
                     /// the specified rounding mode. An [`Ordering`] is also returned, indicating
-                    /// whether the returned value is less than, equal to, or greater than the
-                    /// exact value. If `bits` is negative, then the returned [`Ordering`] is
-                    /// always `Equal`, even if the higher bits of the result are lost.
+                    /// whether the returned value is less than, equal to, or greater than the exact
+                    /// value. If `bits` is negative, then the returned [`Ordering`] is always
+                    /// `Equal`, even if the higher bits of the result are lost.
                     ///
-                    /// Passing `RoundingMode::Floor` or `RoundingMode::Down` is equivalent to
-                    /// using `>>`. To test whether `RoundingMode::Exact` can be passed, use
-                    /// `self.divisible_by_power_of_2(bits)`. Rounding might only be necessary if
-                    /// `bits` is non-negative.
+                    /// Passing `Floor` or `Down` is equivalent to using `>>`. To test whether
+                    /// `Exact` can be passed, use `self.divisible_by_power_of_2(bits)`. Rounding
+                    /// might only be necessary if `bits` is non-negative.
                     ///
                     /// Let $q = \frac{x}{2^k}$, and let $g$ be the function that just returns the
                     /// first element of the pair, without the [`Ordering`]:
@@ -487,8 +472,8 @@ macro_rules! impl_shr_round_primitive_signed {
                     /// Constant time and additional memory.
                     ///
                     /// # Panics
-                    /// Panics if `bits` is positive and `rm` is `RoundingMode::Exact` but `self` is
-                    /// not divisible by $2^b$.
+                    /// Panics if `bits` is positive and `rm` is `Exact` but `self` is not divisible
+                    /// by $2^b$.
                     ///
                     /// # Examples
                     /// See [here](super::shr_round#shr_round).
@@ -505,19 +490,18 @@ macro_rules! impl_shr_round_primitive_signed {
                     /// than the exact value. If `bits` is negative, then the returned [`Ordering`]
                     /// is always `Equal`, even if the higher bits of the result are lost.
                     ///
-                    /// Passing `RoundingMode::Floor` or `RoundingMode::Down` is equivalent to
-                    /// using `>>`. To test whether `RoundingMode::Exact` can be passed, use
-                    /// `self.divisible_by_power_of_2(bits)`. Rounding might only be necessary if
-                    /// `bits` is non-negative.
+                    /// Passing `Floor` or `Down` is equivalent to using `>>`. To test whether
+                    /// `Exact` can be passed, use `self.divisible_by_power_of_2(bits)`. Rounding
+                    /// might only be necessary if `bits` is non-negative.
                     ///
-                    /// See the [`ShrRound`](super::traits::ShrRound) documentation for details.
+                    /// See the [`ShrRound`] documentation for details.
                     ///
                     /// # Worst-case complexity
                     /// Constant time and additional memory.
                     ///
                     /// # Panics
-                    /// Panics if `bits` is positive and `rm` is `RoundingMode::Exact` but `self` is
-                    /// not divisible by $2^b$.
+                    /// Panics if `bits` is positive and `rm` is `Exact` but `self` is not divisible
+                    /// by $2^b$.
                     ///
                     /// # Examples
                     /// See [here](super::shr_round#shr_round_assign).
