@@ -1,5 +1,24 @@
+// Copyright © 2024 Mikhail Hogrefe
+//
+// Uses code adopted from the GNU MP Library.
+//
+//      `mpn_sqrtrem1` contributed to the GNU project by Torbjörn Granlund.
+//
+//      Copyright © 1999-2002, 2004, 2005, 2008, 2010, 2012, 2015, 2017 Free Software Foundation,
+//      Inc.
+//
+// Uses code adopted from the FLINT Library.
+//
+//      Copyright © 2009 William Hart
+//
+// This file is part of Malachite.
+//
+// Malachite is free software: you can redistribute it and/or modify it under the terms of the GNU
+// Lesser General Public License (LGPL) as published by the Free Software Foundation; either version
+// 3 of the License, or (at your option) any later version. See <https://www.gnu.org/licenses/>.
+
 use crate::num::arithmetic::traits::{
-    CeilingSqrt, CeilingSqrtAssign, CheckedSqrt, FloorSqrt, FloorSqrtAssign,
+    CeilingSqrt, CeilingSqrtAssign, CheckedSqrt, FloorSqrt, FloorSqrtAssign, Ln,
     RoundToMultipleOfPowerOf2, ShrRound, Sqrt, SqrtAssign, SqrtAssignRem, SqrtRem,
 };
 use crate::num::basic::integers::PrimitiveInt;
@@ -7,8 +26,8 @@ use crate::num::basic::signeds::PrimitiveSigned;
 use crate::num::basic::unsigneds::PrimitiveUnsigned;
 use crate::num::conversion::traits::WrappingFrom;
 use crate::num::logic::traits::SignificantBits;
-use crate::rounding_modes::RoundingMode;
-use std::cmp::Ordering;
+use crate::rounding_modes::RoundingMode::*;
+use core::cmp::Ordering::*;
 
 const U8_SQUARES: [u8; 16] = [0, 1, 4, 9, 16, 25, 36, 49, 64, 81, 100, 121, 144, 169, 196, 225];
 
@@ -117,15 +136,11 @@ pub(crate) fn floor_inverse_checked_binary<T: PrimitiveUnsigned, F: Fn(T) -> Opt
         if high <= low {
             return low;
         }
-        let mid: T = low
-            .checked_add(high)
-            .unwrap()
-            .shr_round(1, RoundingMode::Ceiling)
-            .0;
+        let mid: T = low.checked_add(high).unwrap().shr_round(1, Ceiling).0;
         match f(mid).map(|mid| mid.cmp(&x)) {
-            Some(Ordering::Equal) => return mid,
-            Some(Ordering::Less) => low = mid,
-            Some(Ordering::Greater) | None => high = mid - T::ONE,
+            Some(Equal) => return mid,
+            Some(Less) => low = mid,
+            Some(Greater) | None => high = mid - T::ONE,
         }
     }
 }
@@ -134,7 +149,7 @@ pub_test! {floor_sqrt_binary<T: PrimitiveUnsigned>(x: T) -> T {
     if x < T::TWO {
         x
     } else {
-        let p = T::power_of_2(x.significant_bits().shr_round(1, RoundingMode::Ceiling).0);
+        let p = T::power_of_2(x.significant_bits().shr_round(1, Ceiling).0);
         floor_inverse_checked_binary(T::checked_square, x, p >> 1, p)
     }
 }}
@@ -208,9 +223,8 @@ pub fn sqrt_rem_newton<
         _ => panic!(),
     };
     assert!(n.leading_zeros() < 2);
-    // Use Newton iterations for approximating 1/sqrt(a) instead of sqrt(a),
-    // since we can do the former without division.  As part of the last
-    // iteration convert from 1/sqrt(a) to sqrt(a).
+    // Use Newton iterations for approximating 1/sqrt(a) instead of sqrt(a), since we can do the
+    // former without division. As part of the last iteration convert from 1/sqrt(a) to sqrt(a).
     let i: usize = (n >> (U::WIDTH - 9)).wrapping_into(); // extract bits for table lookup
     let mut inv_sqrt = U::wrapping_from(INV_SQRT_TAB[i - 0x80]);
     inv_sqrt.set_bit(8); // initial 1/sqrt(a)
@@ -265,17 +279,17 @@ fn floor_sqrt_approx_and_refine<T: PrimitiveUnsigned, F: Fn(T) -> f64, G: Fn(f64
         sqrt.square()
     };
     match square.cmp(&x) {
-        Ordering::Equal => sqrt,
-        Ordering::Less => loop {
+        Equal => sqrt,
+        Less => loop {
             square = square.checked_add((sqrt << 1) + T::ONE).unwrap();
             sqrt += T::ONE;
             match square.cmp(&x) {
-                Ordering::Equal => return sqrt,
-                Ordering::Less => {}
-                Ordering::Greater => return sqrt - T::ONE,
+                Equal => return sqrt,
+                Less => {}
+                Greater => return sqrt - T::ONE,
             }
         },
-        Ordering::Greater => loop {
+        Greater => loop {
             square -= (sqrt << 1) - T::ONE;
             sqrt -= T::ONE;
             if square <= x {
@@ -303,21 +317,21 @@ fn ceiling_sqrt_approx_and_refine<T: PrimitiveUnsigned, F: Fn(T) -> f64, G: Fn(f
         sqrt.square()
     };
     match square.cmp(&x) {
-        Ordering::Equal => sqrt,
-        Ordering::Less => loop {
+        Equal => sqrt,
+        Less => loop {
             square = square.checked_add((sqrt << 1) + T::ONE).unwrap();
             sqrt += T::ONE;
             if square >= x {
                 return sqrt;
             }
         },
-        Ordering::Greater => loop {
+        Greater => loop {
             square -= (sqrt << 1) - T::ONE;
             sqrt -= T::ONE;
             match square.cmp(&x) {
-                Ordering::Equal => return sqrt,
-                Ordering::Greater => {}
-                Ordering::Less => return sqrt + T::ONE,
+                Equal => return sqrt,
+                Greater => {}
+                Less => return sqrt + T::ONE,
             }
         },
     }
@@ -341,23 +355,23 @@ fn checked_sqrt_approx_and_refine<T: PrimitiveUnsigned, F: Fn(T) -> f64, G: Fn(f
         sqrt.square()
     };
     match square.cmp(&x) {
-        Ordering::Equal => Some(sqrt),
-        Ordering::Less => loop {
+        Equal => Some(sqrt),
+        Less => loop {
             square = square.checked_add((sqrt << 1) + T::ONE).unwrap();
             sqrt += T::ONE;
             match square.cmp(&x) {
-                Ordering::Equal => return Some(sqrt),
-                Ordering::Less => {}
-                Ordering::Greater => return None,
+                Equal => return Some(sqrt),
+                Less => {}
+                Greater => return None,
             }
         },
-        Ordering::Greater => loop {
+        Greater => loop {
             square -= (sqrt << 1) - T::ONE;
             sqrt -= T::ONE;
             match square.cmp(&x) {
-                Ordering::Equal => return Some(sqrt),
-                Ordering::Less => return None,
-                Ordering::Greater => {}
+                Equal => return Some(sqrt),
+                Less => return None,
+                Greater => {}
             }
         },
     }
@@ -382,21 +396,21 @@ fn sqrt_rem_approx_and_refine<T: PrimitiveUnsigned, F: Fn(T) -> f64, G: Fn(f64)
         sqrt.square()
     };
     match square.cmp(&x) {
-        Ordering::Equal => (sqrt, T::ZERO),
-        Ordering::Less => loop {
+        Equal => (sqrt, T::ZERO),
+        Less => loop {
             square = square.checked_add((sqrt << 1) + T::ONE).unwrap();
             sqrt += T::ONE;
             match square.cmp(&x) {
-                Ordering::Equal => return (sqrt, T::ZERO),
-                Ordering::Less => {}
-                Ordering::Greater => {
+                Equal => return (sqrt, T::ZERO),
+                Less => {}
+                Greater => {
                     square -= (sqrt << 1) - T::ONE;
                     sqrt -= T::ONE;
                     return (sqrt, x - square);
                 }
             }
         },
-        Ordering::Greater => loop {
+        Greater => loop {
             square -= (sqrt << 1) - T::ONE;
             sqrt -= T::ONE;
             if square <= x {
@@ -417,7 +431,7 @@ fn floor_sqrt_newton_helper<
     }
     let shift = x
         .leading_zeros()
-        .round_to_multiple_of_power_of_2(1, RoundingMode::Floor)
+        .round_to_multiple_of_power_of_2(1, Floor)
         .0;
     sqrt_rem_newton::<U, S>(x << shift).0 >> (shift >> 1)
 }
@@ -433,12 +447,12 @@ fn ceiling_sqrt_newton_helper<
     }
     let shift = x
         .leading_zeros()
-        .round_to_multiple_of_power_of_2(1, RoundingMode::Floor)
+        .round_to_multiple_of_power_of_2(1, Floor)
         .0;
     let (mut sqrt, rem) = sqrt_rem_newton::<U, S>(x << shift);
     sqrt >>= shift >> 1;
     if rem != U::ZERO {
-        sqrt += U::ONE
+        sqrt += U::ONE;
     }
     sqrt
 }
@@ -454,7 +468,7 @@ fn checked_sqrt_newton_helper<
     }
     let shift = x
         .leading_zeros()
-        .round_to_multiple_of_power_of_2(1, RoundingMode::Floor)
+        .round_to_multiple_of_power_of_2(1, Floor)
         .0;
     let (sqrt, rem) = sqrt_rem_newton::<U, S>(x << shift);
     if rem == U::ZERO {
@@ -475,7 +489,7 @@ fn sqrt_rem_newton_helper<
     }
     let shift = x
         .leading_zeros()
-        .round_to_multiple_of_power_of_2(1, RoundingMode::Floor)
+        .round_to_multiple_of_power_of_2(1, Floor)
         .0;
     let (mut sqrt, rem) = sqrt_rem_newton::<U, S>(x << shift);
     if shift == 0 {
@@ -533,8 +547,8 @@ macro_rules! impl_sqrt_newton {
         impl CheckedSqrt for $u {
             type Output = $u;
 
-            /// Returns the the square root of an integer, or `None` if the integer is not a
-            /// perfect square.
+            /// Returns the the square root of an integer, or `None` if the integer is not a perfect
+            /// square.
             ///
             /// $$
             /// f(x) = \\begin{cases}
@@ -656,8 +670,8 @@ impl SqrtRem for u16 {
     type SqrtOutput = u16;
     type RemOutput = u16;
 
-    /// Returns the floor of the square root of a [`u16`], and the remainder (the difference
-    /// between the [`u16`] and the square of the floor).
+    /// Returns the floor of the square root of a [`u16`], and the remainder (the difference between
+    /// the [`u16`] and the square of the floor).
     ///
     /// $f(x) = (\lfloor\sqrt{x}\rfloor, x - \lfloor\sqrt{x}\rfloor^2)$.
     ///
@@ -794,7 +808,7 @@ impl SqrtRem for usize {
     }
 }
 
-//TODO tune
+// TODO tune
 const U128_SQRT_THRESHOLD: u64 = 125;
 const U128_MAX_SQUARE: u128 = 0xfffffffffffffffe0000000000000001;
 
@@ -837,8 +851,8 @@ impl FloorSqrt for u128 {
     ///
     /// $2^{\lceil b/2 \rceil-1} \leq \lfloor\sqrt{x}\rfloor \leq 2^{\lceil b/2 \rceil}$.
     ///
-    /// For example, since $10^9$ has 30 significant bits, we know that
-    /// $2^{14} \leq \lfloor\sqrt{10^9}\rfloor \leq 2^{15}$.
+    /// For example, since $10^9$ has 30 significant bits, we know that $2^{14} \leq
+    /// \lfloor\sqrt{10^9}\rfloor \leq 2^{15}$.
     fn floor_sqrt(self) -> u128 {
         if self.significant_bits() < U128_SQRT_THRESHOLD {
             floor_sqrt_approx_and_refine(|x| x as f64, |x| x as u128, U128_MAX_SQUARE, self)
@@ -887,8 +901,8 @@ impl CeilingSqrt for u128 {
     ///
     /// $2^{\lceil b/2 \rceil-1} \leq \lfloor\sqrt{x}\rfloor \leq 2^{\lceil b/2 \rceil}$.
     ///
-    /// For example, since $10^9$ has 30 significant bits, we know that
-    /// $2^{14} \leq \lfloor\sqrt{10^9}\rfloor \leq 2^{15}$.
+    /// For example, since $10^9$ has 30 significant bits, we know that $2^{14} \leq
+    /// \lfloor\sqrt{10^9}\rfloor \leq 2^{15}$.
     fn ceiling_sqrt(self) -> u128 {
         if self.significant_bits() < U128_SQRT_THRESHOLD {
             ceiling_sqrt_approx_and_refine(|x| x as f64, |x| x as u128, U128_MAX_SQUARE, self)
@@ -943,8 +957,8 @@ impl CheckedSqrt for u128 {
     ///
     /// $2^{\lceil b/2 \rceil-1} \leq \lfloor\sqrt{x}\rfloor \leq 2^{\lceil b/2 \rceil}$.
     ///
-    /// For example, since $10^9$ has 30 significant bits, we know that
-    /// $2^{14} \leq \lfloor\sqrt{10^9}\rfloor \leq 2^{15}$.
+    /// For example, since $10^9$ has 30 significant bits, we know that $2^{14} \leq
+    /// \lfloor\sqrt{10^9}\rfloor \leq 2^{15}$.
     fn checked_sqrt(self) -> Option<u128> {
         if self.significant_bits() < U128_SQRT_THRESHOLD {
             checked_sqrt_approx_and_refine(|x| x as f64, |x| x as u128, U128_MAX_SQUARE, self)
@@ -995,8 +1009,8 @@ impl SqrtRem for u128 {
     ///
     /// $2^{\lceil b/2 \rceil-1} \leq \lfloor\sqrt{x}\rfloor \leq 2^{\lceil b/2 \rceil}$.
     ///
-    /// For example, since $10^9$ has 30 significant bits, we know that
-    /// $2^{14} \leq \lfloor\sqrt{10^9}\rfloor \leq 2^{15}$.
+    /// For example, since $10^9$ has 30 significant bits, we know that $2^{14} \leq
+    /// \lfloor\sqrt{10^9}\rfloor \leq 2^{15}$.
     fn sqrt_rem(self) -> (u128, u128) {
         if self.significant_bits() < U128_SQRT_THRESHOLD {
             sqrt_rem_approx_and_refine(|x| x as f64, |x| x as u128, U128_MAX_SQUARE, self)
@@ -1061,8 +1075,8 @@ macro_rules! impl_sqrt_signed {
         impl CheckedSqrt for $s {
             type Output = $s;
 
-            /// Returns the the square root of an integer, or `None` if the integer is not a
-            /// perfect square.
+            /// Returns the the square root of an integer, or `None` if the integer is not a perfect
+            /// square.
             ///
             /// $$
             /// f(x) = \\begin{cases}
@@ -1170,7 +1184,17 @@ macro_rules! impl_sqrt_primitive_float {
 
             #[inline]
             fn sqrt(self) -> $f {
-                $f::sqrt(self)
+                libm::Libm::<$f>::sqrt(self)
+            }
+        }
+
+        // TODO move to better location
+        impl Ln for $f {
+            type Output = Self;
+
+            #[inline]
+            fn ln(self) -> $f {
+                libm::Libm::<$f>::log(self)
             }
         }
 
