@@ -1,6 +1,14 @@
+// Copyright © 2024 Mikhail Hogrefe
+//
+// This file is part of Malachite.
+//
+// Malachite is free software: you can redistribute it and/or modify it under the terms of the GNU
+// Lesser General Public License (LGPL) as published by the Free Software Foundation; either version
+// 3 of the License, or (at your option) any later version. See <https://www.gnu.org/licenses/>.
+
 use crate::num::basic::traits::Two;
 use crate::rounding_modes::RoundingMode;
-use std::cmp::Ordering;
+use core::cmp::Ordering;
 
 /// Takes the absolute value of a number. Assumes that the number has a representable absolute
 /// number.
@@ -23,6 +31,18 @@ pub trait UnsignedAbs {
     fn unsigned_abs(self) -> Self::Output;
 }
 
+/// Subtracts two numbers and takes the absolute value of the difference.
+pub trait AbsDiff<RHS = Self> {
+    type Output;
+
+    fn abs_diff(self, other: RHS) -> Self::Output;
+}
+
+/// Replaces a number with the absolute value of its difference with another number.
+pub trait AbsDiffAssign<RHS = Self> {
+    fn abs_diff_assign(&mut self, other: RHS);
+}
+
 /// Adds a number and the product of two other numbers.
 pub trait AddMul<Y = Self, Z = Self> {
     type Output;
@@ -114,8 +134,8 @@ pub trait CheckedNeg {
     fn checked_neg(self) -> Option<Self::Output>;
 }
 
-/// Finds the smallest integer power of 2 greater than or equal to a number, returning `None` if
-/// the result is not representable.
+/// Finds the smallest integer power of 2 greater than or equal to a number, returning `None` if the
+/// result is not representable.
 pub trait CheckedNextPowerOf2 {
     type Output;
 
@@ -193,8 +213,8 @@ pub trait DivAssignMod<RHS = Self> {
     fn div_assign_mod(&mut self, other: RHS) -> Self::ModOutput;
 }
 
-/// Divides two numbers, returning the quotient and remainder. The quotient is rounded towards
-/// zero, and the remainder has the same sign as the dividend (first input).
+/// Divides two numbers, returning the quotient and remainder. The quotient is rounded towards zero,
+/// and the remainder has the same sign as the dividend (first input).
 ///
 /// The quotient and remainder satisfy $x = qy + r$ and $0 \leq |r| < |y|$.
 pub trait DivRem<RHS = Self> {
@@ -236,8 +256,8 @@ pub trait CeilingDivAssignNegMod<RHS = Self> {
 }
 
 /// Divides a number by another number, returning the quotient and remainder. The quotient is
-/// rounded towards positive infinity and the remainder has the opposite sign as the divisor
-/// (second input).
+/// rounded towards positive infinity and the remainder has the opposite sign as the divisor (second
+/// input).
 ///
 /// The quotient and remainder satisfy $x = qy + r$ and $0 \leq |r| < |y|$.
 pub trait CeilingDivMod<RHS = Self> {
@@ -248,8 +268,8 @@ pub trait CeilingDivMod<RHS = Self> {
 }
 
 /// Divides a number by another number in place, taking the quotient and returning the remainder.
-/// The quotient is rounded towards positive infinity and the remainder has the opposite sign of
-/// the divisor (second input).
+/// The quotient is rounded towards positive infinity and the remainder has the opposite sign of the
+/// divisor (second input).
 ///
 /// The quotient and remainder satisfy $x = qy + r$ and $0 \leq |r| < |y|$.
 pub trait CeilingDivAssignMod<RHS = Self> {
@@ -294,8 +314,8 @@ pub trait EqMod<RHS = Self, M = Self> {
     fn eq_mod(self, other: RHS, m: M) -> bool;
 }
 
-/// Computes the GCD (greatest common divisor) of two numbers $a$ and $b$, and also the
-/// coefficients $x$ and $y$ in Bézout's identity $ax+by=\gcd(a,b)$.
+/// Computes the GCD (greatest common divisor) of two numbers $a$ and $b$, and also the coefficients
+/// $x$ and $y$ in Bézout's identity $ax+by=\gcd(a,b)$.
 ///
 /// The are infinitely many $x$, $y$ that satisfy the identity, so the full specification is more
 /// detailed:
@@ -305,9 +325,9 @@ pub trait EqMod<RHS = Self, M = Self> {
 /// - $f(a, ak) = (-a, -1, 0)$ if $a < 0$ and $k \neq 1$.
 /// - $f(bk, b) = (b, 0, 1)$ if $b > 0$.
 /// - $f(bk, b) = (-b, 0, -1)$ if $b < 0$.
-/// - $f(a, b) = (g, x, y)$ if $a \neq 0$ and $b \neq 0$ and $\gcd(a, b) \neq \min(|a|, |b|)$,
-///   where $g = \gcd(a, b) \geq 0$, $ax + by = g$, $x \leq \lfloor b/g \rfloor$, and
-///   $y \leq \lfloor a/g \rfloor$.
+/// - $f(a, b) = (g, x, y)$ if $a \neq 0$ and $b \neq 0$ and $\gcd(a, b) \neq \min(|a|, |b|)$, where
+///   $g = \gcd(a, b) \geq 0$, $ax + by = g$, $x \leq \lfloor b/g \rfloor$, and $y \leq \lfloor a/g
+///   \rfloor$.
 pub trait ExtendedGcd<RHS = Self> {
     type Gcd;
     type Cofactor;
@@ -315,36 +335,53 @@ pub trait ExtendedGcd<RHS = Self> {
     fn extended_gcd(self, other: RHS) -> (Self::Gcd, Self::Cofactor, Self::Cofactor);
 }
 
-// TODOOOOO
-
+/// Computes the factorial of a `u64`.
 pub trait Factorial {
     fn factorial(n: u64) -> Self;
 }
 
+/// Computes the factorial of a `u64`, returning `None` if the result is too large to be
+/// represented.
 pub trait CheckedFactorial: Sized {
     fn checked_factorial(n: u64) -> Option<Self>;
 }
 
+/// Computes the double factorial of a `u64`. The double factorial of a non-negative integer is the
+/// product of all the positive integers that are less than or equal to it and have the same parity
+/// as it.
 pub trait DoubleFactorial {
     fn double_factorial(n: u64) -> Self;
 }
 
+/// Computes the double factorial of a `u64`, returning `None` if the result is too large to be
+/// represented. The double factorial of a non-negative integer is the product of all the positive
+/// integers that are less than or equal to it and have the same parity as it.
 pub trait CheckedDoubleFactorial: Sized {
     fn checked_double_factorial(n: u64) -> Option<Self>;
 }
 
+/// Computes the $m$-multifactorial of a `u64`. The $m$-multifactorial of a non-negative integer $n$
+/// is the product of all integers $k$ such that $0<k\leq n$ and $k\equiv n \pmod m$.
 pub trait Multifactorial {
     fn multifactorial(n: u64, m: u64) -> Self;
 }
 
+/// Computes the $m$-multifactorial of a `u64`, returning `None` if the result is too large to be
+/// represented. The $m$-multifactorial of a non-negative integer $n$ is the product of all integers
+/// $k$ such that $0<k\leq n$ and $k\equiv n \pmod m$.
 pub trait CheckedMultifactorial: Sized {
     fn checked_multifactorial(n: u64, m: u64) -> Option<Self>;
 }
 
+/// Computes the subfactorial of a `u64`. The subfactorial of a non-negative integer $n$ counts the
+/// number of derangements of $n$ elements, which are the permutations in which no element is fixed.
 pub trait Subfactorial {
     fn subfactorial(n: u64) -> Self;
 }
 
+/// Computes the subfactorial of a `u64`, returning `None` if the result is too large to be
+/// represented. The subfactorial of a non-negative integer $n$ counts the number of derangements of
+/// $n$ elements, which are the permutations in which no element is fixed.
 pub trait CheckedSubfactorial: Sized {
     fn checked_subfactorial(n: u64) -> Option<Self>;
 }
@@ -390,8 +427,15 @@ pub trait LcmAssign<RHS = Self> {
     fn lcm_assign(&mut self, other: RHS);
 }
 
-/// Calculates the LCM (least common multiple) of two numbers, returning `None` if the result is
-/// not representable.
+/// Takes the natural logarithm of a number.
+pub trait Ln {
+    type Output;
+
+    fn ln(self) -> Self::Output;
+}
+
+/// Calculates the LCM (least common multiple) of two numbers, returning `None` if the result is not
+/// representable.
 pub trait CheckedLcm<RHS = Self> {
     type Output;
 
@@ -480,20 +524,20 @@ pub trait CeilingLogBasePowerOf2<POW> {
     fn ceiling_log_base_power_of_2(self, pow: POW) -> Self::Output;
 }
 
-/// Adds two numbers modulo a third number $m$. Assumes the inputs are already reduced modulo $m$.
+/// Adds two numbers modulo a third number $m$. The inputs must be already reduced modulo $m$.
 pub trait ModAdd<RHS = Self, M = Self> {
     type Output;
 
     fn mod_add(self, other: RHS, m: M) -> Self::Output;
 }
 
-/// Adds two numbers modulo a third number $m$, in place. Assumes the inputs are already reduced
-/// modulo $m$.
+/// Adds two numbers modulo a third number $m$, in place. The inputs must be already reduced modulo
+/// $m$.
 pub trait ModAddAssign<RHS = Self, M = Self> {
     fn mod_add_assign(&mut self, other: RHS, m: M);
 }
 
-/// Finds the multiplicative inverse of a number modulo another number $m$. Assumes the input is
+/// Finds the multiplicative inverse of a number modulo another number $m$. The input must be
 /// already reduced modulo $m$.
 pub trait ModInverse<M = Self> {
     type Output;
@@ -506,22 +550,20 @@ pub trait ModIsReduced<M = Self> {
     fn mod_is_reduced(&self, m: &M) -> bool;
 }
 
-/// Multiplies two numbers modulo a third number $m$. Assumes the inputs are already reduced mod
-/// $m$.
+/// Multiplies two numbers modulo a third number $m$. The inputs must be already reduced modulo $m$.
 pub trait ModMul<RHS = Self, M = Self> {
     type Output;
 
     fn mod_mul(self, other: RHS, m: M) -> Self::Output;
 }
 
-/// Multiplies two numbers modulo a third number $m$, in place. Assumes the inputs are already
-/// reduced modulo $m$.
+/// Multiplies two numbers modulo a third number $m$, in place. The inputs must be already reduced
+/// modulo $m$.
 pub trait ModMulAssign<RHS = Self, M = Self> {
     fn mod_mul_assign(&mut self, other: RHS, m: M);
 }
 
-/// Multiplies two numbers modulo a third number $m$. Assumes the inputs are already reduced mod
-/// $m$.
+/// Multiplies two numbers modulo a third number $m$. The inputs must be already reduced modulo $m$.
 ///
 /// If multiple modular multiplications with the same modulus are necessary, it can be quicker to
 /// precompute some piece of data and reuse it in the multiplication calls. This trait provides a
@@ -536,8 +578,8 @@ pub trait ModMulPrecomputed<RHS = Self, M = Self> {
     fn mod_mul_precomputed(self, other: RHS, m: M, data: &Self::Data) -> Self::Output;
 }
 
-/// Multiplies two numbers modulo a third number $m$, in place. Assumes the inputs are already
-/// reduced modulo $m$.
+/// Multiplies two numbers modulo a third number $m$, in place.The inputs must be already reduced
+/// modulo $m$.
 ///
 /// If multiple modular multiplications with the same modulus are necessary, it can be quicker to
 /// precompute some piece of data and reuse it in the multiplication calls. This trait provides a
@@ -548,15 +590,15 @@ pub trait ModMulPrecomputedAssign<RHS = Self, M = Self>: ModMulPrecomputed<RHS,
     fn mod_mul_precomputed_assign(&mut self, other: RHS, m: M, data: &Self::Data);
 }
 
-/// Negates a number modulo another number $m$. Assumes the input is already reduced modulo $m$.
+/// Negates a number modulo another number $m$. The input must be already reduced modulo $m$.
 pub trait ModNeg<M = Self> {
     type Output;
 
     fn mod_neg(self, m: M) -> Self::Output;
 }
 
-/// Negates a number modulo another number $m$, in place. Assumes the input is already reduced
-/// modulo $m$.
+/// Negates a number modulo another number $m$, in place. The input must be already reduced modulo
+/// $m$.
 pub trait ModNegAssign<M = Self> {
     fn mod_neg_assign(&mut self, m: M);
 }
@@ -564,8 +606,8 @@ pub trait ModNegAssign<M = Self> {
 /// Divides a number by another number, returning just the remainder. The remainder has the same
 /// sign as the divisor (second number).
 ///
-/// If the quotient were computed, the quotient and remainder would satisfy $x = qy + r$ and
-/// $0 \leq |r| < |y|$.
+/// If the quotient were computed, the quotient and remainder would satisfy $x = qy + r$ and $0 \leq
+/// |r| < |y|$.
 pub trait Mod<RHS = Self> {
     type Output;
 
@@ -575,36 +617,35 @@ pub trait Mod<RHS = Self> {
 /// Divides a number by another number, replacing the first number by the remainder. The remainder
 /// has the same sign as the divisor (second number).
 ///
-/// If the quotient were computed, the quotient and remainder would satisfy $x = qy + r$ and
-/// $0 \leq |r| < |y|$.
+/// If the quotient were computed, the quotient and remainder would satisfy $x = qy + r$ and $0 \leq
+/// |r| < |y|$.
 pub trait ModAssign<RHS = Self> {
     fn mod_assign(&mut self, other: RHS);
 }
 
 /// Divides the negative of a number by another number, returning the remainder.
 ///
-/// If the quotient were computed, the quotient and remainder would satisfy $x = qy - r$ and
-/// $0 \leq r < y$.
+/// If the quotient were computed, the quotient and remainder would satisfy $x = qy - r$ and $0 \leq
+/// r < y$.
 pub trait NegMod<RHS = Self> {
     type Output;
 
     fn neg_mod(self, other: RHS) -> Self::Output;
 }
 
-/// Divides the negative of a number by another number, replacing the first number by the
-/// remainder.
+/// Divides the negative of a number by another number, replacing the first number by the remainder.
 ///
-/// If the quotient were computed, the quotient and remainder would satisfy $x = qy - r$ and
-/// $0 \leq r < y$.
+/// If the quotient were computed, the quotient and remainder would satisfy $x = qy - r$ and $0 \leq
+/// r < y$.
 pub trait NegModAssign<RHS = Self> {
     fn neg_mod_assign(&mut self, other: RHS);
 }
 
-/// Divides a number by another number, returning just the remainder. The remainder has the
-/// opposite sign as the divisor (second number).
+/// Divides a number by another number, returning just the remainder. The remainder has the opposite
+/// sign as the divisor (second number).
 ///
-/// If the quotient were computed, the quotient and remainder would satisfy $x = qy + r$ and
-/// $0 \leq |r| < |y|$.
+/// If the quotient were computed, the quotient and remainder would satisfy $x = qy + r$ and $0 \leq
+/// |r| < |y|$.
 pub trait CeilingMod<RHS = Self> {
     type Output;
 
@@ -614,28 +655,28 @@ pub trait CeilingMod<RHS = Self> {
 /// Divides a number by another number, replacing the first number by the remainder. The remainder
 /// has the same sign as the divisor (second number).
 ///
-/// If the quotient were computed, the quotient and remainder would satisfy $x = qy + r$ and
-/// $0 \leq |r| < |y|$.
+/// If the quotient were computed, the quotient and remainder would satisfy $x = qy + r$ and $0 \leq
+/// |r| < |y|$.
 pub trait CeilingModAssign<RHS = Self> {
     fn ceiling_mod_assign(&mut self, other: RHS);
 }
 
-/// Raises a number to a power modulo another number $m$. Assumes the input is already reduced
-/// modulo $m$.
+/// Raises a number to a power modulo another number $m$. The base must be already reduced modulo
+/// $m$.
 pub trait ModPow<RHS = Self, M = Self> {
     type Output;
 
     fn mod_pow(self, exp: RHS, m: M) -> Self::Output;
 }
 
-/// Raises a number to a power modulo another number $m$, in place. Assumes the input is already
-/// reduced modulo $m$.
+/// Raises a number to a power modulo another number $m$, in place. The base must be already reduced
+/// modulo $m$.
 pub trait ModPowAssign<RHS = Self, M = Self> {
     fn mod_pow_assign(&mut self, exp: RHS, m: M);
 }
 
-/// Raises a number to a power modulo another number $m$. Assumes the input is already reduced
-/// modulo $m$.
+/// Raises a number to a power modulo another number $m$. The base must be already reduced modulo
+/// $m$.
 ///
 /// If multiple modular exponentiations with the same modulus are necessary, it can be quicker to
 /// precompute some piece of data and reuse it in the exponentiation calls. This trait provides a
@@ -653,8 +694,8 @@ where
     fn mod_pow_precomputed(self, exp: RHS, m: M, data: &Self::Data) -> Self::Output;
 }
 
-/// Raises a number to a power modulo another number $m$, in place. Assumes the input is already
-/// reduced modulo $m$.
+/// Raises a number to a power modulo another number $m$, in place. The base must be already reduced
+/// modulo $m$.
 ///
 /// If multiple modular exponentiations with the same modulus are necessary, it can be quicker to
 /// precompute some piece of data and reuse it in the exponentiation calls. This trait provides a
@@ -665,19 +706,19 @@ pub trait ModPowPrecomputedAssign<RHS: Two = Self, M = Self>: ModPowPrecomputed<
     fn mod_pow_precomputed_assign(&mut self, exp: RHS, m: M, data: &Self::Data);
 }
 
-/// Adds two numbers modulo $2^k$. Assumes the inputs are already reduced modulo $2^k$.
+/// Adds two numbers modulo $2^k$. The inputs must be already reduced modulo $2^k$.
 pub trait ModPowerOf2Add<RHS = Self> {
     type Output;
 
     fn mod_power_of_2_add(self, other: RHS, pow: u64) -> Self::Output;
 }
 
-/// Adds two numbers modulo $2^k$, in place. Assumes the inputs are already reduced modulo $2^k$.
+/// Adds two numbers modulo $2^k$, in place. The inputs must be already reduced modulo $2^k$.
 pub trait ModPowerOf2AddAssign<RHS = Self> {
     fn mod_power_of_2_add_assign(&mut self, other: RHS, pow: u64);
 }
 
-/// Finds the multiplicative inverse of a number modulo $2^k$. Assumes the input is already reduced
+/// Finds the multiplicative inverse of a number modulo $2^k$. The input must be already reduced
 /// modulo $2^k$.
 pub trait ModPowerOf2Inverse {
     type Output;
@@ -690,45 +731,44 @@ pub trait ModPowerOf2IsReduced {
     fn mod_power_of_2_is_reduced(&self, pow: u64) -> bool;
 }
 
-/// Multiplies two numbers modulo $2^k$. Assumes the inputs are already reduced modulo $2^k$.
+/// Multiplies two numbers modulo $2^k$. The inputs must be already reduced modulo $2^k$.
 pub trait ModPowerOf2Mul<RHS = Self> {
     type Output;
 
     fn mod_power_of_2_mul(self, other: RHS, pow: u64) -> Self::Output;
 }
 
-/// Multiplies two numbers modulo $2^k$, in place. Assumes the inputs are already reduced modulo
-/// $2^k$.
+/// Multiplies two numbers modulo $2^k$, in place. The inputs must be already reduced modulo $2^k$.
 pub trait ModPowerOf2MulAssign<RHS = Self> {
     fn mod_power_of_2_mul_assign(&mut self, other: RHS, pow: u64);
 }
 
-/// Negates a number modulo $2^k$. Assumes the input is already reduced modulo $2^k$.
+/// Negates a number modulo $2^k$. The input must be already reduced modulo $2^k$.
 pub trait ModPowerOf2Neg {
     type Output;
 
     fn mod_power_of_2_neg(self, pow: u64) -> Self::Output;
 }
 
-/// Negates a number modulo $2^k$ in place. Assumes the input is already reduced modulo $2^k$.
+/// Negates a number modulo $2^k$ in place. The input must be already reduced modulo $2^k$.
 pub trait ModPowerOf2NegAssign {
     fn mod_power_of_2_neg_assign(&mut self, pow: u64);
 }
 
-/// Raises a number to a power modulo $2^k$. Assumes the input is already reduced modulo $2^k$.
+/// Raises a number to a power modulo $2^k$. The base must be already reduced modulo $2^k$.
 pub trait ModPowerOf2Pow<RHS = Self> {
     type Output;
 
     fn mod_power_of_2_pow(self, exp: RHS, pow: u64) -> Self::Output;
 }
 
-/// Raises a number to a power modulo $2^k$, in place. Assumes the input is already reduced modulo
+/// Raises a number to a power modulo $2^k$, in place. The base must be already reduced modulo
 /// $2^k$.
 pub trait ModPowerOf2PowAssign<RHS = Self> {
     fn mod_power_of_2_pow_assign(&mut self, exp: RHS, pow: u64);
 }
 
-/// Left-shifts a number (multiplies it by a power of 2) modulo $2^k$. Assumes the input is already
+/// Left-shifts a number (multiplies it by a power of 2) modulo $2^k$. The number must be already
 /// reduced modulo $2^k$.
 pub trait ModPowerOf2Shl<RHS> {
     type Output;
@@ -736,13 +776,13 @@ pub trait ModPowerOf2Shl<RHS> {
     fn mod_power_of_2_shl(self, other: RHS, pow: u64) -> Self::Output;
 }
 
-/// Left-shifts a number (multiplies it by a power of 2) modulo $2^k$, in place. Assumes the input
-/// is already reduced modulo $2^k$.
+/// Left-shifts a number (multiplies it by a power of 2) modulo $2^k$, in place. The number must be
+/// already reduced modulo $2^k$.
 pub trait ModPowerOf2ShlAssign<RHS> {
     fn mod_power_of_2_shl_assign(&mut self, other: RHS, pow: u64);
 }
 
-/// Right-shifts a number (divides it by a power of 2) modulo $2^k$. Assumes the input is already
+/// Right-shifts a number (divides it by a power of 2) modulo $2^k$. The number must be already
 /// reduced modulo $2^k$.
 pub trait ModPowerOf2Shr<RHS> {
     type Output;
@@ -750,52 +790,50 @@ pub trait ModPowerOf2Shr<RHS> {
     fn mod_power_of_2_shr(self, other: RHS, pow: u64) -> Self::Output;
 }
 
-/// Right-shifts a number (divides it by a power of 2) modulo $2^k$, in place. Assumes the input is
+/// Right-shifts a number (divides it by a power of 2) modulo $2^k$, in place. The number must be
 /// already reduced modulo $2^k$.
 pub trait ModPowerOf2ShrAssign<RHS> {
     fn mod_power_of_2_shr_assign(&mut self, other: RHS, pow: u64);
 }
 
-/// Squares a number modulo $2^k$. Assumes the input is already reduced modulo $2^k$.
+/// Squares a number modulo $2^k$. The input must be already reduced modulo $2^k$.
 pub trait ModPowerOf2Square {
     type Output;
 
     fn mod_power_of_2_square(self, pow: u64) -> Self::Output;
 }
 
-/// Squares a number modulo $2^k$ in place. Assumes the input is already reduced modulo $2^k$.
+/// Squares a number modulo $2^k$ in place. The input must be already reduced modulo $2^k$.
 pub trait ModPowerOf2SquareAssign {
     fn mod_power_of_2_square_assign(&mut self, pow: u64);
 }
 
-/// Subtracts two numbers modulo $2^k$. Assumes the inputs are already reduced modulo $2^k$.
+/// Subtracts two numbers modulo $2^k$. The inputs must be already reduced modulo $2^k$.
 pub trait ModPowerOf2Sub<RHS = Self> {
     type Output;
 
     fn mod_power_of_2_sub(self, other: RHS, pow: u64) -> Self::Output;
 }
 
-/// Subtracts two numbers modulo $2^k$, in place. Assumes the inputs are already reduced modulo
-/// $2^k$.
+/// Subtracts two numbers modulo $2^k$, in place. The inputs must be already reduced modulo $2^k$.
 pub trait ModPowerOf2SubAssign<RHS = Self> {
     fn mod_power_of_2_sub_assign(&mut self, other: RHS, pow: u64);
 }
 
 /// Divides a number by $2^k$, returning just the remainder. The remainder is non-negative.
 ///
-/// If the quotient were computed, the quotient and remainder would satisfy $x = q2^k + r$ and
-/// $0 \leq r < 2^k$.
+/// If the quotient were computed, the quotient and remainder would satisfy $x = q2^k + r$ and $0
+/// \leq r < 2^k$.
 pub trait ModPowerOf2 {
     type Output;
 
     fn mod_power_of_2(self, other: u64) -> Self::Output;
 }
 
-/// Divides a number by $2^k$, replacing the number by the remainder. The remainder is
-/// non-negative.
+/// Divides a number by $2^k$, replacing the number by the remainder. The remainder is non-negative.
 ///
-/// If the quotient were computed, the quotient and remainder would satisfy $x = q2^k + r$ and
-/// $0 \leq r < 2^k$.
+/// If the quotient were computed, the quotient and remainder would satisfy $x = q2^k + r$ and $0
+/// \leq r < 2^k$.
 pub trait ModPowerOf2Assign {
     fn mod_power_of_2_assign(&mut self, other: u64);
 }
@@ -803,8 +841,8 @@ pub trait ModPowerOf2Assign {
 /// Divides a number by $2^k$, returning just the remainder. The remainder has the same sign as the
 /// number.
 ///
-/// If the quotient were computed, the quotient and remainder would satisfy $x = q2^k + r$ and
-/// $0 \leq |r| < 2^k$.
+/// If the quotient were computed, the quotient and remainder would satisfy $x = q2^k + r$ and $0
+/// \leq |r| < 2^k$.
 pub trait RemPowerOf2 {
     type Output;
 
@@ -814,16 +852,16 @@ pub trait RemPowerOf2 {
 /// Divides a number by $2^k$, replacing the number by the remainder. The remainder has the same
 /// sign as the number.
 ///
-/// If the quotient were computed, the quotient and remainder would satisfy $x = q2^k + r$ and
-/// $0 \leq |r| < 2^k$.
+/// If the quotient were computed, the quotient and remainder would satisfy $x = q2^k + r$ and $0
+/// \leq |r| < 2^k$.
 pub trait RemPowerOf2Assign {
     fn rem_power_of_2_assign(&mut self, other: u64);
 }
 
 /// Divides the negative of a number by $2^k$, returning the remainder.
 ///
-/// If the quotient were computed, the quotient and remainder would satisfy $x = q2^k - r$ and
-/// $0 \leq r < 2^k$.
+/// If the quotient were computed, the quotient and remainder would satisfy $x = q2^k - r$ and $0
+/// \leq r < 2^k$.
 pub trait NegModPowerOf2 {
     type Output;
 
@@ -832,73 +870,72 @@ pub trait NegModPowerOf2 {
 
 /// Divides the negative of a number by $2^k$, replacing the number by the remainder.
 ///
-/// If the quotient were computed, the quotient and remainder would satisfy $x = q2^k - r$ and
-/// $0 \leq r < 2^k$.
+/// If the quotient were computed, the quotient and remainder would satisfy $x = q2^k - r$ and $0
+/// \leq r < 2^k$.
 pub trait NegModPowerOf2Assign {
     fn neg_mod_power_of_2_assign(&mut self, other: u64);
 }
 
 /// Divides a number by $2^k$, returning just the remainder. The remainder is non-positive.
 ///
-/// If the quotient were computed, the quotient and remainder would satisfy $x = q2^k + r$ and
-/// $0 \leq -r < 2^k$.
+/// If the quotient were computed, the quotient and remainder would satisfy $x = q2^k + r$ and $0
+/// \leq -r < 2^k$.
 pub trait CeilingModPowerOf2 {
     type Output;
 
     fn ceiling_mod_power_of_2(self, other: u64) -> Self::Output;
 }
 
-/// Divides a number by $2^k$, replacing the number by the remainder. The remainder is
-/// non-positive.
+/// Divides a number by $2^k$, replacing the number by the remainder. The remainder is non-positive.
 ///
-/// If the quotient were computed, the quotient and remainder would satisfy $x = q2^k + r$ and
-/// $0 \leq -r < 2^k$.
+/// If the quotient were computed, the quotient and remainder would satisfy $x = q2^k + r$ and $0
+/// \leq -r < 2^k$.
 pub trait CeilingModPowerOf2Assign {
     fn ceiling_mod_power_of_2_assign(&mut self, other: u64);
 }
 
-/// Left-shifts a number (multiplies it by a power of 2) modulo another number $m$. Assumes the
-/// input is already reduced modulo $m$.
+/// Left-shifts a number (multiplies it by a power of 2) modulo another number $m$. The number must
+/// be already reduced modulo $m$.
 pub trait ModShl<RHS, M = Self> {
     type Output;
 
     fn mod_shl(self, other: RHS, m: M) -> Self::Output;
 }
 
-/// Left-shifts a number (multiplies it by a power of 2) modulo another number $m$, in place.
-/// Assumes the input is already reduced modulo $m$.
+/// Left-shifts a number (multiplies it by a power of 2) modulo another number $m$, in place. The
+/// number must be already reduced modulo $m$.
 pub trait ModShlAssign<RHS, M = Self> {
     fn mod_shl_assign(&mut self, other: RHS, m: M);
 }
 
-/// Left-shifts a number (divides it by a power of 2) modulo another number $m$. Assumes the input
-/// is already reduced modulo $m$.
+/// Left-shifts a number (divides it by a power of 2) modulo another number $m$. The number must be
+/// already reduced modulo $m$.
 pub trait ModShr<RHS, M = Self> {
     type Output;
 
     fn mod_shr(self, other: RHS, m: M) -> Self::Output;
 }
 
-/// Left-shifts a number (divides it by a power of 2) modulo another number $m$, in place. Assumes
-/// the input is already reduced modulo $m$.
+/// Left-shifts a number (divides it by a power of 2) modulo another number $m$, in place. The
+/// number must be already reduced modulo $m$.
 pub trait ModShrAssign<RHS, M = Self> {
     fn mod_shr_assign(&mut self, other: RHS, m: M);
 }
 
-/// Squares a number modulo another number $m$. Assumes the input is already reduced modulo $m$.
+/// Squares a number modulo another number $m$. The input must be already reduced modulo $m$.
 pub trait ModSquare<M = Self> {
     type Output;
 
     fn mod_square(self, m: M) -> Self::Output;
 }
 
-/// Squares a number modulo another number $m$, in place. Assumes the input is already reduced
-/// modulo $m$.
+/// Squares a number modulo another number $m$, in place. The input must be already reduced modulo
+/// $m$.
 pub trait ModSquareAssign<M = Self> {
     fn mod_square_assign(&mut self, m: M);
 }
 
-/// Squares a number modulo another number $m$. Assumes the input is already reduced modulo $m$.
+/// Squares a number modulo another number $m$. The input must be already reduced modulo $m$.
 ///
 /// If multiple modular squarings with the same modulus are necessary, it can be quicker to
 /// precompute some piece of data using
@@ -911,8 +948,8 @@ where
     fn mod_square_precomputed(self, m: M, data: &Self::Data) -> Self::Output;
 }
 
-/// Squares a number modulo another number $m$, in place. Assumes the input is already reduced
-/// modulo $m$.
+/// Squares a number modulo another number $m$, in place. The input must be already reduced modulo
+/// $m$.
 ///
 /// If multiple modular squarings with the same modulus are necessary, it can be quicker to
 /// precompute some piece of data using
@@ -922,15 +959,15 @@ pub trait ModSquarePrecomputedAssign<RHS = Self, M = Self>: ModPowPrecomputed<RH
     fn mod_square_precomputed_assign(&mut self, m: M, data: &Self::Data);
 }
 
-/// Adds two numbers modulo a third number $m$. Assumes the inputs are already reduced modulo $m$.
+/// Adds two numbers modulo a third number $m$. The inputs must be already reduced modulo $m$.
 pub trait ModSub<RHS = Self, M = Self> {
     type Output;
 
     fn mod_sub(self, other: RHS, m: M) -> Self::Output;
 }
 
-/// Adds two numbers modulo a third number $m$, in place. Assumes the inputs are already reduced
-/// modulo $m$.
+/// Adds two numbers modulo a third number $m$, in place. The inputs must be already reduced modulo
+/// $m$.
 pub trait ModSubAssign<RHS = Self, M = Self> {
     fn mod_sub_assign(&mut self, other: RHS, m: M);
 }
@@ -1264,8 +1301,8 @@ pub trait RoundToMultiple<RHS = Self> {
 }
 
 /// Rounds a number to a multiple of another number in place, according to a specified rounding
-/// mode. [`Ordering`] is returned, indicating whether the returned value is less than, equal to,
-/// or greater than the original value.
+/// mode. [`Ordering`] is returned, indicating whether the returned value is less than, equal to, or
+/// greater than the original value.
 pub trait RoundToMultipleAssign<RHS = Self> {
     fn round_to_multiple_assign(&mut self, other: RHS, rm: RoundingMode) -> Ordering;
 }
@@ -1392,8 +1429,8 @@ pub trait SaturatingSubAssign<RHS = Self> {
     fn saturating_sub_assign(&mut self, other: RHS);
 }
 
-/// Subtracts a number by the product of two other numbers, saturating at the numeric bounds
-/// instead of overflowing.
+/// Subtracts a number by the product of two other numbers, saturating at the numeric bounds instead
+/// of overflowing.
 pub trait SaturatingSubMul<Y = Self, Z = Self> {
     type Output;
 
@@ -1407,8 +1444,8 @@ pub trait SaturatingSubMulAssign<Y = Self, Z = Self> {
 }
 
 /// Left-shifts a number (multiplies it by a power of 2), rounding the result according to a
-/// specified rounding mode. An [`Ordering`] is also returned, indicating whether the returned
-/// value is less than, equal to, or greater than the exact value.
+/// specified rounding mode. An [`Ordering`] is also returned, indicating whether the returned value
+/// is less than, equal to, or greater than the exact value.
 ///
 /// Rounding might only be necessary if `other` is negative.
 pub trait ShlRound<RHS> {
@@ -1426,9 +1463,9 @@ pub trait ShlRoundAssign<RHS> {
     fn shl_round_assign(&mut self, other: RHS, rm: RoundingMode) -> Ordering;
 }
 
-/// Right-shifts a number (divides it by a power of 2), rounding the result according to a
-/// specified rounding mode. An [`Ordering`] is also returned, indicating whether the returned
-/// value is less than, equal to, or greater than the exact value.
+/// Right-shifts a number (divides it by a power of 2), rounding the result according to a specified
+/// rounding mode. An [`Ordering`] is also returned, indicating whether the returned value is less
+/// than, equal to, or greater than the exact value.
 ///
 /// Rounding might only be necessary if `other` is positive.
 pub trait ShrRound<RHS> {
@@ -1438,8 +1475,8 @@ pub trait ShrRound<RHS> {
 }
 
 /// Right-shifts a number (divides it by a power of 2) in place, rounding the result according to a
-/// specified rounding mode. An [`Ordering`] is also returned, indicating whether the assigned
-/// value is less than, equal to, or greater than the exact value.
+/// specified rounding mode. An [`Ordering`] is also returned, indicating whether the assigned value
+/// is less than, equal to, or greater than the exact value.
 ///
 /// Rounding might only be necessary if `other` is positive.
 pub trait ShrRoundAssign<RHS> {
@@ -1558,16 +1595,15 @@ pub trait WrappingAddAssign<RHS = Self> {
     fn wrapping_add_assign(&mut self, other: RHS);
 }
 
-/// Adds a number and the product of two other numbers, wrapping around at the boundary of the
-/// type.
+/// Adds a number and the product of two other numbers, wrapping around at the boundary of the type.
 pub trait WrappingAddMul<Y = Self, Z = Self> {
     type Output;
 
     fn wrapping_add_mul(self, y: Y, z: Z) -> Self::Output;
 }
 
-/// Adds a number and the product of two other numbers, in place, wrapping around at the boundary
-/// of the type.
+/// Adds a number and the product of two other numbers, in place, wrapping around at the boundary of
+/// the type.
 pub trait WrappingAddMulAssign<Y = Self, Z = Self> {
     fn wrapping_add_mul_assign(&mut self, y: Y, z: Z);
 }
@@ -1707,8 +1743,8 @@ pub trait XXXAddYYYToZZZ: Sized {
     ) -> (Self, Self, Self);
 }
 
-/// Subtracts two numbers, each composed of three `Self` values, returing the difference as a
-/// triple of `Self` values.
+/// Subtracts two numbers, each composed of three `Self` values, returing the difference as a triple
+/// of `Self` values.
 ///
 /// The more significant number always comes first. Subtraction is wrapping, and overflow is not
 /// indicated.
