@@ -1,13 +1,23 @@
+// Copyright © 2024 Mikhail Hogrefe
+//
+// This file is part of Malachite.
+//
+// Malachite is free software: you can redistribute it and/or modify it under the terms of the GNU
+// Lesser General Public License (LGPL) as published by the Free Software Foundation; either version
+// 3 of the License, or (at your option) any later version. See <https://www.gnu.org/licenses/>.
+
 use crate::comparison::traits::{Max, Min};
 use crate::named::Named;
 use crate::num::arithmetic::traits::{
     Abs, AbsAssign, AddMul, AddMulAssign, Ceiling, CeilingAssign, CeilingLogBase2,
     CeilingLogBasePowerOf2, CheckedLogBase2, CheckedLogBasePowerOf2, Floor, FloorAssign,
-    FloorLogBase2, FloorLogBasePowerOf2, IsPowerOf2, NegAssign, NextPowerOf2, NextPowerOf2Assign,
-    Pow, PowAssign, PowerOf2, Sign, Sqrt, SqrtAssign, Square, SquareAssign, SubMul, SubMulAssign,
+    FloorLogBase2, FloorLogBasePowerOf2, IsPowerOf2, Ln, NegAssign, NextPowerOf2,
+    NextPowerOf2Assign, Pow, PowAssign, PowerOf2, Reciprocal, ReciprocalAssign, Sign, Sqrt,
+    SqrtAssign, Square, SquareAssign, SubMul, SubMulAssign,
 };
 use crate::num::basic::traits::{
-    Infinity, NaN, NegativeInfinity, NegativeOne, NegativeZero, One, OneHalf, Two, Zero,
+    Infinity, NaN, NegativeInfinity, NegativeOne, NegativeZero, One, OneHalf, PrimeConstant,
+    ThueMorseConstant, Two, Zero,
 };
 use crate::num::comparison::traits::PartialOrdAbs;
 use crate::num::conversion::traits::{
@@ -16,14 +26,15 @@ use crate::num::conversion::traits::{
 };
 use crate::num::float::FmtRyuString;
 use crate::num::logic::traits::{BitAccess, LowMask, SignificantBits, TrailingZeros};
-use std::cmp::Ordering;
-use std::fmt::{Debug, Display, LowerExp, UpperExp};
-use std::iter::{Product, Sum};
-use std::num::FpCategory;
-use std::ops::{
+use core::cmp::Ordering::*;
+use core::fmt::{Debug, Display, LowerExp, UpperExp};
+use core::iter::{Product, Sum};
+use core::num::FpCategory;
+use core::ops::{
     Add, AddAssign, Div, DivAssign, Mul, MulAssign, Neg, Rem, RemAssign, Sub, SubAssign,
 };
-use std::str::FromStr;
+use core::panic::RefUnwindSafe;
+use core::str::FromStr;
 
 /// This trait defines functions on primitive float types: [`f32`] and [`f64`].
 ///
@@ -39,8 +50,8 @@ use std::str::FromStr;
 /// # raw form
 /// The raw exponent and raw mantissa are the actual bit patterns used to represent the components
 /// of $x$. The raw exponent $e_r$ is an integer in $[0, 2^E-2]$ and the raw mantissa $m_r$ is an
-/// integer in $[0, 2^M-1]$. Since we are dealing with a nonzero $x$, we forbid $e_r$ and $m_r$
-/// from both being zero. We have
+/// integer in $[0, 2^M-1]$. Since we are dealing with a nonzero $x$, we forbid $e_r$ and $m_r$ from
+/// both being zero. We have
 /// $$
 /// x = \\begin{cases}
 ///     2^{2-2^{E-1}-M}m_r & \text{if} \quad e_r = 0, \\\\
@@ -61,8 +72,8 @@ use std::str::FromStr;
 /// $$
 ///
 /// # scientific form
-/// We can write $x = 2^{e_s}m_s$, where $e_s$ is an integer and $m_s$ is a rational number with
-/// $1 \leq m_s < 2$. If $x$ is a valid float, the scientific mantissa $m_s$ is always exactly
+/// We can write $x = 2^{e_s}m_s$, where $e_s$ is an integer and $m_s$ is a rational number with $1
+/// \leq m_s < 2$. If $x$ is a valid float, the scientific mantissa $m_s$ is always exactly
 /// representable as a float of the same type. We have
 /// $$
 /// x = 2^{e_s}m_s,
@@ -128,6 +139,7 @@ pub trait PrimitiveFloat:
     + Into<f64>
     + IsInteger
     + IsPowerOf2
+    + Ln
     + LowerExp
     + Min
     + Max
@@ -151,8 +163,12 @@ pub trait PrimitiveFloat:
     + PowAssign<i64>
     + PowAssign<Self>
     + PowerOf2<i64>
+    + PrimeConstant
     + Product
     + RawMantissaAndExponent<u64, u64>
+    + Reciprocal<Output = Self>
+    + ReciprocalAssign
+    + RefUnwindSafe
     + Rem<Output = Self>
     + RemAssign<Self>
     + RoundingFrom<u8>
@@ -191,6 +207,7 @@ pub trait PrimitiveFloat:
     + SubMul<Output = Self>
     + SubMulAssign<Self, Self>
     + Sum<Self>
+    + ThueMorseConstant
     + Two
     + UpperExp
     + Zero
@@ -285,7 +302,7 @@ pub trait PrimitiveFloat:
     /// ```
     #[inline]
     fn is_negative_zero(self) -> bool {
-        self.sign() == Ordering::Less && self == Self::ZERO
+        self.sign() == Less && self == Self::ZERO
     }
 
     /// If `self` is negative zero, returns positive zero; otherwise, returns `self`.
@@ -373,7 +390,7 @@ pub trait PrimitiveFloat:
     /// ```
     fn next_higher(self) -> Self {
         assert!(!self.is_nan());
-        if self.sign() == Ordering::Greater {
+        if self.sign() == Greater {
             assert_ne!(self, Self::INFINITY);
             Self::from_bits(self.to_bits() + 1)
         } else if self == Self::ZERO {
@@ -406,7 +423,7 @@ pub trait PrimitiveFloat:
     /// ```
     fn next_lower(self) -> Self {
         assert!(!self.is_nan());
-        if self.sign() == Ordering::Less {
+        if self.sign() == Less {
             assert_ne!(self, Self::NEGATIVE_INFINITY);
             Self::from_bits(self.to_bits() + 1)
         } else if self == Self::ZERO {
@@ -421,8 +438,8 @@ pub trait PrimitiveFloat:
     /// adjacent integers.
     ///
     /// Negative infinity is mapped to 0, and positive infinity is mapped to the largest value,
-    /// [`LARGEST_ORDERED_REPRESENTATION`](PrimitiveFloat::LARGEST_ORDERED_REPRESENTATION).
-    /// Negative and positive zero are mapped to distinct adjacent values. Passing in `NaN` panics.
+    /// [`LARGEST_ORDERED_REPRESENTATION`](PrimitiveFloat::LARGEST_ORDERED_REPRESENTATION). Negative
+    /// and positive zero are mapped to distinct adjacent values. Passing in `NaN` panics.
     ///
     /// The inverse operation is
     /// [`from_ordered_representation`](PrimitiveFloat::from_ordered_representation).
@@ -442,15 +459,12 @@ pub trait PrimitiveFloat:
     /// assert_eq!((-0.0f32).to_ordered_representation(), 2139095040);
     /// assert_eq!(0.0f32.to_ordered_representation(), 2139095041);
     /// assert_eq!(1.0f32.to_ordered_representation(), 3204448257);
-    /// assert_eq!(
-    ///     f32::INFINITY.to_ordered_representation(),
-    ///     4278190081
-    /// );
+    /// assert_eq!(f32::INFINITY.to_ordered_representation(), 4278190081);
     /// ```
     fn to_ordered_representation(self) -> u64 {
         assert!(!self.is_nan());
         let bits = self.to_bits();
-        if self.sign() == Ordering::Greater {
+        if self.sign() == Greater {
             (u64::low_mask(Self::EXPONENT_WIDTH) << Self::MANTISSA_WIDTH) + bits + 1
         } else {
             (u64::low_mask(Self::EXPONENT_WIDTH + 1) << Self::MANTISSA_WIDTH) - bits
@@ -462,9 +476,9 @@ pub trait PrimitiveFloat:
     /// float. The map preserves ordering, and adjacent integers are mapped to adjacent floats.
     ///
     /// Zero is mapped to negative infinity, and
-    /// [`LARGEST_ORDERED_REPRESENTATION`](PrimitiveFloat::LARGEST_ORDERED_REPRESENTATION) is
-    /// mapped to positive infinity. Negative and positive zero are produced by two distinct
-    /// adjacent integers. `NaN` is never produced.
+    /// [`LARGEST_ORDERED_REPRESENTATION`](PrimitiveFloat::LARGEST_ORDERED_REPRESENTATION) is mapped
+    /// to positive infinity. Negative and positive zero are produced by two distinct adjacent
+    /// integers. `NaN` is never produced.
     ///
     /// The inverse operation is
     /// [`to_ordered_representation`](PrimitiveFloat::to_ordered_representation).
@@ -485,10 +499,7 @@ pub trait PrimitiveFloat:
     /// assert_eq!(f32::from_ordered_representation(2139095040), -0.0f32);
     /// assert_eq!(f32::from_ordered_representation(2139095041), 0.0f32);
     /// assert_eq!(f32::from_ordered_representation(3204448257), 1.0f32);
-    /// assert_eq!(
-    ///     f32::from_ordered_representation(4278190081),
-    ///     f32::INFINITY
-    /// );
+    /// assert_eq!(f32::from_ordered_representation(4278190081), f32::INFINITY);
     /// ```
     fn from_ordered_representation(n: u64) -> Self {
         let zero_exp = u64::low_mask(Self::EXPONENT_WIDTH) << Self::MANTISSA_WIDTH;
@@ -496,7 +507,7 @@ pub trait PrimitiveFloat:
             Self::from_bits((u64::low_mask(Self::EXPONENT_WIDTH + 1) << Self::MANTISSA_WIDTH) - n)
         } else {
             let f = Self::from_bits(n - zero_exp - 1);
-            assert_eq!(f.sign(), Ordering::Greater);
+            assert_eq!(f.sign(), Greater);
             f
         };
         assert!(!f.is_nan());
@@ -505,9 +516,9 @@ pub trait PrimitiveFloat:
 
     /// Returns the precision of a nonzero finite floating-point number.
     ///
-    /// The precision is the number of significant bits of the integer mantissa. For example,
-    /// the floats with precision 1 are the powers of 2, those with precision 2 are 3 times a power
-    /// of 2, those with precision 3 are 5 or 7 times a power of 2, and so on.
+    /// The precision is the number of significant bits of the integer mantissa. For example, the
+    /// floats with precision 1 are the powers of 2, those with precision 2 are 3 times a power of
+    /// 2, those with precision 3 are 5 or 7 times a power of 2, and so on.
     ///
     /// # Worst-case complexity
     /// Constant time and additional memory.
@@ -552,7 +563,7 @@ pub trait PrimitiveFloat:
     /// Constant time and additional memory.
     ///
     /// # Panics
-    /// Panics if `self` is less than [`MIN_EXPONENT`](PrimitiveFloat::MIN_EXPONENT) or greater
+    /// Panics if `exponent` is less than [`MIN_EXPONENT`](PrimitiveFloat::MIN_EXPONENT) or greater
     /// than [`MAX_EXPONENT`](PrimitiveFloat::MAX_EXPONENT).
     ///
     /// # Examples
@@ -583,13 +594,15 @@ macro_rules! impl_basic_traits_primitive_float {
         $width: expr,
         $min_positive_subnormal: expr,
         $max_subnormal: expr,
-        $min_positive_normal: expr
+        $min_positive_normal: expr,
+        $thue_morse_constant: expr,
+        $prime_constant: expr
     ) => {
         impl PrimitiveFloat for $t {
             const WIDTH: u64 = $width;
-            const MANTISSA_WIDTH: u64 = (std::$t::MANTISSA_DIGITS as u64) - 1;
+            const MANTISSA_WIDTH: u64 = ($t::MANTISSA_DIGITS as u64) - 1;
 
-            const MAX_FINITE: Self = std::$t::MAX;
+            const MAX_FINITE: Self = $t::MAX;
             const MIN_POSITIVE_SUBNORMAL: Self = $min_positive_subnormal;
             const MAX_SUBNORMAL: Self = $max_subnormal;
             const MIN_POSITIVE_NORMAL: Self = $min_positive_normal;
@@ -679,17 +692,17 @@ macro_rules! impl_basic_traits_primitive_float {
 
         /// The constant Infinity for primitive floating-point types.
         impl Infinity for $t {
-            const INFINITY: $t = std::$t::INFINITY;
+            const INFINITY: $t = $t::INFINITY;
         }
 
         /// The constant -Infinity for primitive floating-point types.
         impl NegativeInfinity for $t {
-            const NEGATIVE_INFINITY: $t = std::$t::NEG_INFINITY;
+            const NEGATIVE_INFINITY: $t = $t::NEG_INFINITY;
         }
 
         /// The constant NaN for primitive floating-point types.
         impl NaN for $t {
-            const NAN: $t = std::$t::NAN;
+            const NAN: $t = $t::NAN;
         }
 
         /// The lowest value representable by this type, negative infinity.
@@ -701,13 +714,33 @@ macro_rules! impl_basic_traits_primitive_float {
         impl Max for $t {
             const MAX: $t = $t::INFINITY;
         }
+
+        /// The Thue-Morse constant.
+        impl ThueMorseConstant for $t {
+            const THUE_MORSE_CONSTANT: $t = $thue_morse_constant;
+        }
+
+        /// The prime constant.
+        impl PrimeConstant for $t {
+            const PRIME_CONSTANT: $t = $prime_constant;
+        }
     };
 }
-impl_basic_traits_primitive_float!(f32, 32, 1.0e-45, 1.1754942e-38, 1.1754944e-38);
+impl_basic_traits_primitive_float!(
+    f32,
+    32,
+    1.0e-45,
+    1.1754942e-38,
+    1.1754944e-38,
+    0.41245404,
+    0.4146825
+);
 impl_basic_traits_primitive_float!(
     f64,
     64,
     5.0e-324,
     2.225073858507201e-308,
-    2.2250738585072014e-308
+    2.2250738585072014e-308,
+    0.4124540336401076,
+    0.41468250985111166
 );
