@@ -1,3 +1,11 @@
+// Copyright © 2024 Mikhail Hogrefe
+//
+// This file is part of Malachite.
+//
+// Malachite is free software: you can redistribute it and/or modify it under the terms of the GNU
+// Lesser General Public License (LGPL) as published by the Free Software Foundation; either version
+// 3 of the License, or (at your option) any later version. See <https://www.gnu.org/licenses/>.
+
 use crate::num::arithmetic::traits::{ShrRoundAssign, UnsignedAbs};
 use crate::num::basic::floats::PrimitiveFloat;
 use crate::num::basic::integers::PrimitiveInt;
@@ -9,9 +17,9 @@ use crate::num::conversion::traits::{
     WrappingFrom,
 };
 use crate::num::float::NiceFloat;
-use crate::rounding_modes::RoundingMode;
-use std::cmp::Ordering;
-use std::ops::Neg;
+use crate::rounding_modes::RoundingMode::{self, *};
+use core::cmp::Ordering::{self, *};
+use core::ops::Neg;
 
 // This macro defines conversions from a type to itself.
 macro_rules! identity_conversion {
@@ -189,12 +197,11 @@ macro_rules! lossy_conversion {
             ///
             /// Let $W$ be the width of the target type.
             ///
-            /// If the target type is unsigned, then
-            /// $f_W(n) = m$, where $m < 2^W$ and $n + 2^W k = m$ for some $k \in \Z$.
+            /// If the target type is unsigned, then $f_W(n) = m$, where $m < 2^W$ and $n + 2^W k =
+            /// m$ for some $k \in \Z$.
             ///
-            /// If the target type is signed, then
-            /// $f_W(n) = m$, where $-2^{W-1} \leq m < 2^{W-1}$ and $n + 2^W k = m$ for some
-            /// $k \in \Z$.
+            /// If the target type is signed, then $f_W(n) = m$, where $-2^{W-1} \leq m < 2^{W-1}$
+            /// and $n + 2^W k = m$ for some $k \in \Z$.
             ///
             /// # Worst-case complexity
             /// Constant time and additional memory.
@@ -250,12 +257,11 @@ macro_rules! lossy_conversion {
             ///
             /// Let $W$ be the width of the target type.
             ///
-            /// If the target type is unsigned, then
-            /// $f_W(n) = (m, k \neq 0)$, where $m < 2^W$ and $n + 2^W k = m$ for some $k \in \Z$.
+            /// If the target type is unsigned, then $f_W(n) = (m, k \neq 0)$, where $m < 2^W$ and
+            /// $n + 2^W k = m$ for some $k \in \Z$.
             ///
-            /// If the target type is signed, then
-            /// $f_W(n) = (m, k \neq 0)$, where $-2^{W-1} \leq m < 2^{W-1}$ and $n + 2^W k = m$ for
-            /// some $k \in \Z$.
+            /// If the target type is signed, then $f_W(n) = (m, k \neq 0)$, where $-2^{W-1} \leq m
+            /// < 2^{W-1}$ and $n + 2^W k = m$ for some $k \in \Z$.
             ///
             /// # Worst-case complexity
             /// Constant time and additional memory.
@@ -389,20 +395,18 @@ fn primitive_float_rounding_from_unsigned<T: PrimitiveFloat, U: PrimitiveUnsigne
     rm: RoundingMode,
 ) -> (T, Ordering) {
     if value == U::ZERO {
-        return (T::ZERO, Ordering::Equal);
+        return (T::ZERO, Equal);
     }
     let (mantissa, exponent, o) = sci_mantissa_and_exponent_round(value, rm).unwrap();
     if let Some(f) = T::from_sci_mantissa_and_exponent(mantissa, i64::wrapping_from(exponent)) {
         (f, o)
     } else {
         match rm {
-            RoundingMode::Exact => {
+            Exact => {
                 panic!("Value cannot be represented exactly as an {}", T::NAME)
             }
-            RoundingMode::Floor | RoundingMode::Down | RoundingMode::Nearest => {
-                (T::MAX_FINITE, Ordering::Less)
-            }
-            _ => (T::INFINITY, Ordering::Greater),
+            Floor | Down | Nearest => (T::MAX_FINITE, Less),
+            _ => (T::INFINITY, Greater),
         }
     }
 }
@@ -415,37 +419,31 @@ fn unsigned_rounding_from_primitive_float<T: PrimitiveUnsigned, U: PrimitiveFloa
     if value.is_infinite() {
         return if value.is_sign_positive() {
             match rm {
-                RoundingMode::Exact => {
+                Exact => {
                     panic!("Value cannot be represented exactly as a {}", T::NAME)
                 }
-                RoundingMode::Down | RoundingMode::Floor | RoundingMode::Nearest => {
-                    (T::MAX, Ordering::Less)
-                }
+                Down | Floor | Nearest => (T::MAX, Less),
                 _ => panic!("Cannot round away from positive infinity"),
             }
         } else {
             match rm {
-                RoundingMode::Exact => {
+                Exact => {
                     panic!("Value cannot be represented exactly as a {}", T::NAME)
                 }
-                RoundingMode::Down | RoundingMode::Ceiling | RoundingMode::Nearest => {
-                    (T::ZERO, Ordering::Greater)
-                }
+                Down | Ceiling | Nearest => (T::ZERO, Greater),
                 _ => panic!("Cannot round away from negative infinity"),
             }
         };
     }
     if value == U::ZERO {
-        return (T::ZERO, Ordering::Equal);
+        return (T::ZERO, Equal);
     }
     if value.is_sign_negative() {
         return match rm {
-            RoundingMode::Exact => {
+            Exact => {
                 panic!("Value cannot be represented exactly as a {}", T::NAME)
             }
-            RoundingMode::Ceiling | RoundingMode::Down | RoundingMode::Nearest => {
-                (T::ZERO, Ordering::Greater)
-            }
+            Ceiling | Down | Nearest => (T::ZERO, Greater),
             _ => panic!("Value is less than 0 and rounding mode is {rm}"),
         };
     }
@@ -458,19 +456,17 @@ fn unsigned_rounding_from_primitive_float<T: PrimitiveUnsigned, U: PrimitiveFloa
             T::try_from(mantissa)
                 .ok()
                 .and_then(|n| n.arithmetic_checked_shl(exponent)),
-            Ordering::Equal,
+            Equal,
         )
     };
     if let Some(n) = result {
         (n, o)
     } else {
         match rm {
-            RoundingMode::Exact => {
+            Exact => {
                 panic!("Value cannot be represented exactly as a {}", T::NAME)
             }
-            RoundingMode::Floor | RoundingMode::Down | RoundingMode::Nearest => {
-                (T::MAX, Ordering::Less)
-            }
+            Floor | Down | Nearest => (T::MAX, Less),
             _ => panic!(
                 "Value is greater than {}::MAX and rounding mode is {}",
                 T::NAME,
@@ -489,8 +485,8 @@ fn primitive_float_try_from_unsigned<T: PrimitiveFloat, U: PrimitiveUnsigned>(
     if value == U::ZERO {
         return Ok(T::ZERO);
     }
-    let (mantissa, exponent, _) = sci_mantissa_and_exponent_round(value, RoundingMode::Exact)
-        .ok_or(PrimitiveFloatFromUnsignedError)?;
+    let (mantissa, exponent, _) =
+        sci_mantissa_and_exponent_round(value, Exact).ok_or(PrimitiveFloatFromUnsignedError)?;
     T::from_sci_mantissa_and_exponent(mantissa, i64::wrapping_from(exponent))
         .ok_or(PrimitiveFloatFromUnsignedError)
 }
@@ -556,17 +552,16 @@ macro_rules! impl_from_float_unsigned {
             ($f:ident) => {
                 impl RoundingFrom<$u> for $f {
                     /// Converts a value of an unsigned type to a value of a floating point type
-                    /// according to a specified
-                    /// [`RoundingMode`](crate::rounding_modes::RoundingMode). An [`Ordering`] is
-                    /// also returned, indicating whether the returned value is less than, equal
-                    /// to, or greater than the original value.
+                    /// according to a specified [`RoundingMode`]. An [`Ordering`] is also returned,
+                    /// indicating whether the returned value is less than, equal to, or greater
+                    /// than the original value.
                     ///
                     /// - If the rounding mode is `Floor` or `Down`, the largest float less than or
                     ///   equal to the value is returned.
-                    /// - If the rounding mode is `Ceiling` or `Up`, the smallest float greater
-                    ///   than or equal to the value is returned.
-                    /// - If the rounding mode is `Nearest`, then the nearest float is returned.
-                    ///   If the value is exactly between two floats, the float with the zero
+                    /// - If the rounding mode is `Ceiling` or `Up`, the smallest float greater than
+                    ///   or equal to the value is returned.
+                    /// - If the rounding mode is `Nearest`, then the nearest float is returned. If
+                    ///   the value is exactly between two floats, the float with the zero
                     ///   least-significant bit in its representation is selected. If the value is
                     ///   larger than the maximum finite float (which can only happen when
                     ///   converting a `u128` to an `f32`), the maximum finite float is returned.
@@ -588,10 +583,9 @@ macro_rules! impl_from_float_unsigned {
 
                 impl RoundingFrom<$f> for $u {
                     /// Converts a value of a floating point type to a value of an unsigned type
-                    /// according to a specified
-                    /// [`RoundingMode`](crate::rounding_modes::RoundingMode). An [`Ordering`] is
-                    /// also returned, indicating whether the returned value is less than, equal
-                    /// to, or greater than the original value.
+                    /// according to a specified [`RoundingMode`]. An [`Ordering`] is also returned,
+                    /// indicating whether the returned value is less than, equal to, or greater
+                    /// than the original value.
                     ///
                     /// - If the rounding mode is `Floor`, the largest number less than or equal to
                     ///   the value is returned. If the float is greater than the maximum
@@ -601,15 +595,14 @@ macro_rules! impl_from_float_unsigned {
                     ///   equal to the value is returned. If the float is negative, zero is
                     ///   returned. If the float is greater than the maximum representable unsigned
                     ///   value, the function panics.
-                    /// - If the rounding mode is `Down`, then the rounding proceeds as with
-                    ///   `Floor` if the float is non-negative and as with `Ceiling` if the value
-                    ///   is negative.
-                    /// - If the rounding mode is `Up`, then the rounding proceeds as with
-                    ///   `Ceiling` if the value is non-negative and as with `Floor` if the value
-                    ///   is negative.
+                    /// - If the rounding mode is `Down`, then the rounding proceeds as with `Floor`
+                    ///   if the float is non-negative and as with `Ceiling` if the value is
+                    ///   negative.
+                    /// - If the rounding mode is `Up`, then the rounding proceeds as with `Ceiling`
+                    ///   if the value is non-negative and as with `Floor` if the value is negative.
                     /// - If the rounding mode is `Nearest`, then the nearest value is returned. If
-                    ///   the value is exactly between two numbers, the even one is selected. If
-                    ///   the float is greater than the maximum representable unsigned value, the
+                    ///   the value is exactly between two numbers, the even one is selected. If the
+                    ///   float is greater than the maximum representable unsigned value, the
                     ///   maximum unsigned value is returned. If the float is negative, zero is
                     ///   returned.
                     ///
@@ -620,8 +613,8 @@ macro_rules! impl_from_float_unsigned {
                     /// - If `value` is `NaN`.
                     /// - If `rm` is `Exact` but `value` is not exactly equal to any value of the
                     ///   unsigned type.
-                    /// - If `value` is greater than the maximum value of the unsigned type and
-                    ///   `rm` is `Ceiling` or `Up`.
+                    /// - If `value` is greater than the maximum value of the unsigned type and `rm`
+                    ///   is `Ceiling` or `Up`.
                     /// - If `value` is negative and `rm` is `Floor` or `Up`.
                     ///
                     /// # Examples
@@ -745,28 +738,24 @@ fn signed_rounding_from_primitive_float<
     if value.is_infinite() {
         return if value.is_sign_positive() {
             match rm {
-                RoundingMode::Exact => {
+                Exact => {
                     panic!("Value cannot be represented exactly as a {}", S::NAME)
                 }
-                RoundingMode::Down | RoundingMode::Floor | RoundingMode::Nearest => {
-                    (S::MAX, Ordering::Less)
-                }
+                Down | Floor | Nearest => (S::MAX, Less),
                 _ => panic!("Cannot round away from extreme value"),
             }
         } else {
             match rm {
-                RoundingMode::Exact => {
+                Exact => {
                     panic!("Value cannot be represented exactly as a {}", S::NAME)
                 }
-                RoundingMode::Down | RoundingMode::Nearest | RoundingMode::Ceiling => {
-                    (S::MIN, Ordering::Greater)
-                }
+                Down | Nearest | Ceiling => (S::MIN, Greater),
                 _ => panic!("Cannot round away from extreme value"),
             }
         };
     }
     if value == F::ZERO {
-        return (S::ZERO, Ordering::Equal);
+        return (S::ZERO, Equal);
     }
     if value.is_sign_positive() {
         let (abs, o) = U::rounding_from(value, rm);
@@ -774,12 +763,10 @@ fn signed_rounding_from_primitive_float<
             (n, o)
         } else {
             match rm {
-                RoundingMode::Exact => {
+                Exact => {
                     panic!("Value cannot be represented exactly as an {}", S::NAME)
                 }
-                RoundingMode::Floor | RoundingMode::Down | RoundingMode::Nearest => {
-                    (S::MAX, Ordering::Less)
-                }
+                Floor | Down | Nearest => (S::MAX, Less),
                 _ => panic!(
                     "Value is greater than {}::MAX and rounding mode is {}",
                     S::NAME,
@@ -798,12 +785,10 @@ fn signed_rounding_from_primitive_float<
             (n, o.reverse())
         } else {
             match rm {
-                RoundingMode::Exact => {
+                Exact => {
                     panic!("Value cannot be represented exactly as an {}", S::NAME)
                 }
-                RoundingMode::Ceiling | RoundingMode::Down | RoundingMode::Nearest => {
-                    (S::MIN, Ordering::Greater)
-                }
+                Ceiling | Down | Nearest => (S::MIN, Greater),
                 _ => panic!(
                     "Value is smaller than {}::MIN and rounding mode is {}",
                     S::NAME,
@@ -910,23 +895,21 @@ macro_rules! impl_from_float_signed {
             ($f:ident) => {
                 impl RoundingFrom<$i> for $f {
                     /// Converts a value of a signed type to a value of a floating point type
-                    /// according to a specified
-                    /// [`RoundingMode`](crate::rounding_modes::RoundingMode). An [`Ordering`] is
-                    /// also returned, indicating whether the returned value is less than, equal
-                    /// to, or greater than the original value.
+                    /// according to a specified [`RoundingMode`]. An [`Ordering`] is also returned,
+                    /// indicating whether the returned value is less than, equal to, or greater
+                    /// than the original value.
                     ///
                     /// - If the rounding mode is `Floor`, the largest float less than or equal to
                     ///   the value is returned.
                     /// - If the rounding mode is `Ceiling`, the smallest float greater than or
                     ///   equal to the value is returned.
-                    /// - If the rounding mode is `Down`, then the rounding proceeds as with
-                    ///   `Floor` if the value is non-negative and as with `Ceiling` if the value
-                    ///   is negative.
-                    /// - If the rounding mode is `Up`, then the rounding proceeds as with
-                    ///   `Ceiling` if the value is non-negative and as with `Floor` if the value
-                    ///   is negative.
-                    /// - If the rounding mode is `Nearest`, then the nearest float is returned.
-                    ///   If the value is exactly between two floats, the float with the zero
+                    /// - If the rounding mode is `Down`, then the rounding proceeds as with `Floor`
+                    ///   if the value is non-negative and as with `Ceiling` if the value is
+                    ///   negative.
+                    /// - If the rounding mode is `Up`, then the rounding proceeds as with `Ceiling`
+                    ///   if the value is non-negative and as with `Floor` if the value is negative.
+                    /// - If the rounding mode is `Nearest`, then the nearest float is returned. If
+                    ///   the value is exactly between two floats, the float with the zero
                     ///   least-significant bit in its representation is selected.
                     ///
                     /// # Worst-case complexity
@@ -946,32 +929,30 @@ macro_rules! impl_from_float_signed {
 
                 impl RoundingFrom<$f> for $i {
                     /// Converts a value of a floating point type to a value of a signed type
-                    /// according to a specified
-                    /// [`RoundingMode`](crate::rounding_modes::RoundingMode). An [`Ordering`] is
-                    /// also returned, indicating whether the returned value is less than, equal
-                    /// to, or greater than the original value.
+                    /// according to a specified [`RoundingMode`]. An [`Ordering`] is also returned,
+                    /// indicating whether the returned value is less than, equal to, or greater
+                    /// than the original value.
                     ///
                     /// - If the rounding mode is `Floor`, the largest number less than or equal to
                     ///   the value is returned. If the float is greater than the maximum
                     ///   representable signed value, the maximum signed value is returned. If the
-                    ///   float is smaller than the minimum representable signed value, the
-                    ///   function panics.
+                    ///   float is smaller than the minimum representable signed value, the function
+                    ///   panics.
                     /// - If the rounding mode is `Ceiling`, the smallest number greater than or
                     ///   equal to the value is returned. If the float is smaller than the minimum
                     ///   representable signed value, the minimum signed value is returned. If the
-                    ///   float is greater than the maximum representable signed value, the
-                    ///   function panics.
-                    /// - If the rounding mode is `Down`, then the rounding proceeds as with
-                    ///   `Floor` if the float is non-negative and as with `Ceiling` if the value
-                    ///   is negative.
-                    /// - If the rounding mode is `Up`, then the rounding proceeds as with
-                    ///   `Ceiling` if the value is non-negative and as with `Floor` if the value
-                    ///   is negative.
+                    ///   float is greater than the maximum representable signed value, the function
+                    ///   panics.
+                    /// - If the rounding mode is `Down`, then the rounding proceeds as with `Floor`
+                    ///   if the float is non-negative and as with `Ceiling` if the value is
+                    ///   negative.
+                    /// - If the rounding mode is `Up`, then the rounding proceeds as with `Ceiling`
+                    ///   if the value is non-negative and as with `Floor` if the value is negative.
                     /// - If the rounding mode is `Nearest`, then the nearest value is returned. If
-                    ///   the value is exactly between two numbers, the even one is selected. If
-                    ///   the float is greater than the maximum representable signed value, the
-                    ///   maximum signed value is returned. If the float is smaller than the
-                    ///   minimum representable signed value, the minimum signed value is returned.
+                    ///   the value is exactly between two numbers, the even one is selected. If the
+                    ///   float is greater than the maximum representable signed value, the maximum
+                    ///   signed value is returned. If the float is smaller than the minimum
+                    ///   representable signed value, the minimum signed value is returned.
                     ///
                     /// # Worst-case complexity
                     /// Constant time and additional memory.
