@@ -1,3 +1,11 @@
+// Copyright © 2024 Mikhail Hogrefe
+//
+// This file is part of Malachite.
+//
+// Malachite is free software: you can redistribute it and/or modify it under the terms of the GNU
+// Lesser General Public License (LGPL) as published by the Free Software Foundation; either version
+// 3 of the License, or (at your option) any later version. See <https://www.gnu.org/licenses/>.
+
 use crate::num::arithmetic::traits::{
     ArithmeticCheckedShl, DivisibleByPowerOf2, ModPowerOf2, ShrRound,
 };
@@ -9,8 +17,8 @@ use crate::num::conversion::traits::{
     WrappingFrom,
 };
 use crate::num::logic::traits::{BitAccess, LeadingZeros, LowMask, SignificantBits, TrailingZeros};
-use crate::rounding_modes::RoundingMode;
-use std::cmp::Ordering;
+use crate::rounding_modes::RoundingMode::{self, *};
+use core::cmp::Ordering::{self, *};
 
 fn raw_mantissa_and_exponent<T: PrimitiveFloat>(x: T) -> (u64, u64) {
     let bits = x.to_bits();
@@ -112,10 +120,9 @@ fn from_integer_mantissa_and_exponent_primitive_float<T: PrimitiveFloat>(
     } else if sci_exponent < T::MIN_NORMAL_EXPONENT {
         if adjusted_exponent < T::MIN_EXPONENT {
             return None;
-        } else {
-            raw_exponent = 0;
-            raw_mantissa = integer_mantissa << (adjusted_exponent - T::MIN_EXPONENT);
         }
+        raw_exponent = 0;
+        raw_mantissa = integer_mantissa << (adjusted_exponent - T::MIN_EXPONENT);
     } else if mantissa_bits > T::MANTISSA_WIDTH + 1 {
         return None;
     } else {
@@ -211,8 +218,8 @@ fn from_sci_mantissa_and_exponent_primitive_float<T: PrimitiveFloat>(
 /// returned, indicating whether the mantissa and exponent correspond to a value less than, equal
 /// to, or greater than the original value.
 ///
-/// When $x$ is positive, we can write $x = 2^{e_s}m_s$, where $e_s$ is an integer and $m_s$ is
-/// a rational number with $1 \leq m_s < 2$. We represent the rational mantissa as a float. The
+/// When $x$ is positive, we can write $x = 2^{e_s}m_s$, where $e_s$ is an integer and $m_s$ is a
+/// rational number with $1 \leq m_s < 2$. We represent the rational mantissa as a float. The
 /// conversion might not be exact, so we round to the nearest float using the provided rounding
 /// mode. If the rounding mode is `Exact` but the conversion is not exact, `None` is returned.
 /// $$
@@ -227,10 +234,9 @@ fn from_sci_mantissa_and_exponent_primitive_float<T: PrimitiveFloat>(
 /// use malachite_base::num::basic::floats::PrimitiveFloat;
 /// use malachite_base::num::basic::unsigneds::PrimitiveUnsigned;
 /// use malachite_base::num::conversion::mantissa_and_exponent::*;
-/// use malachite_base::num::conversion::traits::SciMantissaAndExponent;
 /// use malachite_base::num::float::NiceFloat;
-/// use malachite_base::rounding_modes::RoundingMode;
-/// use std::cmp::Ordering;
+/// use malachite_base::rounding_modes::RoundingMode::{self, *};
+/// use std::cmp::Ordering::{self, *};
 ///
 /// fn test<T: PrimitiveUnsigned, U: PrimitiveFloat>(
 ///     n: T,
@@ -242,21 +248,21 @@ fn from_sci_mantissa_and_exponent_primitive_float<T: PrimitiveFloat>(
 ///         out.map(|(m, e, o)| (NiceFloat(m), e, o))
 ///     );
 /// }
-/// test::<u32, f32>(3, RoundingMode::Down, Some((1.5, 1, Ordering::Equal)));
-/// test::<u32, f32>(3, RoundingMode::Ceiling, Some((1.5, 1, Ordering::Equal)));
-/// test::<u32, f32>(3, RoundingMode::Up, Some((1.5, 1, Ordering::Equal)));
-/// test::<u32, f32>(3, RoundingMode::Nearest, Some((1.5, 1, Ordering::Equal)));
-/// test::<u32, f32>(3, RoundingMode::Exact, Some((1.5, 1, Ordering::Equal)));
+/// test::<u32, f32>(3, Down, Some((1.5, 1, Equal)));
+/// test::<u32, f32>(3, Ceiling, Some((1.5, 1, Equal)));
+/// test::<u32, f32>(3, Up, Some((1.5, 1, Equal)));
+/// test::<u32, f32>(3, Nearest, Some((1.5, 1, Equal)));
+/// test::<u32, f32>(3, Exact, Some((1.5, 1, Equal)));
 ///
-/// test::<u32, f32>(123, RoundingMode::Floor, Some((1.921875, 6, Ordering::Equal)));
-/// test::<u32, f32>(123, RoundingMode::Down, Some((1.921875, 6, Ordering::Equal)));
-/// test::<u32, f32>(123, RoundingMode::Ceiling, Some((1.921875, 6, Ordering::Equal)));
-/// test::<u32, f32>(123, RoundingMode::Up, Some((1.921875, 6, Ordering::Equal)));
-/// test::<u32, f32>(123, RoundingMode::Nearest, Some((1.921875, 6, Ordering::Equal)));
-/// test::<u32, f32>(123, RoundingMode::Exact, Some((1.921875, 6, Ordering::Equal)));
+/// test::<u32, f32>(123, Floor, Some((1.921875, 6, Equal)));
+/// test::<u32, f32>(123, Down, Some((1.921875, 6, Equal)));
+/// test::<u32, f32>(123, Ceiling, Some((1.921875, 6, Equal)));
+/// test::<u32, f32>(123, Up, Some((1.921875, 6, Equal)));
+/// test::<u32, f32>(123, Nearest, Some((1.921875, 6, Equal)));
+/// test::<u32, f32>(123, Exact, Some((1.921875, 6, Equal)));
 ///
-/// test::<u32, f32>(1000000000, RoundingMode::Nearest, Some((1.8626451, 29, Ordering::Equal)));
-/// test::<u32, f32>(999999999, RoundingMode::Nearest, Some((1.8626451, 29, Ordering::Greater)));
+/// test::<u32, f32>(1000000000, Nearest, Some((1.8626451, 29, Equal)));
+/// test::<u32, f32>(999999999, Nearest, Some((1.8626451, 29, Greater)));
 /// ```
 pub fn sci_mantissa_and_exponent_round<T: PrimitiveUnsigned, U: PrimitiveFloat>(
     x: T,
@@ -267,17 +273,14 @@ pub fn sci_mantissa_and_exponent_round<T: PrimitiveUnsigned, U: PrimitiveFloat>(
     let mut exponent = significant_bits - 1;
     let (mut raw_mantissa, o) = if significant_bits > U::MANTISSA_WIDTH {
         let shift = significant_bits - U::MANTISSA_WIDTH - 1;
-        if rm == RoundingMode::Exact && TrailingZeros::trailing_zeros(x) < shift {
+        if rm == Exact && TrailingZeros::trailing_zeros(x) < shift {
             return None;
         }
         let (s, o) = x.shr_round(shift, rm);
         (s.wrapping_into(), o)
     } else {
         let x: u64 = x.wrapping_into();
-        (
-            x << (U::MANTISSA_WIDTH - significant_bits + 1),
-            Ordering::Equal,
-        )
+        (x << (U::MANTISSA_WIDTH - significant_bits + 1), Equal)
     };
     if raw_mantissa.significant_bits() == U::MANTISSA_WIDTH + 2 {
         // Rounding up to a power of 2
@@ -294,16 +297,16 @@ pub fn sci_mantissa_and_exponent_round<T: PrimitiveUnsigned, U: PrimitiveFloat>(
 }
 
 /// Constructs a primitive integer from its scientific mantissa and exponent. An [`Ordering`] is
-/// also returned, indicating whether the returned value is less than, equal to, or greater than
-/// the exact value implied by the input.
+/// also returned, indicating whether the returned value is less than, equal to, or greater than the
+/// exact value implied by the input.
 ///
 /// When $x$ is positive, we can write $x = 2^{e_s}m_s$, where $e_s$ is an integer and $m_s$ is a
 /// rational number with $1 \leq m_s < 2$. Here, the rational mantissa is provided as a float. If
 /// the mantissa is outside the range $[1, 2)$, `None` is returned.
 ///
 /// Some combinations of mantissas and exponents do not specify an integer, in which case the
-/// resulting value is rounded to an integer using the specified rounding mode. If the rounding
-/// mode is `Exact` but the input does not exactly specify an integer, `None` is returned.
+/// resulting value is rounded to an integer using the specified rounding mode. If the rounding mode
+/// is `Exact` but the input does not exactly specify an integer, `None` is returned.
 ///
 /// $$
 /// f(x, r) \approx 2^{e_s}m_s.
@@ -320,10 +323,8 @@ pub fn sci_mantissa_and_exponent_round<T: PrimitiveUnsigned, U: PrimitiveFloat>(
 /// use malachite_base::num::basic::floats::PrimitiveFloat;
 /// use malachite_base::num::basic::unsigneds::PrimitiveUnsigned;
 /// use malachite_base::num::conversion::mantissa_and_exponent::*;
-/// use malachite_base::num::conversion::traits::SciMantissaAndExponent;
-/// use malachite_base::rounding_modes::RoundingMode;
-/// use std::cmp::Ordering;
-/// use std::str::FromStr;
+/// use malachite_base::rounding_modes::RoundingMode::{self, *};
+/// use std::cmp::Ordering::{self, *};
 ///
 /// fn test<T: PrimitiveUnsigned, U: PrimitiveFloat>(
 ///     mantissa: U,
@@ -335,24 +336,24 @@ pub fn sci_mantissa_and_exponent_round<T: PrimitiveUnsigned, U: PrimitiveFloat>(
 ///         from_sci_mantissa_and_exponent_round::<T, U>(mantissa, exponent, rm),
 ///         out
 ///     );
-/// };
-/// test::<u32, f32>(1.5, 1, RoundingMode::Floor, Some((3, Ordering::Equal)));
-/// test::<u32, f32>(1.5, 1, RoundingMode::Down, Some((3, Ordering::Equal)));
-/// test::<u32, f32>(1.5, 1, RoundingMode::Ceiling, Some((3, Ordering::Equal)));
-/// test::<u32, f32>(1.5, 1, RoundingMode::Up, Some((3, Ordering::Equal)));
-/// test::<u32, f32>(1.5, 1, RoundingMode::Nearest, Some((3, Ordering::Equal)));
-/// test::<u32, f32>(1.5, 1, RoundingMode::Exact, Some((3, Ordering::Equal)));
+/// }
+/// test::<u32, f32>(1.5, 1, Floor, Some((3, Equal)));
+/// test::<u32, f32>(1.5, 1, Down, Some((3, Equal)));
+/// test::<u32, f32>(1.5, 1, Ceiling, Some((3, Equal)));
+/// test::<u32, f32>(1.5, 1, Up, Some((3, Equal)));
+/// test::<u32, f32>(1.5, 1, Nearest, Some((3, Equal)));
+/// test::<u32, f32>(1.5, 1, Exact, Some((3, Equal)));
 ///
-/// test::<u32, f32>(1.51, 1, RoundingMode::Floor, Some((3, Ordering::Less)));
-/// test::<u32, f32>(1.51, 1, RoundingMode::Down, Some((3, Ordering::Less)));
-/// test::<u32, f32>(1.51, 1, RoundingMode::Ceiling, Some((4, Ordering::Greater)));
-/// test::<u32, f32>(1.51, 1, RoundingMode::Up, Some((4, Ordering::Greater)));
-/// test::<u32, f32>(1.51, 1, RoundingMode::Nearest, Some((3, Ordering::Less)));
-/// test::<u32, f32>(1.51, 1, RoundingMode::Exact, None);
+/// test::<u32, f32>(1.51, 1, Floor, Some((3, Less)));
+/// test::<u32, f32>(1.51, 1, Down, Some((3, Less)));
+/// test::<u32, f32>(1.51, 1, Ceiling, Some((4, Greater)));
+/// test::<u32, f32>(1.51, 1, Up, Some((4, Greater)));
+/// test::<u32, f32>(1.51, 1, Nearest, Some((3, Less)));
+/// test::<u32, f32>(1.51, 1, Exact, None);
 ///
-/// test::<u32, f32>(2.0, 1, RoundingMode::Floor, None);
-/// test::<u32, f32>(10.0, 1, RoundingMode::Floor, None);
-/// test::<u32, f32>(0.5, 1, RoundingMode::Floor, None);
+/// test::<u32, f32>(2.0, 1, Floor, None);
+/// test::<u32, f32>(10.0, 1, Floor, None);
+/// test::<u32, f32>(0.5, 1, Floor, None);
 /// ```
 pub fn from_sci_mantissa_and_exponent_round<T: PrimitiveUnsigned, U: PrimitiveFloat>(
     sci_mantissa: U,
@@ -369,10 +370,10 @@ pub fn from_sci_mantissa_and_exponent_round<T: PrimitiveUnsigned, U: PrimitiveFl
         T::try_from(raw_mantissa)
             .ok()?
             .arithmetic_checked_shl(sci_exponent - U::MANTISSA_WIDTH)
-            .map(|n| (n, Ordering::Equal))
+            .map(|n| (n, Equal))
     } else {
         let shift = U::MANTISSA_WIDTH - sci_exponent;
-        if rm == RoundingMode::Exact && TrailingZeros::trailing_zeros(raw_mantissa) < shift {
+        if rm == Exact && TrailingZeros::trailing_zeros(raw_mantissa) < shift {
             return None;
         }
         let (s, o) = raw_mantissa.shr_round(shift, rm);
@@ -467,8 +468,8 @@ macro_rules! impl_mantissa_and_exponent_unsigned {
             /// or `None` if the result cannot be exactly represented as an integer of the desired
             /// type (this happens if the exponent is too large).
             ///
-            /// The input does not have to be reduced; that is to say, the mantissa does not have
-            /// to be odd.
+            /// The input does not have to be reduced; that is to say, the mantissa does not have to
+            /// be odd.
             ///
             /// # Worst-case complexity
             /// Constant time and additional memory.
@@ -517,8 +518,7 @@ macro_rules! impl_sci_mantissa_and_exponent_unsigned {
                     /// See [here](super::mantissa_and_exponent#sci_mantissa_and_exponent).
                     #[inline]
                     fn sci_mantissa_and_exponent(self) -> ($f, u64) {
-                        let (m, e, _) =
-                            sci_mantissa_and_exponent_round(self, RoundingMode::Nearest).unwrap();
+                        let (m, e, _) = sci_mantissa_and_exponent_round(self, Nearest).unwrap();
                         (m, e)
                     }
 
@@ -551,12 +551,8 @@ macro_rules! impl_sci_mantissa_and_exponent_unsigned {
                         sci_mantissa: $f,
                         sci_exponent: u64,
                     ) -> Option<$u> {
-                        from_sci_mantissa_and_exponent_round(
-                            sci_mantissa,
-                            sci_exponent,
-                            RoundingMode::Nearest,
-                        )
-                        .map(|p| p.0)
+                        from_sci_mantissa_and_exponent_round(sci_mantissa, sci_exponent, Nearest)
+                            .map(|p| p.0)
                     }
                 }
             };
@@ -573,8 +569,7 @@ macro_rules! impl_mantissa_and_exponent_primitive_float {
             ///
             /// The raw exponent and raw mantissa are the actual bit patterns used to represent the
             /// components of `self`. When `self` is nonzero and finite, the raw exponent $e_r$ is
-            /// an integer in $[0, 2^E-2]$ and the raw mantissa $m_r$ is an integer in
-            /// $[0, 2^M-1]$.
+            /// an integer in $[0, 2^E-2]$ and the raw mantissa $m_r$ is an integer in $[0, 2^M-1]$.
             ///
             /// When `self` is nonzero and finite, $f(x) = (m_r, e_r)$, where
             /// $$
@@ -593,9 +588,8 @@ macro_rules! impl_mantissa_and_exponent_primitive_float {
             /// $$
             /// and $M$ and $E$ are the mantissa width and exponent width, respectively.
             ///
-            /// For zeros, infinities, or `NaN`, refer to
-            /// [IEEE 754](https://standards.ieee.org/ieee/754/6210/) or look at the examples
-            /// below.
+            /// For zeros, infinities, or `NaN`, refer to [IEEE
+            /// 754](https://standards.ieee.org/ieee/754/6210/) or look at the examples below.
             ///
             /// The inverse operation is [`Self::from_raw_mantissa_and_exponent`].
             ///
@@ -611,8 +605,8 @@ macro_rules! impl_mantissa_and_exponent_primitive_float {
 
             /// Returns the raw mantissa.
             ///
-            /// The raw mantissa is the actual bit pattern used to represent the mantissa of
-            /// `self`. When `self` is nonzero and finite, it is an integer in $[0, 2^M-1]$.
+            /// The raw mantissa is the actual bit pattern used to represent the mantissa of `self`.
+            /// When `self` is nonzero and finite, it is an integer in $[0, 2^M-1]$.
             ///
             /// When `self` is nonzero and finite,
             /// $$
@@ -624,9 +618,8 @@ macro_rules! impl_mantissa_and_exponent_primitive_float {
             /// $$
             /// where $M$ and $E$ are the mantissa width and exponent width, respectively.
             ///
-            /// For zeros, infinities, or `NaN`, refer to
-            /// [IEEE 754](https://standards.ieee.org/ieee/754/6210/) or look at the examples
-            /// below.
+            /// For zeros, infinities, or `NaN`, refer to [IEEE
+            /// 754](https://standards.ieee.org/ieee/754/6210/) or look at the examples below.
             ///
             /// # Worst-case complexity
             /// Constant time and additional memory.
@@ -640,8 +633,8 @@ macro_rules! impl_mantissa_and_exponent_primitive_float {
 
             /// Returns the raw exponent.
             ///
-            /// The raw exponent is the actual bit pattern used to represent the exponent of
-            /// `self`. When `self` is nonzero and finite, it is an integer in $[0, 2^E-2]$.
+            /// The raw exponent is the actual bit pattern used to represent the exponent of `self`.
+            /// When `self` is nonzero and finite, it is an integer in $[0, 2^E-2]$.
             ///
             /// When `self` is nonzero and finite,
             /// $$
@@ -652,9 +645,8 @@ macro_rules! impl_mantissa_and_exponent_primitive_float {
             /// $$
             /// where $M$ and $E$ are the mantissa width and exponent width, respectively.
             ///
-            /// For zeros, infinities, or `NaN`, refer to
-            /// [IEEE 754](https://standards.ieee.org/ieee/754/6210/) or look at the examples
-            /// below.
+            /// For zeros, infinities, or `NaN`, refer to [IEEE
+            /// 754](https://standards.ieee.org/ieee/754/6210/) or look at the examples below.
             ///
             /// # Worst-case complexity
             /// Constant time and additional memory.
@@ -670,8 +662,8 @@ macro_rules! impl_mantissa_and_exponent_primitive_float {
             ///
             /// The raw exponent and raw mantissa are the actual bit patterns used to represent the
             /// components of a float. When the float is nonzero and finite, the raw exponent $e_r$
-            /// is an integer in $[0, 2^E-2]$ and the raw mantissa $m_r$ is an integer in
-            /// $[0, 2^M-1]$.
+            /// is an integer in $[0, 2^E-2]$ and the raw mantissa $m_r$ is an integer in $[0,
+            /// 2^M-1]$.
             ///
             /// When the exponent is not `2^E-1`,
             /// $$
@@ -682,9 +674,8 @@ macro_rules! impl_mantissa_and_exponent_primitive_float {
             /// $$
             /// where $M$ and $E$ are the mantissa width and exponent width, respectively.
             ///
-            /// For zeros, infinities, or `NaN`, refer to
-            /// [IEEE 754](https://standards.ieee.org/ieee/754/6210/) or look at the examples
-            /// below.
+            /// For zeros, infinities, or `NaN`, refer to [IEEE
+            /// 754](https://standards.ieee.org/ieee/754/6210/) or look at the examples below.
             ///
             /// This function only outputs a single, canonical, `NaN`.
             ///
@@ -776,8 +767,8 @@ macro_rules! impl_mantissa_and_exponent_primitive_float {
             /// $$
             /// f(x) = 2^{e_i}m_i,
             /// $$
-            /// or `None` if the result cannot be exactly represented as a float of the desired
-            /// type (this happens if the exponent is too large or too small, or if the mantissa's
+            /// or `None` if the result cannot be exactly represented as a float of the desired type
+            /// (this happens if the exponent is too large or too small, or if the mantissa's
             /// precision is too high for the exponent).
             ///
             /// The input does not have to be reduced; that is to say, the mantissa does not have to
@@ -829,9 +820,9 @@ macro_rules! impl_mantissa_and_exponent_primitive_float {
             /// Returns the scientific mantissa.
             ///
             /// When $x$ is positive, nonzero, and finite, we can write $x = 2^{e_s}m_s$, where
-            /// $e_s$ is an integer and $m_s$ is a rational number with $1 \leq m_s < 2$. If $x$
-            /// is a valid float, the scientific mantissa $m_s$ is always exactly representable as
-            /// a float of the same type. We have
+            /// $e_s$ is an integer and $m_s$ is a rational number with $1 \leq m_s < 2$. If $x$ is
+            /// a valid float, the scientific mantissa $m_s$ is always exactly representable as a
+            /// float of the same type. We have
             /// $$
             /// f(x) = \frac{x}{2^{\lfloor \log_2 x \rfloor}}.
             /// $$
@@ -878,10 +869,9 @@ macro_rules! impl_mantissa_and_exponent_primitive_float {
             /// $$
             /// f(x) = 2^{e_s}m_s,
             /// $$
-            /// or `None` if the result cannot be exactly represented as a float of the desired
-            /// type (this happens if the exponent is too large or too small, if the mantissa is
-            /// not in the range $[1, 2)$, or if the mantissa's precision is too high for the
-            /// exponent).
+            /// or `None` if the result cannot be exactly represented as a float of the desired type
+            /// (this happens if the exponent is too large or too small, if the mantissa is not in
+            /// the range $[1, 2)$, or if the mantissa's precision is too high for the exponent).
             ///
             /// # Worst-case complexity
             /// Constant time and additional memory.
