@@ -1,3 +1,11 @@
+// Copyright © 2024 Mikhail Hogrefe
+//
+// This file is part of Malachite.
+//
+// Malachite is free software: you can redistribute it and/or modify it under the terms of the GNU
+// Lesser General Public License (LGPL) as published by the Free Software Foundation; either version
+// 3 of the License, or (at your option) any later version. See <https://www.gnu.org/licenses/>.
+
 use crate::iterators::{nonzero_values, NonzeroValues};
 use crate::num::arithmetic::traits::{PowerOf2, RoundToMultipleOfPowerOf2};
 use crate::num::basic::floats::PrimitiveFloat;
@@ -8,15 +16,16 @@ use crate::num::conversion::traits::{ExactFrom, WrappingFrom};
 use crate::num::float::NiceFloat;
 use crate::num::iterators::{ruler_sequence, RulerSequence};
 use crate::num::logic::traits::{BitAccess, NotAssign, SignificantBits};
-use crate::rounding_modes::RoundingMode;
+use crate::rounding_modes::RoundingMode::*;
 use crate::tuples::exhaustive::{
     exhaustive_dependent_pairs, lex_dependent_pairs, ExhaustiveDependentPairs,
     ExhaustiveDependentPairsYsGenerator, LexDependentPairs,
 };
+use alloc::vec::IntoIter;
+use alloc::vec::Vec;
+use core::iter::{once, Chain, Once, Rev};
+use core::marker::PhantomData;
 use itertools::{Interleave, Itertools};
-use std::iter::{once, Chain, Once, Rev};
-use std::marker::PhantomData;
-use std::vec::IntoIter;
 
 /// Generates all primitive integers in an interval.
 ///
@@ -56,13 +65,13 @@ impl<T: PrimitiveInt> DoubleEndedIterator for PrimitiveIntIncreasingRange<T> {
 /// Generates all values of a signed integer type in an interval, in order of increasing absolute
 /// value.
 ///
-/// This `enum` is created by [`exhaustive_signed_range`] and
-/// [`exhaustive_signed_inclusive_range`]; see their documentation for more.
+/// This `enum` is created by [`exhaustive_signed_range`] and [`exhaustive_signed_inclusive_range`];
+/// see their documentation for more.
 #[derive(Clone, Debug)]
 pub enum ExhaustiveSignedRange<T: PrimitiveSigned> {
     NonNegative(PrimitiveIntIncreasingRange<T>),
     NonPositive(Rev<PrimitiveIntIncreasingRange<T>>),
-    BothSigns(Chain<Once<T>, PrimitiveIntUpDown<T>>),
+    BothSigns(ExhaustiveSigneds<T>),
 }
 
 impl<T: PrimitiveSigned> Iterator for ExhaustiveSignedRange<T> {
@@ -132,13 +141,15 @@ pub fn exhaustive_positive_primitive_ints<T: PrimitiveInt>() -> PrimitiveIntIncr
     primitive_int_increasing_inclusive_range(T::ONE, T::MAX)
 }
 
+pub type ExhaustiveSigneds<T> = Chain<Once<T>, PrimitiveIntUpDown<T>>;
+
 /// Generates all signed integers in order of increasing absolute value.
 ///
 /// When two numbers have the same absolute value, the positive one comes first.
 ///
-/// The output satisfies
-/// $(|x_i|, \operatorname{sgn}(-x_i)) <_\mathrm{lex} (|x_j|, \operatorname{sgn}(-x_j))$ whenever
-/// $i, j \\in [-2^{W-1}, 2^{W-1})$, where $W$ is the width of the type, and $i < j$.
+/// The output satisfies $(|x_i|, \operatorname{sgn}(-x_i)) <_\mathrm{lex} (|x_j|,
+/// \operatorname{sgn}(-x_j))$ whenever $i, j \\in [-2^{W-1}, 2^{W-1})$, where $W$ is the width of
+/// the type, and $i < j$.
 ///
 /// The output length is $2^W$.
 ///
@@ -156,7 +167,7 @@ pub fn exhaustive_positive_primitive_ints<T: PrimitiveInt>() -> PrimitiveIntIncr
 /// )
 /// ```
 #[inline]
-pub fn exhaustive_signeds<T: PrimitiveSigned>() -> Chain<Once<T>, PrimitiveIntUpDown<T>> {
+pub fn exhaustive_signeds<T: PrimitiveSigned>() -> ExhaustiveSigneds<T> {
     once(T::ZERO).chain(exhaustive_nonzero_signeds())
 }
 
@@ -212,10 +223,9 @@ pub fn exhaustive_negative_signeds<T: PrimitiveSigned>() -> Rev<PrimitiveIntIncr
 ///
 /// When two numbers have the same absolute value, the positive one comes first.
 ///
-/// The output satisfies
-/// $(|x_i|, \operatorname{sgn}(-x_i)) <_\mathrm{lex} (|x_j|, \operatorname{sgn}(-x_j))$ whenever
-/// $i, j \\in [-2^{W-1}, 2^{W-1}) \\setminus \\{0\\}$, where $W$ is the width of the type, and
-/// $i < j$.
+/// The output satisfies $(|x_i|, \operatorname{sgn}(-x_i)) <_\mathrm{lex} (|x_j|,
+/// \operatorname{sgn}(-x_j))$ whenever $i, j \\in [-2^{W-1}, 2^{W-1}) \\setminus \\{0\\}$, where
+/// $W$ is the width of the type, and $i < j$.
 ///
 /// The output length is $2^W-1$.
 ///
@@ -268,9 +278,7 @@ pub fn primitive_int_increasing_range<T: PrimitiveInt>(
     a: T,
     b: T,
 ) -> PrimitiveIntIncreasingRange<T> {
-    if a > b {
-        panic!("a must be less than or equal to b. a: {a}, b: {b}");
-    }
+    assert!(a <= b, "a must be less than or equal to b. a: {a}, b: {b}");
     PrimitiveIntIncreasingRange {
         a: Some(a),
         b: Some(b),
@@ -307,9 +315,7 @@ pub fn primitive_int_increasing_inclusive_range<T: PrimitiveInt>(
     a: T,
     b: T,
 ) -> PrimitiveIntIncreasingRange<T> {
-    if a > b {
-        panic!("a must be less than or equal to b. a: {a}, b: {b}");
-    }
+    assert!(a <= b, "a must be less than or equal to b. a: {a}, b: {b}");
     PrimitiveIntIncreasingRange {
         a: Some(a),
         b: b.checked_add(T::ONE),
@@ -323,9 +329,8 @@ pub fn primitive_int_increasing_inclusive_range<T: PrimitiveInt>(
 /// than or equal to $b$. If $a$ and $b$ are equal, the range is empty. This function cannot create
 /// a range that includes `T::MAX`; for that, use [`exhaustive_signed_inclusive_range`].
 ///
-/// The output satisfies
-/// $(|x_i|, \operatorname{sgn}(-x_i)) <_\mathrm{lex} (|x_j|, \operatorname{sgn}(-x_j))$ whenever
-/// $i, j \\in [0, b - a)$ and $i < j$.
+/// The output satisfies $(|x_i|, \operatorname{sgn}(-x_i)) <_\mathrm{lex} (|x_j|,
+/// \operatorname{sgn}(-x_j))$ whenever $i, j \\in [0, b - a)$ and $i < j$.
 ///
 /// The output length is $b - a$.
 ///
@@ -346,9 +351,7 @@ pub fn primitive_int_increasing_inclusive_range<T: PrimitiveInt>(
 /// )
 /// ```
 pub fn exhaustive_signed_range<T: PrimitiveSigned>(a: T, b: T) -> ExhaustiveSignedRange<T> {
-    if a > b {
-        panic!("a must be less than or equal to b. a: {a}, b: {b}");
-    }
+    assert!(a <= b, "a must be less than or equal to b. a: {a}, b: {b}");
     if a >= T::ZERO {
         ExhaustiveSignedRange::NonNegative(primitive_int_increasing_range(a, b))
     } else if b <= T::ZERO {
@@ -369,9 +372,8 @@ pub fn exhaustive_signed_range<T: PrimitiveSigned>(a: T, b: T) -> ExhaustiveSign
 /// When two numbers have the same absolute value, the positive one comes first. $a$ must be less
 /// than or equal to $b$. If $a$ and $b$ are equal, the range contains a single element.
 ///
-/// The output satisfies
-/// $(|x_i|, \operatorname{sgn}(-x_i)) <_\mathrm{lex} (|x_j|, \operatorname{sgn}(-x_j))$ whenever
-/// $i, j \\in [0, b - a]$ and $i < j$.
+/// The output satisfies $(|x_i|, \operatorname{sgn}(-x_i)) <_\mathrm{lex} (|x_j|,
+/// \operatorname{sgn}(-x_j))$ whenever $i, j \\in [0, b - a]$ and $i < j$.
 ///
 /// The output length is $b - a + 1$.
 ///
@@ -395,9 +397,7 @@ pub fn exhaustive_signed_inclusive_range<T: PrimitiveSigned>(
     a: T,
     b: T,
 ) -> ExhaustiveSignedRange<T> {
-    if a > b {
-        panic!("a must be less than or equal to b. a: {a}, b: {b}");
-    }
+    assert!(a <= b, "a must be less than or equal to b. a: {a}, b: {b}");
     if a >= T::ZERO {
         ExhaustiveSignedRange::NonNegative(primitive_int_increasing_inclusive_range(a, b))
     } else if b <= T::ZERO {
@@ -473,14 +473,19 @@ impl<T: PrimitiveFloat> DoubleEndedIterator for PrimitiveFloatIncreasingRange<T>
 /// use malachite_base::num::float::NiceFloat;
 ///
 /// assert_eq!(
-///     prefix_to_string(primitive_float_increasing_range::<f32>(1.0, 2.0).map(NiceFloat), 20),
+///     prefix_to_string(
+///         primitive_float_increasing_range::<f32>(1.0, 2.0).map(NiceFloat),
+///         20
+///     ),
 ///     "[1.0, 1.0000001, 1.0000002, 1.0000004, 1.0000005, 1.0000006, 1.0000007, 1.0000008, \
 ///     1.000001, 1.0000011, 1.0000012, 1.0000013, 1.0000014, 1.0000015, 1.0000017, 1.0000018, \
 ///     1.0000019, 1.000002, 1.0000021, 1.0000023, ...]"
 /// );
 /// assert_eq!(
 ///     prefix_to_string(
-///         primitive_float_increasing_range::<f32>(1.0, 2.0).rev().map(NiceFloat),
+///         primitive_float_increasing_range::<f32>(1.0, 2.0)
+///             .rev()
+///             .map(NiceFloat),
 ///         20,
 ///     ),
 ///     "[1.9999999, 1.9999998, 1.9999996, 1.9999995, 1.9999994, 1.9999993, 1.9999992, 1.999999, \
@@ -494,13 +499,12 @@ pub fn primitive_float_increasing_range<T: PrimitiveFloat>(
 ) -> PrimitiveFloatIncreasingRange<T> {
     assert!(!a.is_nan());
     assert!(!b.is_nan());
-    if NiceFloat(a) > NiceFloat(b) {
-        panic!(
-            "a must be less than or equal to b. a: {}, b: {}",
-            NiceFloat(a),
-            NiceFloat(b)
-        );
-    }
+    assert!(
+        NiceFloat(a) <= NiceFloat(b),
+        "a must be less than or equal to b. a: {}, b: {}",
+        NiceFloat(a),
+        NiceFloat(b)
+    );
     PrimitiveFloatIncreasingRange {
         phantom: PhantomData,
         xs: primitive_int_increasing_range(
@@ -548,7 +552,9 @@ pub fn primitive_float_increasing_range<T: PrimitiveFloat>(
 /// );
 /// assert_eq!(
 ///     prefix_to_string(
-///         primitive_float_increasing_inclusive_range::<f32>(1.0, 2.0).rev().map(NiceFloat),
+///         primitive_float_increasing_inclusive_range::<f32>(1.0, 2.0)
+///             .rev()
+///             .map(NiceFloat),
 ///         20
 ///     ),
 ///     "[2.0, 1.9999999, 1.9999998, 1.9999996, 1.9999995, 1.9999994, 1.9999993, 1.9999992, \
@@ -562,13 +568,12 @@ pub fn primitive_float_increasing_inclusive_range<T: PrimitiveFloat>(
 ) -> PrimitiveFloatIncreasingRange<T> {
     assert!(!a.is_nan());
     assert!(!b.is_nan());
-    if NiceFloat(a) > NiceFloat(b) {
-        panic!(
-            "a must be less than or equal to b. a: {}, b: {}",
-            NiceFloat(a),
-            NiceFloat(b)
-        );
-    }
+    assert!(
+        NiceFloat(a) <= NiceFloat(b),
+        "a must be less than or equal to b. a: {}, b: {}",
+        NiceFloat(a),
+        NiceFloat(b)
+    );
     PrimitiveFloatIncreasingRange {
         phantom: PhantomData,
         xs: primitive_int_increasing_inclusive_range(
@@ -605,14 +610,19 @@ pub fn primitive_float_increasing_inclusive_range<T: PrimitiveFloat>(
 /// use malachite_base::num::float::NiceFloat;
 ///
 /// assert_eq!(
-///     prefix_to_string(positive_finite_primitive_floats_increasing::<f32>().map(NiceFloat), 20),
+///     prefix_to_string(
+///         positive_finite_primitive_floats_increasing::<f32>().map(NiceFloat),
+///         20
+///     ),
 ///     "[1.0e-45, 3.0e-45, 4.0e-45, 6.0e-45, 7.0e-45, 8.0e-45, 1.0e-44, 1.1e-44, 1.3e-44, \
 ///     1.4e-44, 1.5e-44, 1.7e-44, 1.8e-44, 2.0e-44, 2.1e-44, 2.2e-44, 2.4e-44, 2.5e-44, 2.7e-44, \
 ///     2.8e-44, ...]"
 /// );
 /// assert_eq!(
 ///     prefix_to_string(
-///         positive_finite_primitive_floats_increasing::<f32>().rev().map(NiceFloat),
+///         positive_finite_primitive_floats_increasing::<f32>()
+///             .rev()
+///             .map(NiceFloat),
 ///         20
 ///     ),
 ///     "[3.4028235e38, 3.4028233e38, 3.402823e38, 3.4028229e38, 3.4028227e38, 3.4028225e38, \
@@ -654,7 +664,10 @@ pub fn positive_finite_primitive_floats_increasing<T: PrimitiveFloat>(
 /// use malachite_base::num::float::NiceFloat;
 ///
 /// assert_eq!(
-///     prefix_to_string(negative_finite_primitive_floats_increasing::<f32>().map(NiceFloat), 20),
+///     prefix_to_string(
+///         negative_finite_primitive_floats_increasing::<f32>().map(NiceFloat),
+///         20
+///     ),
 ///     "[-3.4028235e38, -3.4028233e38, -3.402823e38, -3.4028229e38, -3.4028227e38, \
 ///     -3.4028225e38, -3.4028222e38, -3.402822e38, -3.4028218e38, -3.4028216e38, -3.4028214e38, \
 ///     -3.4028212e38, -3.402821e38, -3.4028208e38, -3.4028206e38, -3.4028204e38, -3.4028202e38, \
@@ -662,7 +675,9 @@ pub fn positive_finite_primitive_floats_increasing<T: PrimitiveFloat>(
 /// );
 /// assert_eq!(
 ///     prefix_to_string(
-///         negative_finite_primitive_floats_increasing::<f32>().rev().map(NiceFloat),
+///         negative_finite_primitive_floats_increasing::<f32>()
+///             .rev()
+///             .map(NiceFloat),
 ///         20
 ///     ),
 ///     "[-1.0e-45, -3.0e-45, -4.0e-45, -6.0e-45, -7.0e-45, -8.0e-45, -1.0e-44, -1.1e-44, \
@@ -681,8 +696,8 @@ pub fn negative_finite_primitive_floats_increasing<T: PrimitiveFloat>(
 /// Positive and negative zero are both excluded.
 ///
 /// [-`MAX_FINITE`](super::basic::floats::PrimitiveFloat::MAX_FINITE) is generated first and
-/// [`MAX_FINITE`](super::basic::floats::PrimitiveFloat::MAX_FINITE) is generated last. The
-/// returned iterator is double-ended, so it may be reversed.
+/// [`MAX_FINITE`](super::basic::floats::PrimitiveFloat::MAX_FINITE) is generated last. The returned
+/// iterator is double-ended, so it may be reversed.
 ///
 /// Let $\varphi$ be
 /// [`to_ordered_representation`](super::basic::floats::PrimitiveFloat::to_ordered_representation):
@@ -706,7 +721,10 @@ pub fn negative_finite_primitive_floats_increasing<T: PrimitiveFloat>(
 /// use malachite_base::num::float::NiceFloat;
 ///
 /// assert_eq!(
-///     prefix_to_string(nonzero_finite_primitive_floats_increasing::<f32>().map(NiceFloat), 20),
+///     prefix_to_string(
+///         nonzero_finite_primitive_floats_increasing::<f32>().map(NiceFloat),
+///         20
+///     ),
 ///     "[-3.4028235e38, -3.4028233e38, -3.402823e38, -3.4028229e38, -3.4028227e38, \
 ///     -3.4028225e38, -3.4028222e38, -3.402822e38, -3.4028218e38, -3.4028216e38, -3.4028214e38, \
 ///     -3.4028212e38, -3.402821e38, -3.4028208e38, -3.4028206e38, -3.4028204e38, -3.4028202e38, \
@@ -714,7 +732,9 @@ pub fn negative_finite_primitive_floats_increasing<T: PrimitiveFloat>(
 /// );
 /// assert_eq!(
 ///     prefix_to_string(
-///         nonzero_finite_primitive_floats_increasing::<f32>().rev().map(NiceFloat),
+///         nonzero_finite_primitive_floats_increasing::<f32>()
+///             .rev()
+///             .map(NiceFloat),
 ///         20
 ///     ),
 ///     "[3.4028235e38, 3.4028233e38, 3.402823e38, 3.4028229e38, 3.4028227e38, 3.4028225e38, \
@@ -756,14 +776,22 @@ pub fn nonzero_finite_primitive_floats_increasing<T: PrimitiveFloat>(
 /// use malachite_base::num::float::NiceFloat;
 ///
 /// assert_eq!(
-///     prefix_to_string(finite_primitive_floats_increasing::<f32>().map(NiceFloat), 20),
+///     prefix_to_string(
+///         finite_primitive_floats_increasing::<f32>().map(NiceFloat),
+///         20
+///     ),
 ///     "[-3.4028235e38, -3.4028233e38, -3.402823e38, -3.4028229e38, -3.4028227e38, \
 ///     -3.4028225e38, -3.4028222e38, -3.402822e38, -3.4028218e38, -3.4028216e38, -3.4028214e38, \
 ///     -3.4028212e38, -3.402821e38, -3.4028208e38, -3.4028206e38, -3.4028204e38, -3.4028202e38, \
 ///     -3.40282e38, -3.4028198e38, -3.4028196e38, ...]",
 /// );
 /// assert_eq!(
-///     prefix_to_string(finite_primitive_floats_increasing::<f32>().rev().map(NiceFloat), 20),
+///     prefix_to_string(
+///         finite_primitive_floats_increasing::<f32>()
+///             .rev()
+///             .map(NiceFloat),
+///         20
+///     ),
 ///     "[3.4028235e38, 3.4028233e38, 3.402823e38, 3.4028229e38, 3.4028227e38, 3.4028225e38, \
 ///     3.4028222e38, 3.402822e38, 3.4028218e38, 3.4028216e38, 3.4028214e38, 3.4028212e38, \
 ///     3.402821e38, 3.4028208e38, 3.4028206e38, 3.4028204e38, 3.4028202e38, 3.40282e38, \
@@ -798,18 +826,25 @@ pub fn finite_primitive_floats_increasing<T: PrimitiveFloat>() -> PrimitiveFloat
 /// # Examples
 /// ```
 /// use malachite_base::iterators::prefix_to_string;
-/// use malachite_base::num::basic::floats::PrimitiveFloat;
 /// use malachite_base::num::exhaustive::positive_primitive_floats_increasing;
 /// use malachite_base::num::float::NiceFloat;
 ///
 /// assert_eq!(
-///     prefix_to_string(positive_primitive_floats_increasing::<f32>().map(NiceFloat), 20),
+///     prefix_to_string(
+///         positive_primitive_floats_increasing::<f32>().map(NiceFloat),
+///         20
+///     ),
 ///     "[1.0e-45, 3.0e-45, 4.0e-45, 6.0e-45, 7.0e-45, 8.0e-45, 1.0e-44, 1.1e-44, 1.3e-44, \
 ///     1.4e-44, 1.5e-44, 1.7e-44, 1.8e-44, 2.0e-44, 2.1e-44, 2.2e-44, 2.4e-44, 2.5e-44, 2.7e-44, \
 ///     2.8e-44, ...]"
 /// );
 /// assert_eq!(
-///     prefix_to_string(positive_primitive_floats_increasing::<f32>().rev().map(NiceFloat), 20),
+///     prefix_to_string(
+///         positive_primitive_floats_increasing::<f32>()
+///             .rev()
+///             .map(NiceFloat),
+///         20
+///     ),
 ///     "[Infinity, 3.4028235e38, 3.4028233e38, 3.402823e38, 3.4028229e38, 3.4028227e38, \
 ///     3.4028225e38, 3.4028222e38, 3.402822e38, 3.4028218e38, 3.4028216e38, 3.4028214e38, \
 ///     3.4028212e38, 3.402821e38, 3.4028208e38, 3.4028206e38, 3.4028204e38, 3.4028202e38, \
@@ -845,19 +880,26 @@ pub fn positive_primitive_floats_increasing<T: PrimitiveFloat>() -> PrimitiveFlo
 /// # Examples
 /// ```
 /// use malachite_base::iterators::prefix_to_string;
-/// use malachite_base::num::basic::floats::PrimitiveFloat;
 /// use malachite_base::num::exhaustive::negative_primitive_floats_increasing;
 /// use malachite_base::num::float::NiceFloat;
 ///
 /// assert_eq!(
-///     prefix_to_string(negative_primitive_floats_increasing::<f32>().map(NiceFloat), 20),
+///     prefix_to_string(
+///         negative_primitive_floats_increasing::<f32>().map(NiceFloat),
+///         20
+///     ),
 ///     "[-Infinity, -3.4028235e38, -3.4028233e38, -3.402823e38, -3.4028229e38, -3.4028227e38, \
 ///     -3.4028225e38, -3.4028222e38, -3.402822e38, -3.4028218e38, -3.4028216e38, -3.4028214e38, \
 ///     -3.4028212e38, -3.402821e38, -3.4028208e38, -3.4028206e38, -3.4028204e38, -3.4028202e38, \
 ///     -3.40282e38, -3.4028198e38, ...]"
 /// );
 /// assert_eq!(
-///     prefix_to_string(negative_primitive_floats_increasing::<f32>().rev().map(NiceFloat), 20),
+///     prefix_to_string(
+///         negative_primitive_floats_increasing::<f32>()
+///             .rev()
+///             .map(NiceFloat),
+///         20
+///     ),
 ///     "[-1.0e-45, -3.0e-45, -4.0e-45, -6.0e-45, -7.0e-45, -8.0e-45, -1.0e-44, -1.1e-44, \
 ///     -1.3e-44, -1.4e-44, -1.5e-44, -1.7e-44, -1.8e-44, -2.0e-44, -2.1e-44, -2.2e-44, -2.4e-44, \
 ///     -2.5e-44, -2.7e-44, -2.8e-44, ...]"
@@ -881,8 +923,8 @@ pub fn negative_primitive_floats_increasing<T: PrimitiveFloat>() -> PrimitiveFlo
 ///
 /// The output is
 /// $$
-/// (\varphi^{-1}(k))_ {k=0}^{2^M(2^E-1)-1} ⧺ (\varphi^{-1}(k))_ {k=2^M(2^E-1)+2}^{2^{M+1}(2^E-1)+1}
-/// $$.
+/// (\varphi^{-1}(k))_ {k=0}^{2^M(2^E-1)-1} ⧺ (\varphi^{-1}(k))_
+/// {k=2^M(2^E-1)+2}^{2^{M+1}(2^E-1)+1} $$.
 ///
 /// The output length is $2^{M+1}(2^E-1)$.
 /// - For [`f32`], this is $2^{32}-2^{24}$, or 4278190080.
@@ -894,19 +936,26 @@ pub fn negative_primitive_floats_increasing<T: PrimitiveFloat>() -> PrimitiveFlo
 /// # Examples
 /// ```
 /// use malachite_base::iterators::prefix_to_string;
-/// use malachite_base::num::basic::floats::PrimitiveFloat;
 /// use malachite_base::num::exhaustive::nonzero_primitive_floats_increasing;
 /// use malachite_base::num::float::NiceFloat;
 ///
 /// assert_eq!(
-///     prefix_to_string(nonzero_primitive_floats_increasing::<f32>().map(NiceFloat), 20),
+///     prefix_to_string(
+///         nonzero_primitive_floats_increasing::<f32>().map(NiceFloat),
+///         20
+///     ),
 ///     "[-Infinity, -3.4028235e38, -3.4028233e38, -3.402823e38, -3.4028229e38, -3.4028227e38, \
 ///     -3.4028225e38, -3.4028222e38, -3.402822e38, -3.4028218e38, -3.4028216e38, -3.4028214e38, \
 ///     -3.4028212e38, -3.402821e38, -3.4028208e38, -3.4028206e38, -3.4028204e38, -3.4028202e38, \
 ///     -3.40282e38, -3.4028198e38, ...]"
 /// );
 /// assert_eq!(
-///     prefix_to_string(nonzero_primitive_floats_increasing::<f32>().rev().map(NiceFloat), 20),
+///     prefix_to_string(
+///         nonzero_primitive_floats_increasing::<f32>()
+///             .rev()
+///             .map(NiceFloat),
+///         20
+///     ),
 ///     "[Infinity, 3.4028235e38, 3.4028233e38, 3.402823e38, 3.4028229e38, 3.4028227e38, \
 ///     3.4028225e38, 3.4028222e38, 3.402822e38, 3.4028218e38, 3.4028216e38, 3.4028214e38, \
 ///     3.4028212e38, 3.402821e38, 3.4028208e38, 3.4028206e38, 3.4028204e38, 3.4028202e38, \
@@ -923,8 +972,8 @@ pub fn nonzero_primitive_floats_increasing<T: PrimitiveFloat>(
 ///
 /// Positive and negative zero are both included. Negative zero comes first.
 ///
-/// `NEGATIVE_INFINITY` is generated first and `INFINITY` is generated last. The returned
-/// iterator is double-ended, so it may be reversed.
+/// `NEGATIVE_INFINITY` is generated first and `INFINITY` is generated last. The returned iterator
+/// is double-ended, so it may be reversed.
 ///
 /// Let $\varphi$ be
 /// [`to_ordered_representation`](super::basic::floats::PrimitiveFloat::to_ordered_representation):
@@ -941,7 +990,6 @@ pub fn nonzero_primitive_floats_increasing<T: PrimitiveFloat>(
 /// # Examples
 /// ```
 /// use malachite_base::iterators::prefix_to_string;
-/// use malachite_base::num::basic::floats::PrimitiveFloat;
 /// use malachite_base::num::exhaustive::primitive_floats_increasing;
 /// use malachite_base::num::float::NiceFloat;
 ///
@@ -953,7 +1001,10 @@ pub fn nonzero_primitive_floats_increasing<T: PrimitiveFloat>(
 ///     -3.40282e38, -3.4028198e38, ...]"
 /// );
 /// assert_eq!(
-///     prefix_to_string(primitive_floats_increasing::<f32>().rev().map(NiceFloat), 20),
+///     prefix_to_string(
+///         primitive_floats_increasing::<f32>().rev().map(NiceFloat),
+///         20
+///     ),
 ///     "[Infinity, 3.4028235e38, 3.4028233e38, 3.402823e38, 3.4028229e38, 3.4028227e38, \
 ///     3.4028225e38, 3.4028222e38, 3.402822e38, 3.4028218e38, 3.4028216e38, 3.4028214e38, \
 ///     3.4028212e38, 3.402821e38, 3.4028208e38, 3.4028206e38, 3.4028204e38, 3.4028202e38, \
@@ -966,8 +1017,8 @@ pub fn primitive_floats_increasing<T: PrimitiveFloat>() -> PrimitiveFloatIncreas
 
 /// Generates all finite positive primitive floats with a specified `sci_exponent` and precision.
 ///
-/// This `struct` is created by [`exhaustive_primitive_floats_with_sci_exponent_and_precision`];
-/// see its documentation for more.
+/// This `struct` is created by [`exhaustive_primitive_floats_with_sci_exponent_and_precision`]; see
+/// its documentation for more.
 #[derive(Clone, Debug, Default)]
 pub struct ConstantPrecisionPrimitiveFloats<T: PrimitiveFloat> {
     phantom: PhantomData<*const T>,
@@ -998,17 +1049,17 @@ impl<T: PrimitiveFloat> Iterator for ConstantPrecisionPrimitiveFloats<T> {
 ///
 /// Positive and negative zero are both excluded.
 ///
-/// A finite positive primitive float may be uniquely expressed as $x = m_s2^e_s$, where
-/// $1 \leq m_s < 2$ and $e_s$ is an integer; then $e_s$ is the sci-exponent. An integer $e_s$
-/// occurs as the sci-exponent of a float iff $2-2^{E-1}-M \leq e_s < 2^{E-1}$.
+/// A finite positive primitive float may be uniquely expressed as $x = m_s2^e_s$, where $1 \leq m_s
+/// < 2$ and $e_s$ is an integer; then $e_s$ is the sci-exponent. An integer $e_s$ occurs as the
+/// sci-exponent of a float iff $2-2^{E-1}-M \leq e_s < 2^{E-1}$.
 ///
 /// In the above equation, $m$ is a dyadic rational. Let $p$ be the smallest integer such that
 /// $m2^{p-1}$ is an integer. Then $p$ is the float's precision. It is also the number of
 /// significant bits.
 ///
 /// For example, consider the float $100.0$. It may be written as $\frac{25}{16}2^6$, so
-/// $m=\frac{25}{16}$ and $e=6$. We can write $m$ in binary as $1.1001_2$. Thus, the sci-exponent
-/// is 6 and the precision is 5.
+/// $m=\frac{25}{16}$ and $e=6$. We can write $m$ in binary as $1.1001_2$. Thus, the sci-exponent is
+/// 6 and the precision is 5.
 ///
 /// If $p$ is 1, the output length is 1; otherwise, it is $2^{p-2}$.
 ///
@@ -1156,9 +1207,9 @@ impl<T: PrimitiveFloat> Iterator for ExhaustivePrimitiveFloatsWithExponent<T> {
 ///
 /// Positive and negative zero are both excluded.
 ///
-/// A finite positive primitive float may be uniquely expressed as $x = m_s2^e_s$, where
-/// $1 \leq m_s < 2$ and $e_s$ is an integer; then $e$ is the sci-exponent. An integer $e_s$ occurs
-/// as the sci-exponent of a float iff $2-2^{E-1}-M \leq e_s < 2^{E-1}$.
+/// A finite positive primitive float may be uniquely expressed as $x = m_s2^e_s$, where $1 \leq m_s
+/// < 2$ and $e_s$ is an integer; then $e$ is the sci-exponent. An integer $e_s$ occurs as the
+/// sci-exponent of a float iff $2-2^{E-1}-M \leq e_s < 2^{E-1}$.
 ///
 /// If $e_s \geq 2-2^{E-1}$ (the float is normal), the output length is $2^M$.
 /// - For [`f32`], this is $2^{23}$, or 8388608.
@@ -1173,7 +1224,7 @@ impl<T: PrimitiveFloat> Iterator for ExhaustivePrimitiveFloatsWithExponent<T> {
 ///
 /// # Panics
 /// Panics if the sci-exponent is less than
-/// [`MIN_EXPONENT`](super::basic::floats::PrimitiveFloat::MIN_EXPONENT)` or greater than
+/// [`MIN_EXPONENT`](super::basic::floats::PrimitiveFloat::MIN_EXPONENT) or greater than
 /// [`MAX_EXPONENT`](super::basic::floats::PrimitiveFloat::MAX_EXPONENT).
 ///
 /// # Examples
@@ -1200,8 +1251,14 @@ impl<T: PrimitiveFloat> Iterator for ExhaustivePrimitiveFloatsWithExponent<T> {
 ///     29.0, 31.0, 16.5, 17.5, 18.5, 19.5, ...]"
 /// );
 /// assert_eq!(
-///     exhaustive_primitive_floats_with_sci_exponent::<f32>(-147).map(NiceFloat).collect_vec(),
-///     [6.0e-45, 8.0e-45, 7.0e-45, 1.0e-44].iter().copied().map(NiceFloat).collect_vec()
+///     exhaustive_primitive_floats_with_sci_exponent::<f32>(-147)
+///         .map(NiceFloat)
+///         .collect_vec(),
+///     [6.0e-45, 8.0e-45, 7.0e-45, 1.0e-44]
+///         .iter()
+///         .copied()
+///         .map(NiceFloat)
+///         .collect_vec()
 /// );
 /// ```
 #[inline]
@@ -1293,7 +1350,10 @@ impl<T: PrimitiveFloat> Iterator for ExhaustivePositiveFinitePrimitiveFloats<T>
 /// use malachite_base::num::float::NiceFloat;
 ///
 /// assert_eq!(
-///     prefix_to_string(exhaustive_positive_finite_primitive_floats::<f32>().map(NiceFloat), 50),
+///     prefix_to_string(
+///         exhaustive_positive_finite_primitive_floats::<f32>().map(NiceFloat),
+///         50
+///     ),
 ///     "[1.0, 2.0, 1.5, 0.5, 1.25, 3.0, 1.75, 4.0, 1.125, 2.5, 1.375, 0.75, 1.625, 3.5, 1.875, \
 ///     0.25, 1.0625, 2.25, 1.1875, 0.625, 1.3125, 2.75, 1.4375, 6.0, 1.5625, 3.25, 1.6875, 0.875, \
 ///     1.8125, 3.75, 1.9375, 8.0, 1.03125, 2.125, 1.09375, 0.5625, 1.15625, 2.375, 1.21875, 5.0, \
@@ -1345,7 +1405,10 @@ impl<T: PrimitiveFloat> Iterator for ExhaustiveNegativeFinitePrimitiveFloats<T>
 /// use malachite_base::num::float::NiceFloat;
 ///
 /// assert_eq!(
-///     prefix_to_string(exhaustive_negative_finite_primitive_floats::<f32>().map(NiceFloat), 50),
+///     prefix_to_string(
+///         exhaustive_negative_finite_primitive_floats::<f32>().map(NiceFloat),
+///         50
+///     ),
 ///     "[-1.0, -2.0, -1.5, -0.5, -1.25, -3.0, -1.75, -4.0, -1.125, -2.5, -1.375, -0.75, -1.625, \
 ///     -3.5, -1.875, -0.25, -1.0625, -2.25, -1.1875, -0.625, -1.3125, -2.75, -1.4375, -6.0, \
 ///     -1.5625, -3.25, -1.6875, -0.875, -1.8125, -3.75, -1.9375, -8.0, -1.03125, -2.125, \
@@ -1406,7 +1469,10 @@ impl<T: PrimitiveFloat> Iterator for ExhaustiveNonzeroFinitePrimitiveFloats<T> {
 /// use malachite_base::num::float::NiceFloat;
 ///
 /// assert_eq!(
-///     prefix_to_string(exhaustive_nonzero_finite_primitive_floats::<f32>().map(NiceFloat), 50),
+///     prefix_to_string(
+///         exhaustive_nonzero_finite_primitive_floats::<f32>().map(NiceFloat),
+///         50
+///     ),
 ///     "[1.0, -1.0, 2.0, -2.0, 1.5, -1.5, 0.5, -0.5, 1.25, -1.25, 3.0, -3.0, 1.75, -1.75, 4.0, \
 ///     -4.0, 1.125, -1.125, 2.5, -2.5, 1.375, -1.375, 0.75, -0.75, 1.625, -1.625, 3.5, -3.5, \
 ///     1.875, -1.875, 0.25, -0.25, 1.0625, -1.0625, 2.25, -2.25, 1.1875, -1.1875, 0.625, -0.625, \
@@ -1423,6 +1489,9 @@ pub fn exhaustive_nonzero_finite_primitive_floats<T: PrimitiveFloat>(
     }
 }
 
+pub type ExhaustiveFinitePrimitiveFloats<T> =
+    Chain<IntoIter<T>, ExhaustiveNonzeroFinitePrimitiveFloats<T>>;
+
 /// Generates all finite primitive floats.
 ///
 /// Positive and negative zero are both included.
@@ -1444,7 +1513,10 @@ pub fn exhaustive_nonzero_finite_primitive_floats<T: PrimitiveFloat>(
 /// use malachite_base::num::float::NiceFloat;
 ///
 /// assert_eq!(
-///     prefix_to_string(exhaustive_finite_primitive_floats::<f32>().map(NiceFloat), 50),
+///     prefix_to_string(
+///         exhaustive_finite_primitive_floats::<f32>().map(NiceFloat),
+///         50
+///     ),
 ///     "[0.0, -0.0, 1.0, -1.0, 2.0, -2.0, 1.5, -1.5, 0.5, -0.5, 1.25, -1.25, 3.0, -3.0, 1.75, \
 ///     -1.75, 4.0, -4.0, 1.125, -1.125, 2.5, -2.5, 1.375, -1.375, 0.75, -0.75, 1.625, -1.625, \
 ///     3.5, -3.5, 1.875, -1.875, 0.25, -0.25, 1.0625, -1.0625, 2.25, -2.25, 1.1875, -1.1875, \
@@ -1454,7 +1526,7 @@ pub fn exhaustive_nonzero_finite_primitive_floats<T: PrimitiveFloat>(
 #[inline]
 pub fn exhaustive_finite_primitive_floats<T: PrimitiveFloat>(
 ) -> Chain<IntoIter<T>, ExhaustiveNonzeroFinitePrimitiveFloats<T>> {
-    vec![T::ZERO, T::NEGATIVE_ZERO]
+    ::alloc::vec![T::ZERO, T::NEGATIVE_ZERO]
         .into_iter()
         .chain(exhaustive_nonzero_finite_primitive_floats())
 }
@@ -1467,10 +1539,8 @@ pub fn exhaustive_finite_primitive_floats<T: PrimitiveFloat>(
 /// ascending order instead, use [`positive_primitive_floats_increasing`].
 ///
 /// The output length is $2^M(2^E-1)$.
-/// - For [`f32`]
-/// , this is $2^{31}-2^{23}$, or 2139095040.
-/// - For [`f64`]
-/// , this is $2^{63}-2^{52}$, or 9218868437227405312.
+/// - For [`f32`], this is $2^{31}-2^{23}$, or 2139095040.
+/// - For [`f64`], this is $2^{63}-2^{52}$, or 9218868437227405312.
 ///
 /// # Complexity per iteration
 /// Constant time and additional memory.
@@ -1478,12 +1548,14 @@ pub fn exhaustive_finite_primitive_floats<T: PrimitiveFloat>(
 /// # Examples
 /// ```
 /// use malachite_base::iterators::prefix_to_string;
-/// use malachite_base::num::basic::floats::PrimitiveFloat;
 /// use malachite_base::num::exhaustive::exhaustive_positive_primitive_floats;
 /// use malachite_base::num::float::NiceFloat;
 ///
 /// assert_eq!(
-///     prefix_to_string(exhaustive_positive_primitive_floats::<f32>().map(NiceFloat), 50),
+///     prefix_to_string(
+///         exhaustive_positive_primitive_floats::<f32>().map(NiceFloat),
+///         50
+///     ),
 ///     "[Infinity, 1.0, 2.0, 1.5, 0.5, 1.25, 3.0, 1.75, 4.0, 1.125, 2.5, 1.375, 0.75, 1.625, \
 ///     3.5, 1.875, 0.25, 1.0625, 2.25, 1.1875, 0.625, 1.3125, 2.75, 1.4375, 6.0, 1.5625, 3.25, \
 ///     1.6875, 0.875, 1.8125, 3.75, 1.9375, 8.0, 1.03125, 2.125, 1.09375, 0.5625, 1.15625, \
@@ -1514,12 +1586,14 @@ pub fn exhaustive_positive_primitive_floats<T: PrimitiveFloat>(
 /// # Examples
 /// ```
 /// use malachite_base::iterators::prefix_to_string;
-/// use malachite_base::num::basic::floats::PrimitiveFloat;
 /// use malachite_base::num::exhaustive::exhaustive_negative_primitive_floats;
 /// use malachite_base::num::float::NiceFloat;
 ///
 /// assert_eq!(
-///     prefix_to_string(exhaustive_negative_primitive_floats::<f32>().map(NiceFloat), 50),
+///     prefix_to_string(
+///         exhaustive_negative_primitive_floats::<f32>().map(NiceFloat),
+///         50
+///     ),
 ///     "[-Infinity, -1.0, -2.0, -1.5, -0.5, -1.25, -3.0, -1.75, -4.0, -1.125, -2.5, -1.375, \
 ///     -0.75, -1.625, -3.5, -1.875, -0.25, -1.0625, -2.25, -1.1875, -0.625, -1.3125, -2.75, \
 ///     -1.4375, -6.0, -1.5625, -3.25, -1.6875, -0.875, -1.8125, -3.75, -1.9375, -8.0, -1.03125, \
@@ -1550,12 +1624,14 @@ pub fn exhaustive_negative_primitive_floats<T: PrimitiveFloat>(
 /// # Examples
 /// ```
 /// use malachite_base::iterators::prefix_to_string;
-/// use malachite_base::num::basic::floats::PrimitiveFloat;
 /// use malachite_base::num::exhaustive::exhaustive_nonzero_primitive_floats;
 /// use malachite_base::num::float::NiceFloat;
 ///
 /// assert_eq!(
-///     prefix_to_string(exhaustive_nonzero_primitive_floats::<f32>().map(NiceFloat), 50),
+///     prefix_to_string(
+///         exhaustive_nonzero_primitive_floats::<f32>().map(NiceFloat),
+///         50
+///     ),
 ///     "[Infinity, -Infinity, 1.0, -1.0, 2.0, -2.0, 1.5, -1.5, 0.5, -0.5, 1.25, -1.25, 3.0, \
 ///     -3.0, 1.75, -1.75, 4.0, -4.0, 1.125, -1.125, 2.5, -2.5, 1.375, -1.375, 0.75, -0.75, \
 ///     1.625, -1.625, 3.5, -3.5, 1.875, -1.875, 0.25, -0.25, 1.0625, -1.0625, 2.25, -2.25, \
@@ -1566,7 +1642,7 @@ pub fn exhaustive_negative_primitive_floats<T: PrimitiveFloat>(
 #[inline]
 pub fn exhaustive_nonzero_primitive_floats<T: PrimitiveFloat>(
 ) -> Chain<IntoIter<T>, ExhaustiveNonzeroFinitePrimitiveFloats<T>> {
-    vec![T::INFINITY, T::NEGATIVE_INFINITY]
+    ::alloc::vec![T::INFINITY, T::NEGATIVE_INFINITY]
         .into_iter()
         .chain(exhaustive_nonzero_finite_primitive_floats())
 }
@@ -1588,7 +1664,6 @@ pub fn exhaustive_nonzero_primitive_floats<T: PrimitiveFloat>(
 /// # Examples
 /// ```
 /// use malachite_base::iterators::prefix_to_string;
-/// use malachite_base::num::basic::floats::PrimitiveFloat;
 /// use malachite_base::num::exhaustive::exhaustive_primitive_floats;
 /// use malachite_base::num::float::NiceFloat;
 ///
@@ -1603,7 +1678,7 @@ pub fn exhaustive_nonzero_primitive_floats<T: PrimitiveFloat>(
 #[inline]
 pub fn exhaustive_primitive_floats<T: PrimitiveFloat>(
 ) -> Chain<IntoIter<T>, ExhaustiveNonzeroFinitePrimitiveFloats<T>> {
-    vec![T::NAN, T::INFINITY, T::NEGATIVE_INFINITY, T::ZERO, T::NEGATIVE_ZERO]
+    ::alloc::vec![T::NAN, T::INFINITY, T::NEGATIVE_INFINITY, T::ZERO, T::NEGATIVE_ZERO]
         .into_iter()
         .chain(exhaustive_nonzero_finite_primitive_floats())
 }
@@ -1649,14 +1724,14 @@ pub_test! {exhaustive_primitive_floats_with_sci_exponent_and_precision_in_range<
     }
     let trailing_zeros = max_precision - precision;
     let increment = u64::power_of_2(trailing_zeros + 1);
-    let mut start_mantissa = am.round_to_multiple_of_power_of_2(trailing_zeros, RoundingMode::Up).0;
+    let mut start_mantissa = am.round_to_multiple_of_power_of_2(trailing_zeros, Up).0;
     if !start_mantissa.get_bit(trailing_zeros) {
         start_mantissa.set_bit(trailing_zeros);
     }
     if start_mantissa > bm {
         return ConstantPrecisionPrimitiveFloats::default();
     }
-    let mut end_mantissa = bm.round_to_multiple_of_power_of_2(trailing_zeros, RoundingMode::Down).0;
+    let mut end_mantissa = bm.round_to_multiple_of_power_of_2(trailing_zeros, Down).0;
     if !end_mantissa.get_bit(trailing_zeros) {
         let adjust = u64::power_of_2(trailing_zeros);
         if adjust > end_mantissa {
@@ -1982,7 +2057,7 @@ impl<T: PrimitiveFloat> Iterator for ExhaustivePrimitiveFloatInclusiveRange<T> {
 /// assert_eq!(
 ///     prefix_to_string(
 ///         exhaustive_primitive_float_range::<f32>(core::f32::consts::E, core::f32::consts::PI)
-///                 .map(NiceFloat),
+///             .map(NiceFloat),
 ///         50
 ///     ),
 ///     "[3.0, 2.75, 2.875, 3.125, 2.8125, 2.9375, 3.0625, 2.71875, 2.78125, 2.84375, 2.90625, \
@@ -2042,7 +2117,8 @@ pub fn exhaustive_primitive_float_range<T: PrimitiveFloat>(
 ///         exhaustive_primitive_float_inclusive_range::<f32>(
 ///             core::f32::consts::E,
 ///             core::f32::consts::PI
-///         ).map(NiceFloat),
+///         )
+///         .map(NiceFloat),
 ///         50
 ///     ),
 ///     "[3.0, 2.75, 2.875, 3.125, 2.8125, 2.9375, 3.0625, 2.71875, 2.78125, 2.84375, 2.90625, \
