@@ -1,3 +1,17 @@
+// Copyright © 2024 Mikhail Hogrefe
+//
+// Uses code adopted from the GNU MP Library.
+//
+//      `primesieve.c` contributed to the GNU project by Marco Bodrato.
+//
+//      Copyright © 2010-2012, 2015, 2016 Free Software Foundation, Inc.
+//
+// This file is part of Malachite.
+//
+// Malachite is free software: you can redistribute it and/or modify it under the terms of the GNU
+// Lesser General Public License (LGPL) as published by the Free Software Foundation; either version
+// 3 of the License, or (at your option) any later version. See <https://www.gnu.org/licenses/>.
+
 use crate::num::basic::integers::PrimitiveInt;
 use crate::num::basic::unsigneds::PrimitiveUnsigned;
 use crate::num::conversion::traits::ExactFrom;
@@ -261,7 +275,7 @@ fn first_block_primesieve<T: PrimitiveUnsigned, F: Fn(&mut [T], u64) -> u64>(
     let mut i = if limbs == 0 {
         0
     } else {
-        fill_bitpattern(&mut bit_array[1..limbs + 1], 0)
+        fill_bitpattern(&mut bit_array[1..=limbs], 0)
     };
     bit_array[0] = sieve_seed;
     if (bits + 1) & T::WIDTH_MASK != 0 {
@@ -277,9 +291,9 @@ fn first_block_primesieve<T: PrimitiveUnsigned, F: Fn(&mut [T], u64) -> u64>(
         for i in i + 1.. {
             if bit_array[index] & mask == T::ZERO {
                 let mut step = id_to_n(i);
-                // lindex = n_to_bit(id_to_n(i)*id_to_n(i));
+                // lindex = n_to_bit(id_to_n(i) * id_to_n(i));
                 let mut lindex = i * (step + 1) - 1 + ((i & 1).wrapping_neg() & (i + 1));
-                // lindex = i*(step+1+(i&1))-1+(i&1);
+                // lindex = i * (step + 1 + (i & 1)) - 1 + (i & 1);
                 if lindex > bits {
                     break;
                 }
@@ -291,13 +305,13 @@ fn first_block_primesieve<T: PrimitiveUnsigned, F: Fn(&mut [T], u64) -> u64>(
                     lmask.rotate_left_assign(maskrot);
                     lindex += step;
                 }
-                // lindex = n_to_bit(id_to_n(i)*bit_to_n(i));
+                // lindex = n_to_bit(id_to_n(i) * bit_to_n(i));
                 lindex = i * (i * 3 + 6) + (i & 1);
                 lmask = T::power_of_2(lindex & T::WIDTH_MASK);
                 while lindex <= bits {
                     bit_array[usize::exact_from(lindex >> T::LOG_WIDTH)] |= lmask;
                     lmask.rotate_left_assign(maskrot);
-                    lindex += step
+                    lindex += step;
                 }
             }
             mask <<= 1;
@@ -388,20 +402,17 @@ pub_test! {limbs_count_ones<T: PrimitiveUnsigned>(xs: &[T]) -> u64 {
     xs.iter().map(|&x| CountOnes::count_ones(x)).sum()
 }}
 
-// Fills bit_array with the characteristic function of composite
-// numbers up to the parameter n. I.e. a bit set to "1" represent a
-// composite, a "0" represent a prime.
+// Fills bit_array with the characteristic function of composite numbers up to the parameter n. I.e.
+// a bit set to "1" represent a composite, a "0" represent a prime.
 //
-// The primesieve_size(n) limbs pointed to by bit_array are
-// overwritten. The returned value counts prime integers in the
-// interval [4, n]. Note that n > 4.
+// The primesieve_size(n) limbs pointed to by bit_array are overwritten. The returned value counts
+// prime integers in the interval [4, n]. Note that n > 4.
 //
-// Even numbers and multiples of 3 are excluded "a priori", only
-// numbers equivalent to +/- 1 mod 6 have their bit in the array.
+// Even numbers and multiples of 3 are excluded "a priori", only numbers equivalent to +/- 1 mod 6
+// have their bit in the array.
 //
-// Once sieved, if the bit b is ZERO it represent a prime, the
-// represented prime is bit_to_n(b), if the LSbit is bit 0, or
-// id_to_n(b), if you call "1" the first bit.
+// Once sieved, if the bit b is ZERO it represent a prime, the represented prime is bit_to_n(b), if
+// the LSbit is bit 0, or id_to_n(b), if you call "1" the first bit.
 //
 // # Worst-case complexity
 // $T(n) = O(n\log\log n)$
