@@ -1,3 +1,11 @@
+// Copyright © 2024 Mikhail Hogrefe
+//
+// This file is part of Malachite.
+//
+// Malachite is free software: you can redistribute it and/or modify it under the terms of the GNU
+// Lesser General Public License (LGPL) as published by the Free Software Foundation; either version
+// 3 of the License, or (at your option) any later version. See <https://www.gnu.org/licenses/>.
+
 use crate::bools::random::{random_bools, RandomBools};
 use crate::iterators::{
     nonzero_values, with_special_value, with_special_values, NonzeroValues, WithSpecialValue,
@@ -17,7 +25,7 @@ use crate::num::random::geometric::{
     geometric_random_unsigneds, GeometricRandomNaturalValues, GeometricRandomSignedRange,
 };
 use crate::random::{Seed, EXAMPLE_SEED};
-use crate::rounding_modes::RoundingMode;
+use crate::rounding_modes::RoundingMode::*;
 use crate::vecs::{random_values_from_vec, RandomValuesFromVec};
 use itertools::Itertools;
 use rand::Rng;
@@ -166,8 +174,7 @@ impl<T: PrimitiveUnsigned> Iterator for RandomUnsignedRange<T> {
 
 /// Uniformly generates random unsigned integers in the closed interval $[a, b]$.
 ///
-/// This `struct` is created by [`random_unsigned_inclusive_range`]; see its documentation for
-/// more.
+/// This `struct` is created by [`random_unsigned_inclusive_range`]; see its documentation for more.
 #[derive(Clone, Debug)]
 pub enum RandomUnsignedInclusiveRange<T: PrimitiveUnsigned> {
     NotAll(RandomUnsignedsLessThan<T>, T),
@@ -619,9 +626,7 @@ pub fn random_unsigned_range<T: PrimitiveUnsigned>(
     a: T,
     b: T,
 ) -> RandomUnsignedRange<T> {
-    if a >= b {
-        panic!("a must be less than b. a: {a}, b: {b}");
-    }
+    assert!(a < b, "a must be less than b. a: {a}, b: {b}");
     RandomUnsignedRange {
         xs: random_unsigneds_less_than(seed, b - a),
         a,
@@ -654,7 +659,10 @@ pub fn random_unsigned_range<T: PrimitiveUnsigned>(
 /// use malachite_base::random::EXAMPLE_SEED;
 ///
 /// assert_eq!(
-///     prefix_to_string(random_unsigned_inclusive_range::<u8>(EXAMPLE_SEED, 10, 19), 10),
+///     prefix_to_string(
+///         random_unsigned_inclusive_range::<u8>(EXAMPLE_SEED, 10, 19),
+///         10
+///     ),
 ///     "[11, 17, 15, 14, 16, 14, 12, 18, 11, 17, ...]"
 /// )
 /// ```
@@ -663,9 +671,7 @@ pub fn random_unsigned_inclusive_range<T: PrimitiveUnsigned>(
     a: T,
     b: T,
 ) -> RandomUnsignedInclusiveRange<T> {
-    if a > b {
-        panic!("a must be less than or equal to b. a: {a}, b: {b}");
-    }
+    assert!(a <= b, "a must be less than or equal to b. a: {a}, b: {b}");
     if a == T::ZERO && b == T::MAX {
         RandomUnsignedInclusiveRange::All(random_primitive_ints(seed))
     } else {
@@ -706,9 +712,7 @@ pub fn random_unsigned_inclusive_range<T: PrimitiveUnsigned>(
 /// ```
 #[inline]
 pub fn random_signed_range<T: PrimitiveSigned>(seed: Seed, a: T, b: T) -> RandomSignedRange<T> {
-    if a >= b {
-        panic!("a must be less than b. a: {a}, b: {b}");
-    }
+    assert!(a < b, "a must be less than b. a: {a}, b: {b}");
     RandomSignedRange {
         xs: T::new_unsigned_range(seed, a, b),
     }
@@ -740,7 +744,10 @@ pub fn random_signed_range<T: PrimitiveSigned>(seed: Seed, a: T, b: T) -> Random
 /// use malachite_base::random::EXAMPLE_SEED;
 ///
 /// assert_eq!(
-///     prefix_to_string(random_signed_inclusive_range::<i8>(EXAMPLE_SEED, -100, 99), 10),
+///     prefix_to_string(
+///         random_signed_inclusive_range::<i8>(EXAMPLE_SEED, -100, 99),
+///         10
+///     ),
 ///     "[13, -31, 8, 68, 61, -13, -68, 10, -17, 88, ...]"
 /// )
 /// ```
@@ -750,9 +757,7 @@ pub fn random_signed_inclusive_range<T: PrimitiveSigned>(
     a: T,
     b: T,
 ) -> RandomSignedInclusiveRange<T> {
-    if a > b {
-        panic!("a must be less than or equal to b. a: {a}, b: {b}");
-    }
+    assert!(a <= b, "a must be less than or equal to b. a: {a}, b: {b}");
     RandomSignedInclusiveRange {
         xs: T::new_unsigned_inclusive_range(seed, a, b),
     }
@@ -922,8 +927,8 @@ impl<T: PrimitiveFloat> Iterator for RandomPrimitiveFloatInclusiveRange<T> {
 ///
 /// `NaN` is never generated.
 ///
-/// $a$ must be less than $b$. This function cannot create a range that includes
-/// `T::INFINITY`; for that, use [`random_primitive_float_inclusive_range`].
+/// $a$ must be less than $b$. This function cannot create a range that includes `T::INFINITY`; for
+/// that, use [`random_primitive_float_inclusive_range`].
 ///
 /// The output length is infinite.
 ///
@@ -957,13 +962,12 @@ pub fn random_primitive_float_range<T: PrimitiveFloat>(
 ) -> RandomPrimitiveFloatRange<T> {
     assert!(!a.is_nan());
     assert!(!b.is_nan());
-    if NiceFloat(a) >= NiceFloat(b) {
-        panic!(
-            "a must be less than b. a: {}, b: {}",
-            NiceFloat(a),
-            NiceFloat(b)
-        );
-    }
+    assert!(
+        NiceFloat(a) < NiceFloat(b),
+        "a must be less than b. a: {}, b: {}",
+        NiceFloat(a),
+        NiceFloat(b)
+    );
     RandomPrimitiveFloatRange {
         phantom: PhantomData,
         xs: random_unsigned_range(
@@ -1020,13 +1024,12 @@ pub fn random_primitive_float_inclusive_range<T: PrimitiveFloat>(
 ) -> RandomPrimitiveFloatInclusiveRange<T> {
     assert!(!a.is_nan());
     assert!(!b.is_nan());
-    if NiceFloat(a) > NiceFloat(b) {
-        panic!(
-            "a must be less than or equal to b. a: {}, b: {}",
-            NiceFloat(a),
-            NiceFloat(b)
-        );
-    }
+    assert!(
+        NiceFloat(a) <= NiceFloat(b),
+        "a must be less than or equal to b. a: {}, b: {}",
+        NiceFloat(a),
+        NiceFloat(b)
+    );
     RandomPrimitiveFloatInclusiveRange {
         phantom: PhantomData,
         xs: random_unsigned_inclusive_range(
@@ -1170,7 +1173,10 @@ pub fn random_nonzero_finite_primitive_floats<T: PrimitiveFloat>(
 /// use malachite_base::random::EXAMPLE_SEED;
 ///
 /// assert_eq!(
-///     prefix_to_string(random_finite_primitive_floats::<f32>(EXAMPLE_SEED).map(NiceFloat), 10),
+///     prefix_to_string(
+///         random_finite_primitive_floats::<f32>(EXAMPLE_SEED).map(NiceFloat),
+///         10
+///     ),
 ///     "[-2.3484663e-27, 2.287989e-18, -2.0729893e-12, 3.360012e28, -9.021723e-32, 3564911.2, \
 ///     -0.0000133769445, -1.8855448e18, 8.2494555e-29, 2.2178014e-38, ...]"
 /// );
@@ -1204,7 +1210,10 @@ pub fn random_finite_primitive_floats<T: PrimitiveFloat>(
 /// use malachite_base::random::EXAMPLE_SEED;
 ///
 /// assert_eq!(
-///     prefix_to_string(random_positive_primitive_floats::<f32>(EXAMPLE_SEED).map(NiceFloat), 10),
+///     prefix_to_string(
+///         random_positive_primitive_floats::<f32>(EXAMPLE_SEED).map(NiceFloat),
+///         10
+///     ),
 ///     "[9.5715654e26, 209.6476, 386935780.0, 7.965817e30, 0.00021030706, 0.0027270128, \
 ///     3.4398167e-34, 2.3397111e14, 44567765000.0, 2.3479653e21, ...]"
 /// );
@@ -1220,8 +1229,8 @@ pub fn random_positive_primitive_floats<T: PrimitiveFloat>(
 ///
 /// Every float within the range has an equal probability of being chosen. This does not mean that
 /// the distribution approximates a uniform distribution over the reals. For example, a float in
-/// $(-1/2, -1/4]$ is as likely to be chosen as a float in $(-2, -1]$, since these subranges
-/// contain an equal number of floats.
+/// $(-1/2, -1/4]$ is as likely to be chosen as a float in $(-2, -1]$, since these subranges contain
+/// an equal number of floats.
 ///
 /// Negative zero is generated; positive zero is not. `NaN` is not generated either.
 ///
@@ -1238,7 +1247,10 @@ pub fn random_positive_primitive_floats<T: PrimitiveFloat>(
 /// use malachite_base::random::EXAMPLE_SEED;
 ///
 /// assert_eq!(
-///     prefix_to_string(random_negative_primitive_floats::<f32>(EXAMPLE_SEED).map(NiceFloat), 10),
+///     prefix_to_string(
+///         random_negative_primitive_floats::<f32>(EXAMPLE_SEED).map(NiceFloat),
+///         10
+///     ),
 ///     "[-2.3484665e-27, -0.010641627, -5.8060587e-9, -2.8182444e-31, -10462.533, -821.13, \
 ///     -6.3031636e33, -9.5037605e-15, -4.956113e-11, -8.565164e-22, ...]"
 /// );
@@ -1272,7 +1284,10 @@ pub fn random_negative_primitive_floats<T: PrimitiveFloat>(
 /// use malachite_base::random::EXAMPLE_SEED;
 ///
 /// assert_eq!(
-///     prefix_to_string(random_nonzero_primitive_floats::<f32>(EXAMPLE_SEED).map(NiceFloat), 10),
+///     prefix_to_string(
+///         random_nonzero_primitive_floats::<f32>(EXAMPLE_SEED).map(NiceFloat),
+///         10
+///     ),
 ///     "[-2.3484665e-27, 2.2879888e-18, -2.0729896e-12, 3.3600117e28, -9.0217234e-32, 3564911.0, \
 ///     -0.000013376945, -1.885545e18, 8.249455e-29, 2.2178013e-38, ...]",
 /// );
@@ -1330,7 +1345,10 @@ impl<T: PrimitiveFloat> Iterator for RandomPrimitiveFloats<T> {
 /// use malachite_base::random::EXAMPLE_SEED;
 ///
 /// assert_eq!(
-///     prefix_to_string(random_primitive_floats::<f32>(EXAMPLE_SEED).map(NiceFloat), 10),
+///     prefix_to_string(
+///         random_primitive_floats::<f32>(EXAMPLE_SEED).map(NiceFloat),
+///         10
+///     ),
 ///     "[-2.3484665e-27, 2.2879888e-18, -2.0729896e-12, 3.3600117e28, -9.0217234e-32, 3564911.0, \
 ///     -0.000013376945, -1.885545e18, 8.249455e-29, 2.2178013e-38, ...]"
 /// );
@@ -1402,16 +1420,16 @@ impl<T: PrimitiveFloat> Iterator for SpecialRandomPositiveFiniteFloats<T> {
 /// and denominators of their means.
 ///
 /// But note that the specified means are only approximate, since the distributions we are sampling
-/// are truncated geometric, and their exact means are somewhat annoying to deal with. The
-/// practical implications are that
+/// are truncated geometric, and their exact means are somewhat annoying to deal with. The practical
+/// implications are that
 /// - The actual means are slightly lower than the specified means.
 /// - However, increasing the specified means increases the actual means, so this still works as a
 ///   mechanism for controlling the sci-exponent and precision.
 /// - The specified sci-exponent mean must be greater than 0 and the precision mean greater than 2,
 ///   but they may be as high as you like.
 ///
-/// Positive zero is generated; negative zero is not. `NaN` is not generated either.
-/// TODO: don't generate any zeros!
+/// Positive zero is generated; negative zero is not. `NaN` is not generated either. TODO: don't
+/// generate any zeros!
 ///
 /// The output length is infinite.
 ///
@@ -1428,7 +1446,7 @@ impl<T: PrimitiveFloat> Iterator for SpecialRandomPositiveFiniteFloats<T> {
 /// assert_eq!(
 ///     prefix_to_string(
 ///         special_random_positive_finite_primitive_floats::<f32>(EXAMPLE_SEED, 10, 1, 10, 1)
-///                 .map(NiceFloat),
+///             .map(NiceFloat),
 ///         20
 ///     ),
 ///     "[0.80126953, 0.0000013709068, 0.015609741, 0.98552704, 65536.0, 0.008257866, \
@@ -1455,7 +1473,7 @@ pub fn special_random_positive_finite_primitive_floats<T: PrimitiveFloat>(
             mean_sci_exponent_denominator,
         ),
         range_map: HashMap::new(),
-        ranges: variable_range_generator(seed.fork("ranges")),
+        ranges: VariableRangeGenerator::new(seed.fork("ranges")),
         mean_precision_n: mean_precision_numerator,
         mean_precision_d: mean_precision_denominator,
         phantom: PhantomData,
@@ -1487,8 +1505,8 @@ impl<T: PrimitiveFloat> Iterator for SpecialRandomNegativeFiniteFloats<T> {
 /// and denominators of their means.
 ///
 /// But note that the specified means are only approximate, since the distributions we are sampling
-/// are truncated geometric, and their exact means are somewhat annoying to deal with. The
-/// practical implications are that
+/// are truncated geometric, and their exact means are somewhat annoying to deal with. The practical
+/// implications are that
 /// - The actual means are slightly lower than the specified means.
 /// - However, increasing the specified means increases the actual means, so this still works as a
 ///   mechanism for controlling the sci-exponent and precision.
@@ -1512,7 +1530,7 @@ impl<T: PrimitiveFloat> Iterator for SpecialRandomNegativeFiniteFloats<T> {
 /// assert_eq!(
 ///     prefix_to_string(
 ///         special_random_negative_finite_primitive_floats::<f32>(EXAMPLE_SEED, 10, 1, 10, 1)
-///                 .map(NiceFloat),
+///             .map(NiceFloat),
 ///         20
 ///     ),
 ///     "[-0.80126953, -0.0000013709068, -0.015609741, -0.98552704, -65536.0, -0.008257866, \
@@ -1564,8 +1582,8 @@ impl<T: PrimitiveFloat> Iterator for SpecialRandomNonzeroFiniteFloats<T> {
 /// and denominators of their means.
 ///
 /// But note that the specified means are only approximate, since the distributions we are sampling
-/// are truncated geometric, and their exact means are somewhat annoying to deal with. The
-/// practical implications are that
+/// are truncated geometric, and their exact means are somewhat annoying to deal with. The practical
+/// implications are that
 /// - The actual means are slightly lower than the specified means.
 /// - However, increasing the specified means increases the actual means, so this still works as a
 ///   mechanism for controlling the sci-exponent and precision.
@@ -1589,7 +1607,7 @@ impl<T: PrimitiveFloat> Iterator for SpecialRandomNonzeroFiniteFloats<T> {
 /// assert_eq!(
 ///     prefix_to_string(
 ///         special_random_nonzero_finite_primitive_floats::<f32>(EXAMPLE_SEED, 10, 1, 10, 1)
-///                 .map(NiceFloat),
+///             .map(NiceFloat),
 ///         20
 ///     ),
 ///     "[-0.6328125, -9.536743e-7, -0.013671875, 0.6875, -70208.0, 0.01550293, -0.028625488, \
@@ -1625,8 +1643,8 @@ pub fn special_random_nonzero_finite_primitive_floats<T: PrimitiveFloat>(
 /// numerators and denominators of their means of the nonzero floats.
 ///
 /// But note that the specified means are only approximate, since the distributions we are sampling
-/// are truncated geometric, and their exact means are somewhat annoying to deal with. The
-/// practical implications are that
+/// are truncated geometric, and their exact means are somewhat annoying to deal with. The practical
+/// implications are that
 /// - The actual means are slightly lower than the specified means.
 /// - However, increasing the specified means increases the actual means, so this still works as a
 ///   mechanism for controlling the sci-exponent and precision.
@@ -1643,7 +1661,6 @@ pub fn special_random_nonzero_finite_primitive_floats<T: PrimitiveFloat>(
 /// # Examples
 /// ```
 /// use malachite_base::iterators::prefix_to_string;
-/// use malachite_base::num::basic::floats::PrimitiveFloat;
 /// use malachite_base::num::float::NiceFloat;
 /// use malachite_base::num::random::special_random_finite_primitive_floats;
 /// use malachite_base::random::EXAMPLE_SEED;
@@ -1651,7 +1668,7 @@ pub fn special_random_nonzero_finite_primitive_floats<T: PrimitiveFloat>(
 /// assert_eq!(
 ///     prefix_to_string(
 ///         special_random_finite_primitive_floats::<f32>(EXAMPLE_SEED, 10, 1, 10, 1, 1, 10)
-///                 .map(NiceFloat),
+///             .map(NiceFloat),
 ///         20
 ///     ),
 ///     "[0.65625, 0.0000014255784, 0.013183594, 0.0, -0.8125, -74240.0, -0.0078125, -0.03060913, \
@@ -1694,8 +1711,8 @@ pub fn special_random_finite_primitive_floats<T: PrimitiveFloat>(
 /// the numerators and denominators of their means of the finite floats.
 ///
 /// But note that the specified means are only approximate, since the distributions we are sampling
-/// are truncated geometric, and their exact means are somewhat annoying to deal with. The
-/// practical implications are that
+/// are truncated geometric, and their exact means are somewhat annoying to deal with. The practical
+/// implications are that
 /// - The actual means are slightly lower than the specified means.
 /// - However, increasing the specified means increases the actual means, so this still works as a
 ///   mechanism for controlling the sci-exponent and precision.
@@ -1712,7 +1729,6 @@ pub fn special_random_finite_primitive_floats<T: PrimitiveFloat>(
 /// # Examples
 /// ```
 /// use malachite_base::iterators::prefix_to_string;
-/// use malachite_base::num::basic::floats::PrimitiveFloat;
 /// use malachite_base::num::float::NiceFloat;
 /// use malachite_base::num::random::special_random_positive_primitive_floats;
 /// use malachite_base::random::EXAMPLE_SEED;
@@ -1720,7 +1736,7 @@ pub fn special_random_finite_primitive_floats<T: PrimitiveFloat>(
 /// assert_eq!(
 ///     prefix_to_string(
 ///         special_random_positive_primitive_floats::<f32>(EXAMPLE_SEED, 10, 1, 10, 1, 1, 10)
-///                 .map(NiceFloat),
+///             .map(NiceFloat),
 ///         20
 ///     ),
 ///     "[0.6328125, 9.536743e-7, 0.013671875, Infinity, 0.6875, 70208.0, 0.01550293, \
@@ -1763,8 +1779,8 @@ pub fn special_random_positive_primitive_floats<T: PrimitiveFloat>(
 /// the numerators and denominators of their means of the finite floats.
 ///
 /// But note that the specified means are only approximate, since the distributions we are sampling
-/// are truncated geometric, and their exact means are somewhat annoying to deal with. The
-/// practical implications are that
+/// are truncated geometric, and their exact means are somewhat annoying to deal with. The practical
+/// implications are that
 /// - The actual means are slightly lower than the specified means.
 /// - However, increasing the specified means increases the actual means, so this still works as a
 ///   mechanism for controlling the sci-exponent and precision.
@@ -1781,7 +1797,6 @@ pub fn special_random_positive_primitive_floats<T: PrimitiveFloat>(
 /// # Examples
 /// ```
 /// use malachite_base::iterators::prefix_to_string;
-/// use malachite_base::num::basic::floats::PrimitiveFloat;
 /// use malachite_base::num::float::NiceFloat;
 /// use malachite_base::num::random::special_random_negative_primitive_floats;
 /// use malachite_base::random::EXAMPLE_SEED;
@@ -1789,7 +1804,7 @@ pub fn special_random_positive_primitive_floats<T: PrimitiveFloat>(
 /// assert_eq!(
 ///     prefix_to_string(
 ///         special_random_negative_primitive_floats::<f32>(EXAMPLE_SEED, 10, 1, 10, 1, 1, 10)
-///                 .map(NiceFloat),
+///             .map(NiceFloat),
 ///         20
 ///     ),
 ///     "[-0.6328125, -9.536743e-7, -0.013671875, -Infinity, -0.6875, -70208.0, -0.01550293, \
@@ -1832,8 +1847,8 @@ pub fn special_random_negative_primitive_floats<T: PrimitiveFloat>(
 /// numerators and denominators of their means of the finite floats.
 ///
 /// But note that the specified means are only approximate, since the distributions we are sampling
-/// are truncated geometric, and their exact means are somewhat annoying to deal with. The
-/// practical implications are that
+/// are truncated geometric, and their exact means are somewhat annoying to deal with. The practical
+/// implications are that
 /// - The actual means are slightly lower than the specified means.
 /// - However, increasing the specified means increases the actual means, so this still works as a
 ///   mechanism for controlling the sci-exponent and precision.
@@ -1850,7 +1865,6 @@ pub fn special_random_negative_primitive_floats<T: PrimitiveFloat>(
 /// # Examples
 /// ```
 /// use malachite_base::iterators::prefix_to_string;
-/// use malachite_base::num::basic::floats::PrimitiveFloat;
 /// use malachite_base::num::float::NiceFloat;
 /// use malachite_base::num::random::special_random_nonzero_primitive_floats;
 /// use malachite_base::random::EXAMPLE_SEED;
@@ -1858,7 +1872,7 @@ pub fn special_random_negative_primitive_floats<T: PrimitiveFloat>(
 /// assert_eq!(
 ///     prefix_to_string(
 ///         special_random_nonzero_primitive_floats::<f32>(EXAMPLE_SEED, 10, 1, 10, 1, 1, 10)
-///                 .map(NiceFloat),
+///             .map(NiceFloat),
 ///         20
 ///     ),
 ///     "[0.65625, 0.0000014255784, 0.013183594, Infinity, -0.8125, -74240.0, -0.0078125, \
@@ -1896,13 +1910,13 @@ pub fn special_random_nonzero_primitive_floats<T: PrimitiveFloat>(
 /// Generates primitive floats.
 ///
 /// Simpler floats (those with a lower absolute sci-exponent or precision) are more likely to be
-/// chosen. You can specify the numerator and denominator of the probability that zero, infinity,
-/// or NaN will be generated. You can also specify the mean absolute sci-exponent and precision by
+/// chosen. You can specify the numerator and denominator of the probability that zero, infinity, or
+/// NaN will be generated. You can also specify the mean absolute sci-exponent and precision by
 /// passing the numerators and denominators of their means of the finite floats.
 ///
 /// But note that the specified means are only approximate, since the distributions we are sampling
-/// are truncated geometric, and their exact means are somewhat annoying to deal with. The
-/// practical implications are that
+/// are truncated geometric, and their exact means are somewhat annoying to deal with. The practical
+/// implications are that
 /// - The actual means are slightly lower than the specified means.
 /// - However, increasing the specified means increases the actual means, so this still works as a
 ///   mechanism for controlling the sci-exponent and precision.
@@ -1917,7 +1931,6 @@ pub fn special_random_nonzero_primitive_floats<T: PrimitiveFloat>(
 /// # Examples
 /// ```
 /// use malachite_base::iterators::prefix_to_string;
-/// use malachite_base::num::basic::floats::PrimitiveFloat;
 /// use malachite_base::num::float::NiceFloat;
 /// use malachite_base::num::random::special_random_primitive_floats;
 /// use malachite_base::random::EXAMPLE_SEED;
@@ -1925,7 +1938,7 @@ pub fn special_random_nonzero_primitive_floats<T: PrimitiveFloat>(
 /// assert_eq!(
 ///     prefix_to_string(
 ///         special_random_primitive_floats::<f32>(EXAMPLE_SEED, 10, 1, 10, 1, 1, 10)
-///                 .map(NiceFloat),
+///             .map(NiceFloat),
 ///         20
 ///     ),
 ///     "[0.65625, 0.0000014255784, 0.013183594, Infinity, -0.8125, -74240.0, -0.0078125, \
@@ -1978,11 +1991,11 @@ fn mantissas_inclusive<T: PrimitiveFloat>(
         bm.set_bit(T::MANTISSA_WIDTH);
         T::MANTISSA_WIDTH + 1 - precision
     };
-    let mut lo = am.shr_round(p, RoundingMode::Up).0;
+    let mut lo = am.shr_round(p, Up).0;
     if lo.even() {
         lo += 1;
     }
-    let mut hi = bm.shr_round(p, RoundingMode::Down).0;
+    let mut hi = bm.shr_round(p, Down).0;
     if hi == 0 {
         return None;
     } else if hi.even() {
@@ -2097,7 +2110,7 @@ fn special_random_positive_finite_float_inclusive_range<T: PrimitiveFloat>(
             mean_precision_numerator,
             mean_precision_denominator,
         ),
-        ranges: variable_range_generator(seed.fork("ranges")),
+        ranges: VariableRangeGenerator::new(seed.fork("ranges")),
     }
 }
 
@@ -2228,10 +2241,10 @@ impl<T: PrimitiveFloat> Iterator for SpecialRandomFloatInclusiveRange<T> {
 /// Generates random primitive floats in the half-open interval $[a, b)$.
 ///
 /// Simpler floats (those with a lower absolute sci-exponent or precision) are more likely to be
-/// chosen. You can specify the numerator and denominator of the probability that any special
-/// values (positive or negative zero or infinity) are generated, provided that they are in the
-/// range. You can also specify the mean absolute sci-exponent and precision by passing the
-/// numerators and denominators of their means of the finite floats.
+/// chosen. You can specify the numerator and denominator of the probability that any special values
+/// (positive or negative zero or infinity) are generated, provided that they are in the range. You
+/// can also specify the mean absolute sci-exponent and precision by passing the numerators and
+/// denominators of their means of the finite floats.
 ///
 /// But note that the means are only approximate, since the distributions we are sampling are
 /// truncated geometric, and their exact means are somewhat annoying to deal with. The practical
@@ -2239,13 +2252,13 @@ impl<T: PrimitiveFloat> Iterator for SpecialRandomFloatInclusiveRange<T> {
 /// - The actual mean is lower than the specified means.
 /// - However, increasing the approximate mean increases the actual means, so this still works as a
 ///   mechanism for controlling the sci-exponent and precision.
-/// - The specified sci-exponent mean must be greater the smallest absolute of any sci-exponent of
-///   a float in the range, and the precision mean greater than 2, but they may be as high as you
-/// like.
+/// - The specified sci-exponent mean must be greater the smallest absolute of any sci-exponent of a
+///   float in the range, and the precision mean greater than 2, but they may be as high as you
+///   like.
 ///
 /// But note that the specified means are only approximate, since the distributions we are sampling
-/// are truncated geometric, and their exact means are somewhat annoying to deal with. The
-/// practical implications are that
+/// are truncated geometric, and their exact means are somewhat annoying to deal with. The practical
+/// implications are that
 /// - The actual means are slightly lower than the specified means.
 /// - However, increasing the specified means increases the actual means, so this still works as a
 ///   mechanism for controlling the sci-exponent and precision.
@@ -2269,7 +2282,6 @@ impl<T: PrimitiveFloat> Iterator for SpecialRandomFloatInclusiveRange<T> {
 /// # Examples
 /// ```
 /// use malachite_base::iterators::prefix_to_string;
-/// use malachite_base::num::basic::floats::PrimitiveFloat;
 /// use malachite_base::num::float::NiceFloat;
 /// use malachite_base::num::random::special_random_primitive_float_range;
 /// use malachite_base::random::EXAMPLE_SEED;
@@ -2286,7 +2298,8 @@ impl<T: PrimitiveFloat> Iterator for SpecialRandomFloatInclusiveRange<T> {
 ///             1,
 ///             1,
 ///             100
-///         ).map(NiceFloat),
+///         )
+///         .map(NiceFloat),
 ///         20
 ///     ),
 ///     "[2.9238281, 2.953125, 3.0, 2.8671875, 2.8125, 3.125, 3.015625, 2.8462658, 3.140625, \
@@ -2323,14 +2336,14 @@ pub fn special_random_primitive_float_range<T: PrimitiveFloat>(
 /// Generates random primitive floats in the closed interval $[a, b]$.
 ///
 /// Simpler floats (those with a lower absolute sci-exponent or precision) are more likely to be
-/// chosen. You can specify the numerator and denominator of the probability that any special
-/// values (positive or negative zero or infinity) are generated, provided that they are in the
-/// range. You can also specify the mean absolute sci-exponent and precision by passing the
-/// numerators and denominators of their means of the finite floats.
+/// chosen. You can specify the numerator and denominator of the probability that any special values
+/// (positive or negative zero or infinity) are generated, provided that they are in the range. You
+/// can also specify the mean absolute sci-exponent and precision by passing the numerators and
+/// denominators of their means of the finite floats.
 ///
 /// But note that the specified means are only approximate, since the distributions we are sampling
-/// are truncated geometric, and their exact means are somewhat annoying to deal with. The
-/// practical implications are that
+/// are truncated geometric, and their exact means are somewhat annoying to deal with. The practical
+/// implications are that
 /// - The actual means are slightly lower than the specified means.
 /// - However, increasing the specified means increases the actual means, so this still works as a
 ///   mechanism for controlling the sci-exponent and precision.
@@ -2354,7 +2367,6 @@ pub fn special_random_primitive_float_range<T: PrimitiveFloat>(
 /// # Examples
 /// ```
 /// use malachite_base::iterators::prefix_to_string;
-/// use malachite_base::num::basic::floats::PrimitiveFloat;
 /// use malachite_base::num::float::NiceFloat;
 /// use malachite_base::num::random::special_random_primitive_float_inclusive_range;
 /// use malachite_base::random::EXAMPLE_SEED;
@@ -2371,7 +2383,8 @@ pub fn special_random_primitive_float_range<T: PrimitiveFloat>(
 ///             1,
 ///             1,
 ///             100
-///         ).map(NiceFloat),
+///         )
+///         .map(NiceFloat),
 ///         20
 ///     ),
 ///     "[2.9238281, 2.953125, 3.0, 2.8671875, 2.8125, 3.125, 3.015625, 2.8462658, 3.140625, \
@@ -2457,7 +2470,7 @@ pub fn special_random_primitive_float_inclusive_range<T: PrimitiveFloat>(
 /// Generates unsigneds sampled from ranges. A single generator can sample from different ranges of
 /// different types.
 ///
-/// This `struct` is created by [`variable_range_generator`]; see its documentation for more.
+/// This `struct` is created by [`VariableRangeGenerator::new`]; see its documentation for more.
 #[derive(Clone, Debug)]
 pub struct VariableRangeGenerator {
     xs: RandomPrimitiveInts<u32>,
@@ -2467,6 +2480,64 @@ pub struct VariableRangeGenerator {
 }
 
 impl VariableRangeGenerator {
+    /// Generates unsigneds sampled from ranges. A single generator can sample from different ranges
+    /// of different types.
+    ///
+    /// If you only need to generate values from a single range, it is slightly more efficient to
+    /// use [`random_unsigned_bit_chunks`], [`random_unsigneds_less_than`],
+    /// [`random_unsigned_range`], or [`random_unsigned_inclusive_range`].
+    ///
+    /// # Worst-case complexity
+    /// Constant time and additional memory.
+    ///
+    /// # Examples
+    /// ```
+    /// use malachite_base::num::random::VariableRangeGenerator;
+    /// use malachite_base::random::EXAMPLE_SEED;
+    ///
+    /// let mut generator = VariableRangeGenerator::new(EXAMPLE_SEED);
+    /// assert_eq!(generator.next_bit_chunk::<u16>(10), 881);
+    /// assert_eq!(generator.next_less_than::<u8>(100), 34);
+    /// assert_eq!(generator.next_in_range::<u32>(10, 20), 16);
+    /// assert_eq!(generator.next_in_inclusive_range::<u64>(10, 20), 14);
+    /// ```
+    pub fn new(seed: Seed) -> VariableRangeGenerator {
+        VariableRangeGenerator {
+            xs: random_primitive_ints(seed),
+            x: 0,
+            in_inner_loop: false,
+            remaining_x_bits: 0,
+        }
+    }
+
+    /// Uniformly generates a `bool`.
+    ///
+    /// $$
+    /// $P(\text{false}) = P(\text{true}) = \frac{1}{2}$.
+    /// $$
+    ///
+    /// # Worst-case complexity
+    /// Constant time and additional memory.
+    ///
+    /// # Examples
+    /// ```
+    /// use malachite_base::num::random::VariableRangeGenerator;
+    /// use malachite_base::random::EXAMPLE_SEED;
+    ///
+    /// let mut xs = Vec::with_capacity(10);
+    /// let mut generator = VariableRangeGenerator::new(EXAMPLE_SEED);
+    /// for _ in 0..10 {
+    ///     xs.push(generator.next_bool());
+    /// }
+    /// assert_eq!(
+    ///     xs,
+    ///     &[true, false, true, false, true, true, true, true, true, false]
+    /// );
+    /// ```
+    pub fn next_bool(&mut self) -> bool {
+        self.xs.next().unwrap().odd()
+    }
+
     /// Uniformly generates an unsigned integer with up to some number of bits.
     ///
     /// $$
@@ -2477,8 +2548,6 @@ impl VariableRangeGenerator {
     /// $$
     /// where $c$ is `chunk_size`.
     ///
-    /// The output length is infinite.
-    ///
     /// # Worst-case complexity
     /// Constant time and additional memory.
     ///
@@ -2487,11 +2556,11 @@ impl VariableRangeGenerator {
     ///
     /// # Examples
     /// ```
-    /// use malachite_base::num::random::variable_range_generator;
+    /// use malachite_base::num::random::VariableRangeGenerator;
     /// use malachite_base::random::EXAMPLE_SEED;
     ///
     /// let mut xs = Vec::with_capacity(10);
-    /// let mut generator = variable_range_generator(EXAMPLE_SEED);
+    /// let mut generator = VariableRangeGenerator::new(EXAMPLE_SEED);
     /// for _ in 0..10 {
     ///     xs.push(generator.next_bit_chunk::<u8>(3));
     /// }
@@ -2546,11 +2615,11 @@ impl VariableRangeGenerator {
     ///
     /// # Examples
     /// ```
-    /// use malachite_base::num::random::variable_range_generator;
+    /// use malachite_base::num::random::VariableRangeGenerator;
     /// use malachite_base::random::EXAMPLE_SEED;
     ///
     /// let mut xs = Vec::with_capacity(10);
-    /// let mut generator = variable_range_generator(EXAMPLE_SEED);
+    /// let mut generator = VariableRangeGenerator::new(EXAMPLE_SEED);
     /// for _ in 0..10 {
     ///     xs.push(generator.next_less_than(10u8));
     /// }
@@ -2591,11 +2660,11 @@ impl VariableRangeGenerator {
     ///
     /// # Examples
     /// ```
-    /// use malachite_base::num::random::variable_range_generator;
+    /// use malachite_base::num::random::VariableRangeGenerator;
     /// use malachite_base::random::EXAMPLE_SEED;
     ///
     /// let mut xs = Vec::with_capacity(10);
-    /// let mut generator = variable_range_generator(EXAMPLE_SEED);
+    /// let mut generator = VariableRangeGenerator::new(EXAMPLE_SEED);
     /// for _ in 0..10 {
     ///     xs.push(generator.next_in_range(10u8, 20));
     /// }
@@ -2624,11 +2693,11 @@ impl VariableRangeGenerator {
     ///
     /// # Examples
     /// ```
-    /// use malachite_base::num::random::variable_range_generator;
+    /// use malachite_base::num::random::VariableRangeGenerator;
     /// use malachite_base::random::EXAMPLE_SEED;
     ///
     /// let mut xs = Vec::with_capacity(10);
-    /// let mut generator = variable_range_generator(EXAMPLE_SEED);
+    /// let mut generator = VariableRangeGenerator::new(EXAMPLE_SEED);
     /// for _ in 0..10 {
     ///     xs.push(generator.next_in_inclusive_range(10u8, 19));
     /// }
@@ -2643,36 +2712,6 @@ impl VariableRangeGenerator {
     }
 }
 
-/// Generates unsigneds sampled from ranges. A single generator can sample from different ranges of
-/// different types.
-///
-/// If you only need to generate values from a single range, it is slightly more efficient to use
-/// [`random_unsigned_bit_chunks`], [`random_unsigneds_less_than`], [`random_unsigned_range`], or
-/// [`random_unsigned_inclusive_range`].
-///
-/// # Worst-case complexity
-/// Constant time and additional memory.
-///
-/// # Examples
-/// ```
-/// use malachite_base::num::random::variable_range_generator;
-/// use malachite_base::random::EXAMPLE_SEED;
-///
-/// let mut generator = variable_range_generator(EXAMPLE_SEED);
-/// assert_eq!(generator.next_bit_chunk::<u16>(10), 881);
-/// assert_eq!(generator.next_less_than::<u8>(100), 34);
-/// assert_eq!(generator.next_in_range::<u32>(10, 20), 16);
-/// assert_eq!(generator.next_in_inclusive_range::<u64>(10, 20), 14);
-/// ```
-pub fn variable_range_generator(seed: Seed) -> VariableRangeGenerator {
-    VariableRangeGenerator {
-        xs: random_primitive_ints(seed),
-        x: 0,
-        in_inner_loop: false,
-        remaining_x_bits: 0,
-    }
-}
-
 /// Iterators that generate primitive integers from geometric-like distributions.
 pub mod geometric;
 
@@ -2708,21 +2747,20 @@ pub mod geometric;
 /// * If $m=2$, the sequence is uniformly random. A sample sequence with $m=2$ is
 ///   $1100110001101010100101101001000001100001\ldots$.
 /// * If $m>2$, the sequence tends to have longer runs of 0s and 1s than a uniformly random
-///   sequence. A sample sequence with $m=32$ is
-///   $1111111111111111110000000011111111111111\ldots$.
+///   sequence. A sample sequence with $m=32$ is $1111111111111111110000000011111111111111\ldots$.
 ///
 /// An alternative way to generate a striped sequence is to start with 0 or 1 with equal probability
 /// and then determine the length of each block of equal bits using a geometric distribution with
 /// mean $m$. In practice, this isn't any more efficient than the naive algorithm.
 ///
 /// We can generate a random striped unsigned integer of type `T` by taking the first $W$ bits of a
-/// striped sequence. Fixing the parameter $m$ defines a distribution over `T`s. A few things can
-/// be said about the probability $P_m(n)$ of an unsigned integer $n$ of width $W$ being generated:
+/// striped sequence. Fixing the parameter $m$ defines a distribution over `T`s. A few things can be
+/// said about the probability $P_m(n)$ of an unsigned integer $n$ of width $W$ being generated:
 /// * $P_m(n) = P_m(\lnot n)$
 /// * $P_m(0) = P_m(2^W-1) = \frac{1}{2} \left ( 1-\frac{1}{m} \right )^{W-1}$. If $m>2$, this is
 ///   the maximum probability achieved; if $m<2$, the minimum.
-/// * $P_m(\lfloor 2^W/3 \rfloor) = P_m(\lfloor 2^{W+1}/3 \rfloor) = 1/(2m^{W-1})$. If $m>2$,
-///   this is the minimum probability achieved; if $m<2$, the maximum.
+/// * $P_m(\lfloor 2^W/3 \rfloor) = P_m(\lfloor 2^{W+1}/3 \rfloor) = 1/(2m^{W-1})$. If $m>2$, this
+///   is the minimum probability achieved; if $m<2$, the maximum.
 /// * Because of these distributions' symmetry, their mean is $(2^W-1)/2$ and their skewness is 0.
 ///   It's hard to say anything about their standard deviations or excess kurtoses, although these
 ///   can be computed quickly for specific values of $m$ when $W$ is 8 or 16.
@@ -2730,8 +2768,7 @@ pub mod geometric;
 /// We can similarly generate random striped signed integers of width $W$. The sign bit is chosen
 /// uniformly, and the remaining $W-1$ are taken from a striped sequence.
 ///
-/// To generate striped integers from a range, the integers are constructed one bit at a time.
-/// Some bits are forced; they must be 0 or 1 in order for the final integer to be within the
-/// specified range. If a bit is _not_ forced, it is different from the preceding bit with
-/// probability $1/m$.
+/// To generate striped integers from a range, the integers are constructed one bit at a time. Some
+/// bits are forced; they must be 0 or 1 in order for the final integer to be within the specified
+/// range. If a bit is _not_ forced, it is different from the preceding bit with probability $1/m$.
 pub mod striped;
