@@ -1,3 +1,11 @@
+// Copyright © 2024 Mikhail Hogrefe
+//
+// This file is part of Malachite.
+//
+// Malachite is free software: you can redistribute it and/or modify it under the terms of the GNU
+// Lesser General Public License (LGPL) as published by the Free Software Foundation; either version
+// 3 of the License, or (at your option) any later version. See <https://www.gnu.org/licenses/>.
+
 use crate::bools::random::{random_bools, weighted_random_bools, RandomBools, WeightedRandomBools};
 use crate::iterators::{nonzero_values, NonzeroValues};
 use crate::num::basic::signeds::PrimitiveSigned;
@@ -67,7 +75,7 @@ impl StripedBitSource {
     /// use malachite_base::num::random::striped::StripedBitSource;
     /// use malachite_base::random::EXAMPLE_SEED;
     ///
-    /// let mut bit_source = StripedBitSource::new(EXAMPLE_SEED, 4, 1);
+    /// let bit_source = StripedBitSource::new(EXAMPLE_SEED, 4, 1);
     /// let mut string = String::with_capacity(40);
     /// for bit in bit_source.take(40) {
     ///     if bit {
@@ -263,8 +271,7 @@ impl<T: PrimitiveSigned> Iterator for StripedRandomNaturalSigneds<T> {
 
 /// Generates random negative signed integers from a random striped distribution.
 ///
-/// This `struct` is created by [`striped_random_negative_signeds`]; see its documentation for
-/// more.
+/// This `struct` is created by [`striped_random_negative_signeds`]; see its documentation for more.
 #[derive(Clone, Debug)]
 pub struct StripedRandomNegativeSigneds<T: PrimitiveSigned> {
     phantom: PhantomData<*const T>,
@@ -288,37 +295,12 @@ impl<T: PrimitiveSigned> Iterator for StripedRandomNegativeSigneds<T> {
     }
 }
 
-/// Generates random unsigned integers less than a positive limit, from a random striped
-/// distribution.
-///
-/// This `struct` is created by [`striped_random_unsigneds_less_than`]; see its documentation for
-/// more.
-#[derive(Clone, Debug)]
-struct StripedRandomUnsignedsLessThan<T: PrimitiveUnsigned> {
-    pub(crate) xs: StripedRandomUnsignedBitChunks<T>,
-    pub(crate) limit: T,
-}
-
-impl<T: PrimitiveUnsigned> Iterator for StripedRandomUnsignedsLessThan<T> {
-    type Item = T;
-
-    #[inline]
-    fn next(&mut self) -> Option<T> {
-        loop {
-            let x = self.xs.next();
-            if x.unwrap() < self.limit {
-                return x;
-            }
-        }
-    }
-}
-
 /// Generates random unsigned integers from a random striped distribution.
 ///
 /// See [here](self) for more information.
 ///
-/// The mean run length (before the bit sequences are truncated) is
-/// $m$ = `m_numerator / m_denominator`.
+/// The mean run length (before the bit sequences are truncated) is $m$ = `m_numerator /
+/// m_denominator`.
 ///
 /// The output length is infinite.
 ///
@@ -360,8 +342,8 @@ pub fn striped_random_unsigneds<T: PrimitiveUnsigned>(
 ///
 /// See [here](self) for more information.
 ///
-/// The mean run length (before the bit sequences are truncated) is
-/// $m$ = `m_numerator / m_denominator`.
+/// The mean run length (before the bit sequences are truncated) is $m$ = `m_numerator /
+/// m_denominator`.
 ///
 /// The output length is infinite.
 ///
@@ -385,7 +367,7 @@ pub fn striped_random_unsigneds<T: PrimitiveUnsigned>(
 /// assert_eq!(
 ///     prefix_to_string(
 ///         striped_random_positive_unsigneds::<u8>(EXAMPLE_SEED, 4, 1)
-///                 .map(|x| x.to_binary_string()),
+///             .map(|x| x.to_binary_string()),
 ///         10
 ///     ),
 ///     "[1, 1001100, 1111111, 11000011, 10000000, 1111, 1110110, 11111000, 11111111, 11111101, \
@@ -404,8 +386,8 @@ pub fn striped_random_positive_unsigneds<T: PrimitiveUnsigned>(
 ///
 /// See [here](self) for more information.
 ///
-/// The mean run length (before the bit sequences are truncated) is
-/// $m$ = `m_numerator / m_denominator`.
+/// The mean run length (before the bit sequences are truncated) is $m$ = `m_numerator /
+/// m_denominator`.
 ///
 /// The output length is infinite.
 ///
@@ -451,8 +433,8 @@ pub fn striped_random_signeds<T: PrimitiveSigned>(
 ///
 /// See [here](self) for more information.
 ///
-/// The mean run length (before the bit sequences are truncated) is
-/// $m$ = `m_numerator / m_denominator`.
+/// The mean run length (before the bit sequences are truncated) is $m$ = `m_numerator /
+/// m_denominator`.
 ///
 /// The output length is infinite.
 ///
@@ -496,8 +478,8 @@ pub fn striped_random_natural_signeds<T: PrimitiveSigned>(
 ///
 /// See [here](self) for more information.
 ///
-/// The mean run length (before the bit sequences are truncated) is
-/// $m$ = `m_numerator / m_denominator`.
+/// The mean run length (before the bit sequences are truncated) is $m$ = `m_numerator /
+/// m_denominator`.
 ///
 /// The output length is infinite.
 ///
@@ -520,8 +502,7 @@ pub fn striped_random_natural_signeds<T: PrimitiveSigned>(
 ///
 /// assert_eq!(
 ///     prefix_to_string(
-///         striped_random_positive_signeds::<i8>(EXAMPLE_SEED, 4, 1)
-///                 .map(|x| x.to_binary_string()),
+///         striped_random_positive_signeds::<i8>(EXAMPLE_SEED, 4, 1).map(|x| x.to_binary_string()),
 ///         10
 ///     ),
 ///     "[101100, 110000, 1111100, 1111, 1111110, 111, 11101, 1100000, 1111111, 1100000, ...]"
@@ -543,8 +524,8 @@ pub fn striped_random_positive_signeds<T: PrimitiveSigned>(
 ///
 /// See [here](self) for more information.
 ///
-/// The mean run length (before the bit sequences are truncated) is
-/// $m$ = `m_numerator / m_denominator`.
+/// The mean run length (before the bit sequences are truncated) is $m$ = `m_numerator /
+/// m_denominator`.
 ///
 /// The output length is infinite.
 ///
@@ -567,8 +548,7 @@ pub fn striped_random_positive_signeds<T: PrimitiveSigned>(
 ///
 /// assert_eq!(
 ///     prefix_to_string(
-///         striped_random_negative_signeds::<i8>(EXAMPLE_SEED, 4, 1)
-///                 .map(|x| x.to_binary_string()),
+///         striped_random_negative_signeds::<i8>(EXAMPLE_SEED, 4, 1).map(|x| x.to_binary_string()),
 ///         10
 ///     ),
 ///     "[10000000, 10101100, 10110000, 11111100, 10001111, 11111110, 10000000, 10000111, \
@@ -590,8 +570,8 @@ pub fn striped_random_negative_signeds<T: PrimitiveSigned>(
 ///
 /// See [here](self) for more information.
 ///
-/// The mean run length (before the bit sequences are truncated) is
-/// $m$ = `m_numerator / m_denominator`.
+/// The mean run length (before the bit sequences are truncated) is $m$ = `m_numerator /
+/// m_denominator`.
 ///
 /// The output length is infinite.
 ///
@@ -634,8 +614,8 @@ pub fn striped_random_nonzero_signeds<T: PrimitiveSigned>(
 ///
 /// See [here](self) for more information.
 ///
-/// The mean run length (before the bit sequences are truncated) is
-/// $m$ = `m_numerator / m_denominator`.
+/// The mean run length (before the bit sequences are truncated) is $m$ = `m_numerator /
+/// m_denominator`.
 ///
 /// The output length is infinite.
 ///
@@ -660,7 +640,7 @@ pub fn striped_random_nonzero_signeds<T: PrimitiveSigned>(
 /// assert_eq!(
 ///     prefix_to_string(
 ///         striped_random_unsigned_bit_chunks::<u8>(EXAMPLE_SEED, 3, 4, 1)
-///                 .map(|x| x.to_binary_string()),
+///             .map(|x| x.to_binary_string()),
 ///         10
 ///     ),
 ///     "[0, 0, 0, 101, 11, 100, 11, 11, 0, 111, ...]"
@@ -732,19 +712,18 @@ impl<I: Iterator<Item = u64>> Iterator for StripedRandomBoolVecs<I> {
 ///
 /// See [here](self) for more information.
 ///
-/// The mean run length (before the bit sequences are truncated) is
-/// $m$ = `mean_stripe_numerator / mean_stripe_denominator`.
+/// The mean run length (before the bit sequences are truncated) is $m$ = `mean_stripe_numerator /
+/// mean_stripe_denominator`.
 ///
 /// # Panics
-/// Panics if `mean_stripe_denominator` is zero or if
-/// `mean_stripe_numerator <= mean_stripe_denominator`.
+/// Panics if `mean_stripe_denominator` is zero or if `mean_stripe_numerator <=
+/// mean_stripe_denominator`.
 ///
 /// # Examples
 /// ```
 /// use malachite_base::iterators::prefix_to_string;
 /// use malachite_base::num::random::striped::striped_random_bool_vecs_from_length_iterator;
 /// use malachite_base::random::EXAMPLE_SEED;
-/// use malachite_base::strings::ToBinaryString;
 /// use malachite_base::vecs::random_values_from_vec;
 ///
 /// assert_eq!(
@@ -754,7 +733,11 @@ impl<I: Iterator<Item = u64>> Iterator for StripedRandomBoolVecs<I> {
 ///             &|seed| random_values_from_vec(seed, vec![0, 2, 4]),
 ///             10,
 ///             1,
-///         ).map(|bs| bs.into_iter().map(|b| if b { '1' } else { '0' }).collect::<String>()),
+///         )
+///         .map(|bs| bs
+///             .into_iter()
+///             .map(|b| if b { '1' } else { '0' })
+///             .collect::<String>()),
 ///         20
 ///     ),
 ///     "[00, 0000, 00, 0000, 0000, 11, , 00, , 1111, 0001, 11, 1100, 00, 0000, 0000, 1110, , \
@@ -782,8 +765,8 @@ pub fn striped_random_bool_vecs_from_length_iterator<I: Iterator<Item = u64>>(
 ///
 /// See [here](self) for more information.
 ///
-/// The mean run length (before the bit sequences are truncated) is
-/// $m$ = `mean_stripe_numerator / mean_stripe_denominator`.
+/// The mean run length (before the bit sequences are truncated) is $m$ = `mean_stripe_numerator /
+/// mean_stripe_denominator`.
 ///
 /// If `len` is 0, the output consists of the empty list, repeated.
 ///
@@ -799,14 +782,13 @@ pub fn striped_random_bool_vecs_from_length_iterator<I: Iterator<Item = u64>>(
 /// use malachite_base::iterators::prefix_to_string;
 /// use malachite_base::num::random::striped::striped_random_fixed_length_bool_vecs;
 /// use malachite_base::random::EXAMPLE_SEED;
-/// use malachite_base::strings::ToBinaryString;
 ///
 /// assert_eq!(
 ///     prefix_to_string(
-///         striped_random_fixed_length_bool_vecs(EXAMPLE_SEED, 5, 10, 1)
-///                 .map(
-///                     |bs| bs.into_iter().map(|b| if b { '1' } else { '0' }).collect::<String>()
-///                 ),
+///         striped_random_fixed_length_bool_vecs(EXAMPLE_SEED, 5, 10, 1).map(|bs| bs
+///             .into_iter()
+///             .map(|b| if b { '1' } else { '0' })
+///             .collect::<String>()),
 ///         20
 ///     ),
 ///     "[00000, 00000, 00000, 00000, 00011, 11000, 00000, 11111, 01111, 11111, 10000, 00011, \
@@ -831,39 +813,37 @@ pub fn striped_random_fixed_length_bool_vecs(
 ///
 /// See [here](self) for more information.
 ///
-/// The lengths of the [`Vec`]s are sampled from a geometric distribution with a specified mean
-/// $m$, equal to `mean_length_numerator / mean_length_denominator`. $m$ must be greater than 0.
+/// The lengths of the [`Vec`]s are sampled from a geometric distribution with a specified mean $m$,
+/// equal to `mean_length_numerator / mean_length_denominator`. $m$ must be greater than 0.
 ///
-/// The mean run length (before the bit sequences are truncated) is
-/// $m$ = `mean_stripe_numerator / mean_stripe_denominator`.
+/// The mean run length (before the bit sequences are truncated) is $m$ = `mean_stripe_numerator /
+/// mean_stripe_denominator`.
 ///
 /// # Expected complexity per iteration
 /// $T(n) = O(n)$
 ///
 /// $M(n) = O(n)$
 ///
-/// where $T$ is time, $M$ is additional memory, and $n$ is
-/// `mean_length_numerator / mean_length_denominator`.
+/// where $T$ is time, $M$ is additional memory, and $n$ is `mean_length_numerator /
+/// mean_length_denominator`.
 ///
 /// # Panics
-/// Panics if `mean_stripe_denominator` is zero, if
-/// `mean_stripe_numerator <= mean_stripe_denominator`, if `mean_length_numerator` or
-/// `mean_length_denominator` are zero, or, if after being reduced to lowest terms, their sum is
-/// greater than or equal to $2^{64}$.
+/// Panics if `mean_stripe_denominator` is zero, if `mean_stripe_numerator <=
+/// mean_stripe_denominator`, if `mean_length_numerator` or `mean_length_denominator` are zero, or,
+/// if after being reduced to lowest terms, their sum is greater than or equal to $2^{64}$.
 ///
 /// # Examples
 /// ```
 /// use malachite_base::iterators::prefix_to_string;
 /// use malachite_base::num::random::striped::striped_random_bool_vecs;
 /// use malachite_base::random::EXAMPLE_SEED;
-/// use malachite_base::strings::ToBinaryString;
 ///
 /// assert_eq!(
 ///     prefix_to_string(
-///         striped_random_bool_vecs(EXAMPLE_SEED, 10, 1, 2, 1)
-///                 .map(
-///                     |bs| bs.into_iter().map(|b| if b { '1' } else { '0' }).collect::<String>()
-///                 ),
+///         striped_random_bool_vecs(EXAMPLE_SEED, 10, 1, 2, 1).map(|bs| bs
+///             .into_iter()
+///             .map(|b| if b { '1' } else { '0' })
+///             .collect::<String>()),
 ///         20
 ///     ),
 ///     "[000000, 0, 00000000, 0, 00000001110000, , 11111, 0000, 1, , 011111, 11, , , 1, 000, , \
@@ -892,40 +872,39 @@ pub fn striped_random_bool_vecs(
 ///
 /// See [here](self) for more information.
 ///
-/// The lengths of the [`Vec`]s are sampled from a geometric distribution with a specified mean
-/// $m$, equal to `mean_length_numerator / mean_length_denominator`. $m$ must be greater than
+/// The lengths of the [`Vec`]s are sampled from a geometric distribution with a specified mean $m$,
+/// equal to `mean_length_numerator / mean_length_denominator`. $m$ must be greater than
 /// `min_length`.
 ///
-/// The mean run length (before the bit sequences are truncated) is
-/// $m$ = `mean_stripe_numerator / mean_stripe_denominator`.
+/// The mean run length (before the bit sequences are truncated) is $m$ = `mean_stripe_numerator /
+/// mean_stripe_denominator`.
 ///
 /// # Expected complexity per iteration
 /// $T(n) = O(n)$
 ///
 /// $M(n) = O(n)$
 ///
-/// where $T$ is time, $M$ is additional memory, and $n$ is
-/// `mean_length_numerator / mean_length_denominator`.
+/// where $T$ is time, $M$ is additional memory, and $n$ is `mean_length_numerator /
+/// mean_length_denominator`.
 ///
 /// # Panics
-/// Panics if `mean_stripe_denominator` is zero, if
-/// `mean_stripe_numerator <= mean_stripe_denominator`, if `mean_length_numerator` or
-/// `mean_length_denominator` are zero, if their ratio is less than or equal to `min_length`, or if
-/// they are too large and manipulating them leads to arithmetic overflow.
+/// Panics if `mean_stripe_denominator` is zero, if `mean_stripe_numerator <=
+/// mean_stripe_denominator`, if `mean_length_numerator` or `mean_length_denominator` are zero, if
+/// their ratio is less than or equal to `min_length`, or if they are too large and manipulating
+/// them leads to arithmetic overflow.
 ///
 /// # Examples
 /// ```
 /// use malachite_base::iterators::prefix_to_string;
 /// use malachite_base::num::random::striped::striped_random_bool_vecs_min_length;
 /// use malachite_base::random::EXAMPLE_SEED;
-/// use malachite_base::strings::ToBinaryString;
 ///
 /// assert_eq!(
 ///     prefix_to_string(
-///         striped_random_bool_vecs_min_length(EXAMPLE_SEED, 3, 10, 1, 5, 1)
-///                 .map(
-///                     |bs| bs.into_iter().map(|b| if b { '1' } else { '0' }).collect::<String>()
-///                 ),
+///         striped_random_bool_vecs_min_length(EXAMPLE_SEED, 3, 10, 1, 5, 1).map(|bs| bs
+///             .into_iter()
+///             .map(|b| if b { '1' } else { '0' })
+///             .collect::<String>()),
 ///         20
 ///     ),
 ///     "[000000000, 0000, 00000000111, 0111, 00000000011111111, 100, 00000111, 1111111, 0001, \
@@ -964,8 +943,8 @@ pub fn striped_random_bool_vecs_min_length(
 /// The lengths of the [`Vec`]s are sampled from a uniform distribution on $[a, b)$. $a$ must be
 /// less than $b$.
 ///
-/// The mean run length (before the bit sequences are truncated) is
-/// $m$ = `mean_stripe_numerator / mean_stripe_denominator`.
+/// The mean run length (before the bit sequences are truncated) is $m$ = `mean_stripe_numerator /
+/// mean_stripe_denominator`.
 ///
 /// $$
 /// P((x_0, x_1, \ldots, x_{n-1})) = \\begin{cases}
@@ -982,22 +961,21 @@ pub fn striped_random_bool_vecs_min_length(
 /// where $T$ is time and $M$ is additional memory.
 ///
 /// # Panics
-/// Panics if `mean_stripe_denominator` is zero, if
-/// `mean_stripe_numerator <= mean_stripe_denominator`, or if $a \geq b$.
+/// Panics if `mean_stripe_denominator` is zero, if `mean_stripe_numerator <=
+/// mean_stripe_denominator`, or if $a \geq b$.
 ///
 /// # Examples
 /// ```
 /// use malachite_base::iterators::prefix_to_string;
 /// use malachite_base::num::random::striped::striped_random_bool_vecs_length_range;
 /// use malachite_base::random::EXAMPLE_SEED;
-/// use malachite_base::strings::ToBinaryString;
 ///
 /// assert_eq!(
 ///     prefix_to_string(
-///         striped_random_bool_vecs_length_range(EXAMPLE_SEED, 4, 10, 10, 1)
-///                 .map(
-///                     |bs| bs.into_iter().map(|b| if b { '1' } else { '0' }).collect::<String>()
-///                 ),
+///         striped_random_bool_vecs_length_range(EXAMPLE_SEED, 4, 10, 10, 1).map(|bs| bs
+///             .into_iter()
+///             .map(|b| if b { '1' } else { '0' })
+///             .collect::<String>()),
 ///         20
 ///     ),
 ///     "[000000000, 000000000, 000111000, 000000000, 0111, 11111, 00111111, 1000000, 00000011, \
@@ -1028,8 +1006,8 @@ pub fn striped_random_bool_vecs_length_range(
 /// The lengths of the [`Vec`]s are sampled from a uniform distribution on $[a, b]$. $a$ must be
 /// less than $b$.
 ///
-/// The mean run length (before the bit sequences are truncated) is
-/// $m$ = `mean_stripe_numerator / mean_stripe_denominator`.
+/// The mean run length (before the bit sequences are truncated) is $m$ = `mean_stripe_numerator /
+/// mean_stripe_denominator`.
 ///
 /// $$
 /// P((x_0, x_1, \ldots, x_{n-1})) = \\begin{cases}
@@ -1046,22 +1024,21 @@ pub fn striped_random_bool_vecs_length_range(
 /// where $T$ is time and $M$ is additional memory.
 ///
 /// # Panics
-/// Panics if `mean_stripe_denominator` is zero, if
-/// `mean_stripe_numerator <= mean_stripe_denominator`, or if $a \geq b$.
+/// Panics if `mean_stripe_denominator` is zero, if `mean_stripe_numerator <=
+/// mean_stripe_denominator`, or if $a \geq b$.
 ///
 /// # Examples
 /// ```
 /// use malachite_base::iterators::prefix_to_string;
 /// use malachite_base::num::random::striped::striped_random_bool_vecs_length_inclusive_range;
 /// use malachite_base::random::EXAMPLE_SEED;
-/// use malachite_base::strings::ToBinaryString;
 ///
 /// assert_eq!(
 ///     prefix_to_string(
-///         striped_random_bool_vecs_length_inclusive_range(EXAMPLE_SEED, 4, 9, 10, 1)
-///                 .map(
-///                     |bs| bs.into_iter().map(|b| if b { '1' } else { '0' }).collect::<String>()
-///                 ),
+///         striped_random_bool_vecs_length_inclusive_range(EXAMPLE_SEED, 4, 9, 10, 1).map(|bs| bs
+///             .into_iter()
+///             .map(|b| if b { '1' } else { '0' })
+///             .collect::<String>()),
 ///         20
 ///     ),
 ///     "[000000000, 000000000, 000111000, 000000000, 0111, 11111, 00111111, 1000000, 00000011, \
@@ -1090,7 +1067,7 @@ pub fn striped_random_bool_vecs_length_inclusive_range(
 ///
 /// See [here](self) for more information.
 ///
-/// The output length is `bit_len.div_round(T::WIDTH, RoundingMode::Ceiling)`.
+/// The output length is `bit_len.div_round(T::WIDTH, Ceiling)`.
 ///
 /// # Expected complexity
 /// $T(n) = O(n)$
@@ -1155,19 +1132,19 @@ impl<T: PrimitiveUnsigned, I: Iterator<Item = u64>> Iterator for StripedRandomUn
 ///
 /// See [here](self) for more information.
 ///
-/// The mean run length (before the bit sequences are truncated) is
-/// $m$ = `mean_stripe_numerator / mean_stripe_denominator`.
+/// The mean run length (before the bit sequences are truncated) is $m$ = `mean_stripe_numerator /
+/// mean_stripe_denominator`.
 ///
 /// # Panics
-/// Panics if `mean_stripe_denominator` is zero or if
-/// `mean_stripe_numerator <= mean_stripe_denominator`.
+/// Panics if `mean_stripe_denominator` is zero or if `mean_stripe_numerator <=
+/// mean_stripe_denominator`.
 ///
 /// # Examples
 /// ```
 /// use malachite_base::iterators::prefix_to_string;
 /// use malachite_base::num::random::striped::striped_random_unsigned_vecs_from_length_iterator;
 /// use malachite_base::random::EXAMPLE_SEED;
-/// use malachite_base::strings::{ToBinaryString, ToDebugString};
+/// use malachite_base::strings::ToBinaryString;
 /// use malachite_base::vecs::random_values_from_vec;
 ///
 /// assert_eq!(
@@ -1177,7 +1154,8 @@ impl<T: PrimitiveUnsigned, I: Iterator<Item = u64>> Iterator for StripedRandomUn
 ///             &|seed| random_values_from_vec(seed, vec![0, 2, 4]),
 ///             10,
 ///             1,
-///         ).map(|xs| prefix_to_string(xs.into_iter().map(|x: u8| x.to_binary_string()), 100)),
+///         )
+///         .map(|xs| prefix_to_string(xs.into_iter().map(|x: u8| x.to_binary_string()), 100)),
 ///         10,
 ///     ),
 ///     "[[0, 0], [1110000, 0, 11111100, 11], [11111110, 1111], [0, 0, 0, 11111000], \
@@ -1210,8 +1188,8 @@ pub fn striped_random_unsigned_vecs_from_length_iterator<
 ///
 /// See [here](self) for more information.
 ///
-/// The mean run length (before the bit sequences are truncated) is
-/// $m$ = `mean_stripe_numerator / mean_stripe_denominator`.
+/// The mean run length (before the bit sequences are truncated) is $m$ = `mean_stripe_numerator /
+/// mean_stripe_denominator`.
 ///
 /// If `len` is 0, the output consists of the empty list, repeated.
 ///
@@ -1227,17 +1205,13 @@ pub fn striped_random_unsigned_vecs_from_length_iterator<
 /// use malachite_base::iterators::prefix_to_string;
 /// use malachite_base::num::random::striped::striped_random_fixed_length_unsigned_vecs;
 /// use malachite_base::random::EXAMPLE_SEED;
-/// use malachite_base::strings::{ToBinaryString, ToDebugString};
+/// use malachite_base::strings::ToBinaryString;
 ///
 /// assert_eq!(
 ///     prefix_to_string(
-///         striped_random_fixed_length_unsigned_vecs::<u8>(EXAMPLE_SEED, 3, 10, 1)
-///                 .map(
-///                     |xs| prefix_to_string(
-///                         xs.into_iter().map(|x: u8| x.to_binary_string()),
-///                         100
-///                     )
-///                 ),
+///         striped_random_fixed_length_unsigned_vecs::<u8>(EXAMPLE_SEED, 3, 10, 1).map(|xs| {
+///             prefix_to_string(xs.into_iter().map(|x: u8| x.to_binary_string()), 100)
+///         }),
 ///         10,
 ///     ),
 ///     "[[0, 0, 111000], [0, 11111100, 11], [11111110, 1111, 0], [0, 0, 11111000], \
@@ -1264,11 +1238,11 @@ pub fn striped_random_fixed_length_unsigned_vecs<T: PrimitiveUnsigned>(
 ///
 /// See [here](self) for more information.
 ///
-/// The lengths of the [`Vec`]s are sampled from a geometric distribution with a specified mean
-/// $m$, equal to `mean_length_numerator / mean_length_denominator`. $m$ must be greater than 0.
+/// The lengths of the [`Vec`]s are sampled from a geometric distribution with a specified mean $m$,
+/// equal to `mean_length_numerator / mean_length_denominator`. $m$ must be greater than 0.
 ///
-/// The mean run length (before the bit sequences are truncated) is
-/// $m$ = `mean_stripe_numerator / mean_stripe_denominator`.
+/// The mean run length (before the bit sequences are truncated) is $m$ = `mean_stripe_numerator /
+/// mean_stripe_denominator`.
 ///
 /// $$
 /// P((x_0, x_1, \ldots, x_{n-1})) = \frac{m^n}{(m+1)^{n+1}}\prod_{i=0}^{n-1}P(x_i).
@@ -1279,31 +1253,25 @@ pub fn striped_random_fixed_length_unsigned_vecs<T: PrimitiveUnsigned>(
 ///
 /// $M(n) = O(n)$
 ///
-/// where $T$ is time, $M$ is additional memory, and $n$ is
-/// `mean_length_numerator / mean_length_denominator`.
+/// where $T$ is time, $M$ is additional memory, and $n$ is `mean_length_numerator /
+/// mean_length_denominator`.
 ///
 /// # Panics
-/// Panics if `mean_stripe_denominator` is zero, if
-/// `mean_stripe_numerator <= mean_stripe_denominator`, if `mean_length_numerator` or
-/// `mean_length_denominator` are zero, or, if after being reduced to lowest terms, their sum is
-/// greater than or equal to $2^{64}$.
+/// Panics if `mean_stripe_denominator` is zero, if `mean_stripe_numerator <=
+/// mean_stripe_denominator`, if `mean_length_numerator` or `mean_length_denominator` are zero, or,
+/// if after being reduced to lowest terms, their sum is greater than or equal to $2^{64}$.
 ///
 /// # Examples
 /// ```
 /// use malachite_base::iterators::prefix_to_string;
 /// use malachite_base::num::random::striped::striped_random_unsigned_vecs;
 /// use malachite_base::random::EXAMPLE_SEED;
-/// use malachite_base::strings::{ToBinaryString, ToDebugString};
+/// use malachite_base::strings::ToBinaryString;
 ///
 /// assert_eq!(
 ///     prefix_to_string(
 ///         striped_random_unsigned_vecs::<u8>(EXAMPLE_SEED, 10, 1, 2, 1)
-///                 .map(
-///                     |xs| prefix_to_string(
-///                         xs.into_iter().map(|x: u8| x.to_binary_string()),
-///                         100
-///                     )
-///                 ),
+///             .map(|xs| prefix_to_string(xs.into_iter().map(|x: u8| x.to_binary_string()), 100)),
 ///         10,
 ///     ),
 ///     "[[0, 0, 111000, 0, 11111110, 10000001], [0], \
@@ -1335,12 +1303,12 @@ pub fn striped_random_unsigned_vecs<T: PrimitiveUnsigned>(
 ///
 /// See [here](self) for more information.
 ///
-/// The lengths of the [`Vec`]s are sampled from a geometric distribution with a specified mean
-/// $m$, equal to `mean_length_numerator / mean_length_denominator`. $m$ must be greater than
+/// The lengths of the [`Vec`]s are sampled from a geometric distribution with a specified mean $m$,
+/// equal to `mean_length_numerator / mean_length_denominator`. $m$ must be greater than
 /// `min_length`.
 ///
-/// The mean run length (before the bit sequences are truncated) is
-/// $m$ = `mean_stripe_numerator / mean_stripe_denominator`.
+/// The mean run length (before the bit sequences are truncated) is $m$ = `mean_stripe_numerator /
+/// mean_stripe_denominator`.
 ///
 /// $$
 /// P((x_0, x_1, \ldots, x_{n-1})) = \\begin{cases}
@@ -1355,14 +1323,14 @@ pub fn striped_random_unsigned_vecs<T: PrimitiveUnsigned>(
 ///
 /// $M(n) = O(n)$
 ///
-/// where $T$ is time, $M$ is additional memory, and $n$ is
-/// `mean_length_numerator / mean_length_denominator`.
+/// where $T$ is time, $M$ is additional memory, and $n$ is `mean_length_numerator /
+/// mean_length_denominator`.
 ///
 /// # Panics
-/// Panics if `mean_stripe_denominator` is zero, if
-/// `mean_stripe_numerator <= mean_stripe_denominator`, if `mean_length_numerator` or
-/// `mean_length_denominator` are zero, if their ratio is less than or equal to `min_length`, or if
-/// they are too large and manipulating them leads to arithmetic overflow.
+/// Panics if `mean_stripe_denominator` is zero, if `mean_stripe_numerator <=
+/// mean_stripe_denominator`, if `mean_length_numerator` or `mean_length_denominator` are zero, if
+/// their ratio is less than or equal to `min_length`, or if they are too large and manipulating
+/// them leads to arithmetic overflow.
 ///
 /// # Examples
 /// ```
@@ -1374,12 +1342,7 @@ pub fn striped_random_unsigned_vecs<T: PrimitiveUnsigned>(
 /// assert_eq!(
 ///     prefix_to_string(
 ///         striped_random_unsigned_vecs_min_length::<u8>(EXAMPLE_SEED, 2, 10, 1, 3, 1)
-///                 .map(
-///                     |xs| prefix_to_string(
-///                         xs.into_iter().map(|x: u8| x.to_binary_string()),
-///                         100
-///                     )
-///                 ),
+///             .map(|xs| prefix_to_string(xs.into_iter().map(|x: u8| x.to_binary_string()), 100)),
 ///         10,
 ///     ),
 ///     "[[0, 0, 111000], [0, 11111100, 11, 11111111], \
@@ -1420,8 +1383,8 @@ pub fn striped_random_unsigned_vecs_min_length<T: PrimitiveUnsigned>(
 /// The lengths of the [`Vec`]s are sampled from a uniform distribution on $[a, b)$. $a$ must be
 /// less than $b$.
 ///
-/// The mean run length (before the bit sequences are truncated) is
-/// $m$ = `mean_stripe_numerator / mean_stripe_denominator`.
+/// The mean run length (before the bit sequences are truncated) is $m$ = `mean_stripe_numerator /
+/// mean_stripe_denominator`.
 ///
 /// $$
 /// P((x_0, x_1, \ldots, x_{n-1})) = \\begin{cases}
@@ -1438,8 +1401,8 @@ pub fn striped_random_unsigned_vecs_min_length<T: PrimitiveUnsigned>(
 /// where $T$ is time and $M$ is additional memory.
 ///
 /// # Panics
-/// Panics if `mean_stripe_denominator` is zero, if
-/// `mean_stripe_numerator <= mean_stripe_denominator`, or if $a \geq b$.
+/// Panics if `mean_stripe_denominator` is zero, if `mean_stripe_numerator <=
+/// mean_stripe_denominator`, or if $a \geq b$.
 ///
 /// # Examples
 /// ```
@@ -1451,12 +1414,7 @@ pub fn striped_random_unsigned_vecs_min_length<T: PrimitiveUnsigned>(
 /// assert_eq!(
 ///     prefix_to_string(
 ///         striped_random_unsigned_vecs_length_range::<u8>(EXAMPLE_SEED, 2, 4, 10, 1)
-///                 .map(
-///                     |xs| prefix_to_string(
-///                         xs.into_iter().map(|x: u8| x.to_binary_string()),
-///                         100
-///                     )
-///                 ),
+///             .map(|xs| prefix_to_string(xs.into_iter().map(|x: u8| x.to_binary_string()), 100)),
 ///         10,
 ///     ),
 ///     "[[0, 0, 111000], [0, 11111100], [11111000, 1, 11110000], [0, 0, 0], \
@@ -1487,8 +1445,8 @@ pub fn striped_random_unsigned_vecs_length_range<T: PrimitiveUnsigned>(
 /// The lengths of the [`Vec`]s are sampled from a uniform distribution on $[a, b]$. $a$ must be
 /// less than $b$.
 ///
-/// The mean run length (before the bit sequences are truncated) is
-/// $m$ = `mean_stripe_numerator / mean_stripe_denominator`.
+/// The mean run length (before the bit sequences are truncated) is $m$ = `mean_stripe_numerator /
+/// mean_stripe_denominator`.
 ///
 /// $$
 /// P((x_0, x_1, \ldots, x_{n-1})) = \\begin{cases}
@@ -1505,8 +1463,8 @@ pub fn striped_random_unsigned_vecs_length_range<T: PrimitiveUnsigned>(
 /// where $T$ is time and $M$ is additional memory.
 ///
 /// # Panics
-/// Panics if `mean_stripe_denominator` is zero, if
-/// `mean_stripe_numerator <= mean_stripe_denominator`, or if $a \geq b$.
+/// Panics if `mean_stripe_denominator` is zero, if `mean_stripe_numerator <=
+/// mean_stripe_denominator`, or if $a \geq b$.
 ///
 /// # Examples
 /// ```
@@ -1518,12 +1476,7 @@ pub fn striped_random_unsigned_vecs_length_range<T: PrimitiveUnsigned>(
 /// assert_eq!(
 ///     prefix_to_string(
 ///         striped_random_unsigned_vecs_length_inclusive_range::<u8>(EXAMPLE_SEED, 2, 3, 10, 1)
-///                 .map(
-///                     |xs| prefix_to_string(
-///                         xs.into_iter().map(|x: u8| x.to_binary_string()),
-///                         100
-///                     )
-///                 ),
+///             .map(|xs| prefix_to_string(xs.into_iter().map(|x: u8| x.to_binary_string()), 100)),
 ///         10,
 ///     ),
 ///     "[[0, 0, 111000], [0, 11111100], [11111000, 1, 11110000], [0, 0, 0], \
@@ -1621,8 +1574,8 @@ impl<T: PrimitiveUnsigned> Iterator for StripedRandomUnsignedInclusiveRange<T> {
 ///
 /// See [here](self) for more information.
 ///
-/// The unsigneds are generated using a striped bit sequence with mean run length
-/// $m$ = `mean_stripe_numerator / mean_stripe_denominator`.
+/// The unsigneds are generated using a striped bit sequence with mean run length $m$ =
+/// `mean_stripe_numerator / mean_stripe_denominator`.
 ///
 /// Because the unsigneds are constrained to be within a certain range, the actual mean run length
 /// will usually not be $m$. Nonetheless, setting a higher $m$ will result in a higher mean run
@@ -1636,8 +1589,8 @@ impl<T: PrimitiveUnsigned> Iterator for StripedRandomUnsignedInclusiveRange<T> {
 /// where $T$ is time, $M$ is additional memory, and $n$ is `b.significant_bits()`.
 ///
 /// # Panics
-/// Panics if `mean_stripe_denominator` is zero, if
-/// `mean_stripe_numerator <= mean_stripe_denominator`, or if $a \geq b$.
+/// Panics if `mean_stripe_denominator` is zero, if `mean_stripe_numerator <=
+/// mean_stripe_denominator`, or if $a \geq b$.
 ///
 /// # Examples
 /// ```
@@ -1649,7 +1602,7 @@ impl<T: PrimitiveUnsigned> Iterator for StripedRandomUnsignedInclusiveRange<T> {
 /// assert_eq!(
 ///     prefix_to_string(
 ///         striped_random_unsigned_range::<u8>(EXAMPLE_SEED, 1, 7, 4, 1)
-///                 .map(|x| x.to_binary_string()),
+///             .map(|x| x.to_binary_string()),
 ///         10
 ///     ),
 ///     "[1, 1, 1, 110, 1, 110, 10, 11, 11, 100, ...]"
@@ -1677,8 +1630,8 @@ pub fn striped_random_unsigned_range<T: PrimitiveUnsigned>(
 ///
 /// See [here](self) for more information.
 ///
-/// The unsigneds are generated using a striped bit sequence with mean run length
-/// $m$ = `mean_stripe_numerator / mean_stripe_denominator`.
+/// The unsigneds are generated using a striped bit sequence with mean run length $m$ =
+/// `mean_stripe_numerator / mean_stripe_denominator`.
 ///
 /// Because the unsigneds are constrained to be within a certain range, the actual mean run length
 /// will usually not be $m$. Nonetheless, setting a higher $m$ will result in a higher mean run
@@ -1692,8 +1645,8 @@ pub fn striped_random_unsigned_range<T: PrimitiveUnsigned>(
 /// where $T$ is time, $M$ is additional memory, and $n$ is `b.significant_bits()`.
 ///
 /// # Panics
-/// Panics if `mean_stripe_denominator` is zero, if
-/// `mean_stripe_numerator <= mean_stripe_denominator`, or if $a > b$.
+/// Panics if `mean_stripe_denominator` is zero, if `mean_stripe_numerator <=
+/// mean_stripe_denominator`, or if $a > b$.
 ///
 /// # Examples
 /// ```
@@ -1705,7 +1658,7 @@ pub fn striped_random_unsigned_range<T: PrimitiveUnsigned>(
 /// assert_eq!(
 ///     prefix_to_string(
 ///         striped_random_unsigned_inclusive_range::<u8>(EXAMPLE_SEED, 1, 6, 4, 1)
-///                 .map(|x| x.to_binary_string()),
+///             .map(|x| x.to_binary_string()),
 ///         10
 ///     ),
 ///     "[1, 1, 1, 110, 1, 110, 10, 11, 11, 100, ...]"
@@ -1778,8 +1731,8 @@ impl<U: PrimitiveUnsigned, S: PrimitiveSigned + WrappingFrom<U>> Iterator
 ///
 /// See [here](self) for more information.
 ///
-/// The unsigneds are generated using a striped bit sequence with mean run length
-/// $m$ = `mean_stripe_numerator / mean_stripe_denominator`.
+/// The unsigneds are generated using a striped bit sequence with mean run length $m$ =
+/// `mean_stripe_numerator / mean_stripe_denominator`.
 ///
 /// Because the signeds are constrained to be within a certain range, the actual mean run length
 /// will usually not be $m$. Nonetheless, setting a higher $m$ will result in a higher mean run
@@ -1790,12 +1743,12 @@ impl<U: PrimitiveUnsigned, S: PrimitiveSigned + WrappingFrom<U>> Iterator
 ///
 /// $M(n) = O(n)$
 ///
-/// where $T$ is time, $M$ is additional memory, and $n$ is
-/// `max(a.significant_bits(), b.significant_bits())`.
+/// where $T$ is time, $M$ is additional memory, and $n$ is `max(a.significant_bits(),
+/// b.significant_bits())`.
 ///
 /// # Panics
-/// Panics if `mean_stripe_denominator` is zero, if
-/// `mean_stripe_numerator <= mean_stripe_denominator`, or if $a > b$.
+/// Panics if `mean_stripe_denominator` is zero, if `mean_stripe_numerator <=
+/// mean_stripe_denominator`, or if $a \geq b$.
 ///
 /// # Examples
 /// ```
@@ -1807,7 +1760,7 @@ impl<U: PrimitiveUnsigned, S: PrimitiveSigned + WrappingFrom<U>> Iterator
 /// assert_eq!(
 ///     prefix_to_string(
 ///         striped_random_signed_inclusive_range::<u8, i8>(EXAMPLE_SEED, -5, 10, 4, 1)
-///                 .map(|x| x.to_binary_string()),
+///             .map(|x| x.to_binary_string()),
 ///         10
 ///     ),
 ///     "[11111011, 11111100, 1000, 111, 11111111, 1000, 11, 1000, 0, 1000, ...]"
@@ -1838,8 +1791,8 @@ pub fn striped_random_signed_range<
 ///
 /// See [here](self) for more information.
 ///
-/// The unsigneds are generated using a striped bit sequence with mean run length
-/// $m$ = `mean_stripe_numerator / mean_stripe_denominator`.
+/// The unsigneds are generated using a striped bit sequence with mean run length $m$ =
+/// `mean_stripe_numerator / mean_stripe_denominator`.
 ///
 /// Because the signeds are constrained to be within a certain range, the actual mean run length
 /// will usually not be $m$. Nonetheless, setting a higher $m$ will result in a higher mean run
@@ -1850,12 +1803,12 @@ pub fn striped_random_signed_range<
 ///
 /// $M(n) = O(n)$
 ///
-/// where $T$ is time, $M$ is additional memory, and $n$ is
-/// `max(a.significant_bits(), b.significant_bits())`.
+/// where $T$ is time, $M$ is additional memory, and $n$ is `max(a.significant_bits(),
+/// b.significant_bits())`.
 ///
 /// # Panics
-/// Panics if `mean_stripe_denominator` is zero, if
-/// `mean_stripe_numerator <= mean_stripe_denominator`, or if $a \geq b$.
+/// Panics if `mean_stripe_denominator` is zero, if `mean_stripe_numerator <=
+/// mean_stripe_denominator`, or if $a > b$.
 ///
 /// # Examples
 /// ```
@@ -1867,7 +1820,7 @@ pub fn striped_random_signed_range<
 /// assert_eq!(
 ///     prefix_to_string(
 ///         striped_random_signed_range::<u8, i8>(EXAMPLE_SEED, -5, 11, 4, 1)
-///                 .map(|x| x.to_binary_string()),
+///             .map(|x| x.to_binary_string()),
 ///         10
 ///     ),
 ///     "[11111011, 11111100, 1000, 111, 11111111, 1000, 11, 1000, 0, 1000, ...]"
