@@ -1,9 +1,17 @@
-use crate::orderings::ORDERINGS;
-use std::cmp::Ordering;
-use std::iter::Cloned;
-use std::slice::Iter;
+// Copyright © 2024 Mikhail Hogrefe
+//
+// This file is part of Malachite.
+//
+// Malachite is free software: you can redistribute it and/or modify it under the terms of the GNU
+// Lesser General Public License (LGPL) as published by the Free Software Foundation; either version
+// 3 of the License, or (at your option) any later version. See <https://www.gnu.org/licenses/>.
 
-pub type ExhaustiveOrderings = Cloned<Iter<'static, Ordering>>;
+use crate::orderings::ORDERINGS;
+use core::cmp::Ordering::{self, *};
+use core::iter::Copied;
+use core::slice::Iter;
+
+pub type ExhaustiveOrderings = Copied<Iter<'static, Ordering>>;
 
 /// Generates all [`Ordering`]s, in increasing order.
 ///
@@ -16,18 +24,16 @@ pub type ExhaustiveOrderings = Cloned<Iter<'static, Ordering>>;
 /// ```
 /// use itertools::Itertools;
 /// use malachite_base::orderings::exhaustive::orderings_increasing;
-/// use std::cmp::Ordering;
+/// use std::cmp::Ordering::*;
 ///
 /// assert_eq!(
 ///     orderings_increasing().collect_vec(),
-///     &[Ordering::Less, Ordering::Equal, Ordering::Greater]
+///     &[Less, Equal, Greater]
 /// );
 /// ```
 #[inline]
 pub fn orderings_increasing() -> ExhaustiveOrderings {
-    [Ordering::Less, Ordering::Equal, Ordering::Greater]
-        .iter()
-        .cloned()
+    [Less, Equal, Greater].iter().copied()
 }
 
 /// Generates all [`Ordering`]s, with `Equal` coming first.
@@ -41,14 +47,14 @@ pub fn orderings_increasing() -> ExhaustiveOrderings {
 /// ```
 /// use itertools::Itertools;
 /// use malachite_base::orderings::exhaustive::exhaustive_orderings;
-/// use std::cmp::Ordering;
+/// use std::cmp::Ordering::*;
 ///
 /// assert_eq!(
 ///     exhaustive_orderings().collect_vec(),
-///     &[Ordering::Equal, Ordering::Less, Ordering::Greater]
+///     &[Equal, Less, Greater]
 /// );
 /// ```
 #[inline]
-pub fn exhaustive_orderings() -> Cloned<Iter<'static, Ordering>> {
-    ORDERINGS.iter().cloned()
+pub fn exhaustive_orderings() -> Copied<Iter<'static, Ordering>> {
+    ORDERINGS.iter().copied()
 }
