@@ -1,6 +1,14 @@
-use crate::itertools::Itertools;
+// Copyright © 2024 Mikhail Hogrefe
+//
+// This file is part of Malachite.
+//
+// Malachite is free software: you can redistribute it and/or modify it under the terms of the GNU
+// Lesser General Public License (LGPL) as published by the Free Software Foundation; either version
+// 3 of the License, or (at your option) any later version. See <https://www.gnu.org/licenses/>.
+
 use crate::num::basic::floats::PrimitiveFloat;
 use crate::num::float::NiceFloat;
+use itertools::Itertools;
 
 pub fn exhaustive_primitive_floats_helper_helper_with_limit<
     T: PrimitiveFloat,
