@@ -1,3 +1,11 @@
+// Copyright © 2024 Mikhail Hogrefe
+//
+// This file is part of Malachite.
+//
+// Malachite is free software: you can redistribute it and/or modify it under the terms of the GNU
+// Lesser General Public License (LGPL) as published by the Free Software Foundation; either version
+// 3 of the License, or (at your option) any later version. See <https://www.gnu.org/licenses/>.
+
 use crate::num::arithmetic::traits::{SaturatingAbs, UnsignedAbs};
 use crate::num::basic::integers::PrimitiveInt;
 use crate::num::basic::signeds::PrimitiveSigned;
@@ -22,6 +30,7 @@ pub trait CheckedToF64 {
 macro_rules! impl_checked_to_f64_for_primitive_ints {
     ($t: ident) => {
         impl CheckedToF64 for $t {
+            #[allow(clippy::cast_lossless)]
             #[inline]
             fn checked_to_f64(&self) -> f64 {
                 // No primitive integer type, not even u128 and i128, exceeds the finite range of
