@@ -1,6 +1,14 @@
-use crate::itertools::Itertools;
+// Copyright © 2024 Mikhail Hogrefe
+//
+// This file is part of Malachite.
+//
+// Malachite is free software: you can redistribute it and/or modify it under the terms of the GNU
+// Lesser General Public License (LGPL) as published by the Free Software Foundation; either version
+// 3 of the License, or (at your option) any later version. See <https://www.gnu.org/licenses/>.
+
 use crate::test_util::stats::common_values_map::common_values_map_debug;
 use crate::test_util::stats::median;
+use itertools::Itertools;
 use std::fmt::Debug;
 use std::hash::Hash;
 
