@@ -1,3 +1,11 @@
+// Copyright © 2024 Mikhail Hogrefe
+//
+// This file is part of Malachite.
+//
+// Malachite is free software: you can redistribute it and/or modify it under the terms of the GNU
+// Lesser General Public License (LGPL) as published by the Free Software Foundation; either version
+// 3 of the License, or (at your option) any later version. See <https://www.gnu.org/licenses/>.
+
 use crate::iterators::bit_distributor::{BitDistributor, BitDistributorOutputType};
 use crate::iterators::iterator_cache::IteratorCache;
 use crate::num::arithmetic::traits::CheckedPow;
@@ -6,11 +14,13 @@ use crate::num::logic::traits::SignificantBits;
 use crate::vecs::exhaustive::{
     fixed_length_ordered_unique_indices_helper, next_bit_pattern, unique_indices, UniqueIndices,
 };
-use std::cmp::max;
-use std::fmt::Debug;
-use std::iter::{once, Once};
-use std::marker::PhantomData;
-use std::mem::take;
+use alloc::vec;
+use alloc::vec::Vec;
+use core::cmp::max;
+use core::fmt::Debug;
+use core::iter::{once, Once};
+use core::marker::PhantomData;
+use core::mem::take;
 
 /// Generates the only unit: `()`.
 ///
@@ -41,16 +51,12 @@ clone_helper<T: Clone>(x: &T, _i: usize) -> T {
 ///
 /// Malachite provides [`lex_pairs`] and [`lex_pairs_from_single`], but you can also define
 /// `lex_triples`, `lex_quadruples`, and so on, and `lex_triples_from_single`,
-/// `lex_quadruples_from_single`, and so on, in your program using the code below. The
-/// documentation for [`lex_pairs`] and [`lex_pairs_from_single`] describes these other functions
-/// as well.
+/// `lex_quadruples_from_single`, and so on, in your program using the code below. The documentation
+/// for [`lex_pairs`] and [`lex_pairs_from_single`] describes these other functions as well.
 ///
 /// See usage examples [here](self#lex_pairs) and [here](self#lex_pairs_from_single).
 ///
 /// ```
-/// # #[macro_use]
-/// # extern crate malachite_base;
-/// # fn main() {
 /// use malachite_base::iterators::iterator_cache::IteratorCache;
 /// use malachite_base::lex_tuples;
 ///
@@ -145,7 +151,6 @@ clone_helper<T: Clone>(x: &T, _i: usize) -> T {
 ///     [6, T, O, ts, t],
 ///     [7, S, P, ss, s]
 /// );
-/// # }
 /// ```
 #[macro_export]
 macro_rules! lex_tuples {
@@ -187,9 +192,8 @@ macro_rules! lex_tuples {
                         return false;
                     } else if i == 0 {
                         return true;
-                    } else {
-                        *counter = 0;
                     }
+                    *counter = 0;
                 }
                 false
             }
@@ -275,9 +279,8 @@ macro_rules! lex_tuples {
                         return false;
                     } else if i == 0 {
                         return true;
-                    } else {
-                        *counter = 0;
                     }
+                    *counter = 0;
                 }
                 false
             }
@@ -368,18 +371,15 @@ lex_tuples!(
 /// See usage examples [here](self#lex_triples_xxy).
 ///
 /// ```
-/// # #[macro_use]
-/// # extern crate malachite_base;
-/// # fn main() {
 /// use malachite_base::iterators::iterator_cache::IteratorCache;
-/// use malachite_base::lex_tuples;
+/// use malachite_base::lex_custom_tuples;
 ///
 /// fn unwrap_triple<X, Y, Z>((a, b, c): (Option<X>, Option<Y>, Option<Z>)) -> (X, Y, Z) {
 ///     (a.unwrap(), b.unwrap(), c.unwrap())
 /// }
 ///
 /// lex_custom_tuples! {
-///     (pub(crate)),
+///     (pub),
 ///     LexTriplesXXY,
 ///     (X, X, Y),
 ///     (None, None, None),
@@ -389,7 +389,7 @@ lex_tuples!(
 ///     [Y, J, ys, [2, y_2]]
 /// }
 /// lex_custom_tuples!(
-///     (pub(crate)),
+///     (pub),
 ///     LexTriplesXYX,
 ///     (X, Y, X),
 ///     (None, None, None),
@@ -399,7 +399,7 @@ lex_tuples!(
 ///     [Y, J, ys, [1, y_1]]
 /// );
 /// lex_custom_tuples!(
-///     (pub(crate)),
+///     (pub),
 ///     LexTriplesXYY,
 ///     (X, Y, Y),
 ///     (None, None, None),
@@ -408,7 +408,6 @@ lex_tuples!(
 ///     [X, I, xs, [0, x_0]],
 ///     [Y, J, ys, [1, y_1], [2, y_2]]
 /// );
-/// # }
 /// ```
 #[macro_export]
 macro_rules! lex_custom_tuples {
@@ -452,9 +451,8 @@ macro_rules! lex_custom_tuples {
                         return false;
                     } else if i == 0 {
                         return true;
-                    } else {
-                        *counter = 0;
                     }
+                    *counter = 0;
                 }
                 false
             }
@@ -497,15 +495,15 @@ macro_rules! lex_custom_tuples {
         //
         // The order is lexicographic with respect to the order of the element iterators.
         //
-        // The mapping from iterators to tuple slots is indicated by the function name; for
-        // example, `lex_triples_xyx` takes two iterators, `xs`, and `ys`; `xs` generates the
-        // elements in the first and third slots of the output triples, and `ys` generates the
-        // elements in the second slots.
+        // The mapping from iterators to tuple slots is indicated by the function name; for example,
+        // `lex_triples_xyx` takes two iterators, `xs`, and `ys`; `xs` generates the elements in the
+        // first and third slots of the output triples, and `ys` generates the elements in the
+        // second slots.
         //
         // Let `xs` be the input iterator mapped to the first slot of the output tuples. All the
         // input iterators, except possibly `xs`, must be finite. If `xs` is finite, the output
-        // length is the product of the lengths of all the input iterators. If `xs` is infinite,
-        // the output is also infinite.
+        // length is the product of the lengths of all the input iterators. If `xs` is infinite, the
+        // output is also infinite.
         //
         // If any of the input iterators is empty, the output is also empty.
         //
@@ -536,9 +534,7 @@ macro_rules! lex_custom_tuples {
 /// [here](self#exhaustive_pairs_1_input).
 ///
 /// ```
-/// # #[macro_use]
-/// # extern crate malachite_base;
-/// # fn main() {
+/// use malachite_base::exhaustive_tuples_1_input;
 /// use malachite_base::iterators::bit_distributor::{BitDistributor, BitDistributorOutputType};
 /// use malachite_base::iterators::iterator_cache::IteratorCache;
 /// use malachite_base::num::arithmetic::traits::CheckedPow;
@@ -639,7 +635,6 @@ macro_rules! lex_custom_tuples {
 ///     [6, output_type_t],
 ///     [7, output_type_s]
 /// );
-/// # }
 /// ```
 #[macro_export]
 macro_rules! exhaustive_tuples_1_input {
@@ -732,17 +727,14 @@ macro_rules! exhaustive_tuples_1_input {
         /// Generates all length-$n$ tuples with elements from a single iterator.
         ///
         /// These functions differ from `exhaustive_[n-tuples]_from_single` in that different
-        /// [`BitDistributorOutputType`](BitDistributorOutputType)s may be specified for each
-        /// output element.
+        /// [`BitDistributorOutputType`]s may be specified for each output element.
         ///
-        /// The $i$th parameter `output_types_[x_i]` is a
-        /// [`BitDistributorOutputType`](BitDistributorOutputType) that determines how quickly the
-        /// $i$th output slot advances through the iterator; see the
-        /// [`BitDistributor`](BitDistributor) documentation for a description of the different
-        /// types.
+        /// The $i$th parameter `output_types_[x_i]` is a [`BitDistributorOutputType`] that
+        /// determines how quickly the $i$th output slot advances through the iterator; see the
+        /// [`BitDistributor`] documentation for a description of the different types.
         ///
-        /// If `xs` is finite, the output length is $k^n$, where $k$ is `xs.count()` and $n$
-        /// is the width of the tuples. If `xs` is infinite, the output is also infinite.
+        /// If `xs` is finite, the output length is $k^n$, where $k$ is `xs.count()` and $n$ is the
+        /// width of the tuples. If `xs` is infinite, the output is also infinite.
         ///
         /// If `xs` is empty, the output is also empty.
         ///
@@ -801,8 +793,8 @@ exhaustive_tuples_1_input!(
 
 /// Defines exhaustive tuple generators.
 ///
-/// Malachite provides [`exhaustive_pairs`] and [`exhaustive_pairs_custom_output`], but you can
-/// also define `exhaustive_triples`, `exhaustive_quadruples`, and so on, and
+/// Malachite provides [`exhaustive_pairs`] and [`exhaustive_pairs_custom_output`], but you can also
+/// define `exhaustive_triples`, `exhaustive_quadruples`, and so on, and
 /// `exhaustive_triples_custom_output`, `exhaustive_quadruples_custom_output`, and so on, in your
 /// program using the code below. The documentation for [`exhaustive_pairs`] and
 /// [`exhaustive_pairs_custom_output`] describes these other functions as well.
@@ -811,9 +803,7 @@ exhaustive_tuples_1_input!(
 /// [here](self#exhaustive_pairs_custom_output).
 ///
 /// ```
-/// # #[macro_use]
-/// # extern crate malachite_base;
-/// # fn main() {
+/// use malachite_base::exhaustive_tuples;
 /// use malachite_base::iterators::bit_distributor::{BitDistributor, BitDistributorOutputType};
 /// use malachite_base::iterators::iterator_cache::IteratorCache;
 /// use malachite_base::num::conversion::traits::{ExactFrom, WrappingFrom};
@@ -889,7 +879,6 @@ exhaustive_tuples_1_input!(
 ///     [6, T, O, ts, ts_done, output_type_t],
 ///     [7, S, P, ss, ss_done, output_type_s]
 /// );
-/// # }
 /// ```
 #[macro_export]
 macro_rules! exhaustive_tuples {
@@ -998,11 +987,9 @@ macro_rules! exhaustive_tuples {
         /// Generates all $n$-tuples with elements from $n$ iterators, possibly with different
         /// output growth rates.
         ///
-        /// The $i$th `output_type_[x_i]` parameter is a
-        /// [`BitDistributorOutputType`](BitDistributorOutputType) that determines how quickly the
-        /// $i$th output slot advances through its iterator; see the
-        /// [`BitDistributor`](BitDistributor) documentation for a description of the different
-        /// types.
+        /// The $i$th `output_type_[x_i]` parameter is a [`BitDistributorOutputType`] that
+        /// determines how quickly the $i$th output slot advances through its iterator; see the
+        /// [`BitDistributor`] documentation for a description of the different types.
         ///
         /// If all of `xs`, `ys`, `zs`, ... are finite, the output length is the product of their
         /// lengths. If any of `xs`, `ys`, `zs`, ... are infinite, the output is also infinite.
@@ -1105,9 +1092,7 @@ exhaustive_tuples!(
 /// [here](self#exhaustive_triples_xyx_custom_output).
 ///
 /// ```
-/// # #[macro_use]
-/// # extern crate malachite_base;
-/// # fn main() {
+/// use malachite_base::custom_tuples;
 /// use malachite_base::iterators::bit_distributor::{BitDistributor, BitDistributorOutputType};
 /// use malachite_base::iterators::iterator_cache::IteratorCache;
 /// use malachite_base::num::conversion::traits::{ExactFrom, WrappingFrom};
@@ -1134,7 +1119,7 @@ exhaustive_tuples!(
 /// }
 ///
 /// custom_tuples!(
-///     (pub(crate)),
+///     (pub),
 ///     ExhaustiveTriplesXXY,
 ///     (X, X, Y),
 ///     (None, None, None),
@@ -1145,7 +1130,7 @@ exhaustive_tuples!(
 ///     [Y, J, ys, ys_done, [2, output_type_ys_2]]
 /// );
 /// custom_tuples!(
-///     (pub(crate)),
+///     (pub),
 ///     ExhaustiveTriplesXYX,
 ///     (X, Y, X),
 ///     (None, None, None),
@@ -1156,7 +1141,7 @@ exhaustive_tuples!(
 ///     [Y, J, ys, ys_done, [1, output_type_xs_2]]
 /// );
 /// custom_tuples!(
-///     (pub(crate)),
+///     (pub),
 ///     ExhaustiveTriplesXYY,
 ///     (X, Y, Y),
 ///     (None, None, None),
@@ -1167,7 +1152,7 @@ exhaustive_tuples!(
 ///     [Y, J, ys, ys_done, [1, output_type_ys_1], [2, output_type_ys_2]]
 /// );
 /// custom_tuples!(
-///     (pub(crate)),
+///     (pub),
 ///     ExhaustiveQuadruplesXXXY,
 ///     (X, X, X, Y),
 ///     (None, None, None, None),
@@ -1178,7 +1163,7 @@ exhaustive_tuples!(
 ///     [Y, J, ys, ys_done, [3, output_type_ys_3]]
 /// );
 /// custom_tuples!(
-///     (pub(crate)),
+///     (pub),
 ///     ExhaustiveQuadruplesXXYX,
 ///     (X, X, Y, X),
 ///     (None, None, None, None),
@@ -1189,7 +1174,7 @@ exhaustive_tuples!(
 ///     [Y, J, ys, ys_done, [2, output_type_ys_2]]
 /// );
 /// custom_tuples!(
-///     (pub(crate)),
+///     (pub),
 ///     ExhaustiveQuadruplesXXYZ,
 ///     (X, X, Y, Z),
 ///     (None, None, None, None),
@@ -1201,7 +1186,7 @@ exhaustive_tuples!(
 ///     [Z, K, zs, zs_done, [3, output_type_zs_3]]
 /// );
 /// custom_tuples!(
-///     (pub(crate)),
+///     (pub),
 ///     ExhaustiveQuadruplesXYXZ,
 ///     (X, Y, X, Z),
 ///     (None, None, None, None),
@@ -1213,7 +1198,7 @@ exhaustive_tuples!(
 ///     [Z, K, zs, zs_done, [3, output_type_zs_3]]
 /// );
 /// custom_tuples!(
-///     (pub(crate)),
+///     (pub),
 ///     ExhaustiveQuadruplesXYYX,
 ///     (X, Y, Y, X),
 ///     (None, None, None, None),
@@ -1224,7 +1209,7 @@ exhaustive_tuples!(
 ///     [Y, J, ys, ys_done, [1, output_type_ys_1], [2, output_type_ys_2]]
 /// );
 /// custom_tuples!(
-///     (pub(crate)),
+///     (pub),
 ///     ExhaustiveQuadruplesXYYZ,
 ///     (X, Y, Y, Z),
 ///     (None, None, None, None),
@@ -1236,7 +1221,7 @@ exhaustive_tuples!(
 ///     [Z, K, zs, zs_done, [3, output_type_zs_3]]
 /// );
 /// custom_tuples!(
-///     (pub(crate)),
+///     (pub),
 ///     ExhaustiveQuadruplesXYZZ,
 ///     (X, Y, Z, Z),
 ///     (None, None, None, None),
@@ -1248,7 +1233,7 @@ exhaustive_tuples!(
 ///     [Z, K, zs, zs_done, [2, output_type_zs_2], [3, output_type_zs_3]]
 /// );
 /// custom_tuples!(
-///     (pub(crate)),
+///     (pub),
 ///     ExhaustiveQuintuplesXYYYZ,
 ///     (X, Y, Y, Y, Z),
 ///     (None, None, None, None, None),
@@ -1259,7 +1244,6 @@ exhaustive_tuples!(
 ///     [Y, J, ys, ys_done, [1, output_type_ys_1], [2, output_type_ys_2], [3, output_type_ys_3]],
 ///     [Z, K, zs, zs_done, [4, output_type_zs_4]]
 /// );
-/// # }
 /// ```
 #[macro_export]
 macro_rules! custom_tuples {
@@ -1277,8 +1261,8 @@ macro_rules! custom_tuples {
         //
         // The mapping from iterators to tuple slots is indicated by the struct name; for example,
         // in `TriplesXYX` there are two iterators, `X`, and `Y`; `X` generates the elements in the
-        // first and third slots of the output triples, and `Y` generates the elements in the
-        // second slots.
+        // first and third slots of the output triples, and `Y` generates the elements in the second
+        // slots.
         #[derive(Clone, Debug)]
         $($vis)* struct $exhaustive_struct<$($t: Clone, $it: Iterator<Item = $t>,)*> {
             i: u64,
@@ -1380,9 +1364,9 @@ macro_rules! custom_tuples {
         // Generates all $n$-tuples with elements from $m$ iterators, where $m \leq n$, possibly
         // with different output growth rates.
         //
-        // The mapping from iterators to tuple slots is indicated by the function name; for
-        // example, `exhaustive_triples_xyx_custom_output` takes two iterators, `xs`, and `ys`;
-        // `xs` generates the elements in the first and third slots of the output triples, and `ys`
+        // The mapping from iterators to tuple slots is indicated by the function name; for example,
+        // `exhaustive_triples_xyx_custom_output` takes two iterators, `xs`, and `ys`; `xs`
+        // generates the elements in the first and third slots of the output triples, and `ys`
         // generates the elements in the second slots.
         //
         // Let $i$ be the index of the input iterators and $j$ be the index of the output slots. So
@@ -1391,9 +1375,9 @@ macro_rules! custom_tuples {
         //
         // The $j$th `output_type_[i_j]` parameter is a
         // [`BitDistributorOutputType`](crate::iterators::bit_distributor::BitDistributorOutputType)
-        // that determines how quickly the $j$th output slot advances through its iterator; see
-        // the [`BitDistributor`](crate::iterators::bit_distributor::BitDistributor) documentation
-        // for a description of the different types.
+        // that determines how quickly the $j$th output slot advances through its iterator; see the
+        // [`BitDistributor`](crate::iterators::bit_distributor::BitDistributor) documentation for a
+        // description of the different types.
         //
         // If all of `xs`, `ys`, `zs`, ... are finite, then the output length may be obtained by
         // raising the length of each input iterator to power of the number of outputs it maps to,
@@ -1423,10 +1407,10 @@ macro_rules! custom_tuples {
 
         // Generates all $n$-tuples with elements from $m$ iterators, where $m \leq n$.
         //
-        // The mapping from iterators to tuple slots is indicated by the function name; for
-        // example, `exhaustive_triples_xyx` takes two iterators, `xs`, and `ys`; `xs` generates
-        // the elements in the first and third slots of the output triples, and `ys` generates the
-        // elements in the second slots.
+        // The mapping from iterators to tuple slots is indicated by the function name; for example,
+        // `exhaustive_triples_xyx` takes two iterators, `xs`, and `ys`; `xs` generates the elements
+        // in the first and third slots of the output triples, and `ys` generates the elements in
+        // the second slots.
         //
         // If all of `xs`, `ys`, `zs`, ... are finite, then the output length may be obtained by
         // raising the length of each input iterator to power of the number of outputs it maps to,
@@ -1537,9 +1521,8 @@ impl<
                 if self.advance_xs() {
                     self.done = true;
                     return None;
-                } else {
-                    new_ys = true;
                 }
+                new_ys = true;
             }
             loop {
                 if let Some(y) = self.ys.as_mut().unwrap().next() {
@@ -1547,9 +1530,8 @@ impl<
                 } else if self.stop_after_empty_ys && new_ys || self.advance_xs() {
                     self.done = true;
                     return None;
-                } else {
-                    new_ys = true;
                 }
+                new_ys = true;
             }
         }
     }
@@ -1577,13 +1559,11 @@ impl<
 ///
 /// # Examples
 /// ```
-/// #[macro_use]
-/// extern crate maplit;
-///
 /// use itertools::Itertools;
 /// use malachite_base::tuples::exhaustive::{
 ///     lex_dependent_pairs, ExhaustiveDependentPairsYsGenerator,
 /// };
+/// use maplit::hashmap;
 /// use std::collections::HashMap;
 /// use std::hash::Hash;
 /// use std::iter::Cloned;
@@ -1604,64 +1584,52 @@ impl<
 ///     }
 /// }
 ///
-/// fn main() {
-///     let xs = ["a", "b", "c", "b", "a"].iter().cloned();
-///     let xss = lex_dependent_pairs(
-///         xs,
-///         DPGeneratorFromMap {
-///             map: hashmap! {
-///                 "a" => &[2, 3, 4][..],
-///                 "b" => &[20][..],
-///                 "c" => &[30, 40][..]
-///             },
+/// let xs = ["a", "b", "c", "b", "a"].iter().cloned();
+/// let xss = lex_dependent_pairs(
+///     xs,
+///     DPGeneratorFromMap {
+///         map: hashmap! {
+///             "a" => &[2, 3, 4][..],
+///             "b" => &[20][..],
+///             "c" => &[30, 40][..]
 ///         },
-///     )
-///     .take(20)
-///     .collect_vec();
-///     assert_eq!(
-///         xss.as_slice(),
-///         &[
-///             ("a", 2),
-///             ("a", 3),
-///             ("a", 4),
-///             ("b", 20),
-///             ("c", 30),
-///             ("c", 40),
-///             ("b", 20),
-///             ("a", 2),
-///             ("a", 3),
-///             ("a", 4)
-///         ]
-///     );
+///     },
+/// )
+/// .take(20)
+/// .collect_vec();
+/// assert_eq!(
+///     xss.as_slice(),
+///     &[
+///         ("a", 2),
+///         ("a", 3),
+///         ("a", 4),
+///         ("b", 20),
+///         ("c", 30),
+///         ("c", 40),
+///         ("b", 20),
+///         ("a", 2),
+///         ("a", 3),
+///         ("a", 4)
+///     ]
+/// );
 ///
-///     let xs = [1, 2, 3, 2, 3, 2, 2].iter().cloned();
-///     let xss = lex_dependent_pairs(
-///         xs,
-///         DPGeneratorFromMap {
-///             map: hashmap! {
-///                 1 => &[100, 101, 102][..],
-///                 2 => &[][..],
-///                 3 => &[300, 301, 302][..]
-///             },
+/// let xs = [1, 2, 3, 2, 3, 2, 2].iter().cloned();
+/// let xss = lex_dependent_pairs(
+///     xs,
+///     DPGeneratorFromMap {
+///         map: hashmap! {
+///             1 => &[100, 101, 102][..],
+///             2 => &[][..],
+///             3 => &[300, 301, 302][..]
 ///         },
-///     )
-///     .take(20)
-///     .collect_vec();
-///     assert_eq!(
-///         xss.as_slice(),
-///         &[
-///             (1, 100),
-///             (1, 101),
-///             (1, 102),
-///             (3, 300),
-///             (3, 301),
-///             (3, 302),
-///             (3, 300),
-///             (3, 301),
-///             (3, 302),
-///         ]
-///     );
-/// }
+///     },
+/// )
+/// .take(20)
+/// .collect_vec();
+/// assert_eq!(
+///     xss.as_slice(),
+///     &[(1, 100), (1, 101), (1, 102), (3, 300), (3, 301), (3, 302), (3, 300), (3, 301), (3, 302),]
+/// );
 /// ```
 #[inline]
 pub const fn lex_dependent_pairs<
@@ -1695,13 +1663,11 @@ pub const fn lex_dependent_pairs<
 ///
 /// # Examples
 /// ```
-/// #[macro_use]
-/// extern crate maplit;
-///
 /// use itertools::Itertools;
 /// use malachite_base::tuples::exhaustive::{
 ///     lex_dependent_pairs_stop_after_empty_ys, ExhaustiveDependentPairsYsGenerator,
 /// };
+/// use maplit::hashmap;
 /// use std::collections::HashMap;
 /// use std::hash::Hash;
 /// use std::iter::Cloned;
@@ -1722,23 +1688,21 @@ pub const fn lex_dependent_pairs<
 ///     }
 /// }
 ///
-/// fn main() {
-///     let xs = [1, 2, 3, 2, 3, 2, 2].iter().cloned();
-///     let xss = lex_dependent_pairs_stop_after_empty_ys(
-///         xs,
-///         DPGeneratorFromMap {
-///             map: hashmap! {
-///                 1 => &[100, 101, 102][..],
-///                 2 => &[][..],
-///                 3 => &[300, 301, 302][..]
-///             },
+/// let xs = [1, 2, 3, 2, 3, 2, 2].iter().cloned();
+/// let xss = lex_dependent_pairs_stop_after_empty_ys(
+///     xs,
+///     DPGeneratorFromMap {
+///         map: hashmap! {
+///             1 => &[100, 101, 102][..],
+///             2 => &[][..],
+///             3 => &[300, 301, 302][..]
 ///         },
-///     )
-///     .take(20)
-///     .collect_vec();
-///     // Stops after seeing 2, which maps to an empty iterator
-///     assert_eq!(xss.as_slice(), &[(1, 100), (1, 101), (1, 102)]);
-/// }
+///     },
+/// )
+/// .take(20)
+/// .collect_vec();
+/// // Stops after seeing 2, which maps to an empty iterator
+/// assert_eq!(xss.as_slice(), &[(1, 100), (1, 101), (1, 102)]);
 /// ```
 #[inline]
 pub const fn lex_dependent_pairs_stop_after_empty_ys<
@@ -1825,12 +1789,11 @@ impl<
                 } else if self.stop_after_empty_ys && t.2 {
                     self.done = true;
                     return None;
-                } else {
-                    self.xs_yss.remove(i);
-                    if self.xs_done && self.xs_yss.is_empty() {
-                        self.done = true;
-                        return None;
-                    }
+                }
+                self.xs_yss.remove(i);
+                if self.xs_done && self.xs_yss.is_empty() {
+                    self.done = true;
+                    return None;
                 }
             }
         }
@@ -1860,22 +1823,20 @@ impl<
 ///
 /// # Examples
 /// ```
-/// #[macro_use]
-/// extern crate maplit;
-///
 /// use itertools::Itertools;
 /// use malachite_base::num::exhaustive::exhaustive_positive_primitive_ints;
 /// use malachite_base::num::iterators::ruler_sequence;
 /// use malachite_base::tuples::exhaustive::{
 ///     exhaustive_dependent_pairs, ExhaustiveDependentPairsYsGenerator,
 /// };
+/// use maplit::hashmap;
 /// use std::collections::HashMap;
 /// use std::hash::Hash;
 /// use std::iter::Cloned;
 /// use std::slice::Iter;
 ///
 /// #[derive(Clone, Debug)]
-/// struct MultiplesGeneratorHelper {
+/// pub struct MultiplesGeneratorHelper {
 ///     u: u64,
 ///     step: u64,
 /// }
@@ -1917,97 +1878,87 @@ impl<
 ///     }
 /// }
 ///
-/// fn main() {
-///     // All (x, y) where x is a positive natural and y is a positive multiple of x. It would be
-///     // easier to do
-///     //
-///     // exhaustive_pairs_from_single(exhaustive_positive_primitive_ints::<u64>())
-///     //      .map(|(x, y)| (x, x * y))
-///     //
-///     // in this case.
-///     let xs = exhaustive_positive_primitive_ints::<u64>();
-///     let xss = exhaustive_dependent_pairs(ruler_sequence(), xs.clone(), MultiplesGenerator {})
-///         .take(50)
-///         .collect_vec();
-///     assert_eq!(
-///         xss.as_slice(),
-///         &[
-///             (1, 1),
-///             (2, 2),
-///             (1, 2),
-///             (3, 3),
-///             (1, 3),
-///             (2, 4),
-///             (1, 4),
-///             (4, 4),
-///             (1, 5),
-///             (2, 6),
-///             (1, 6),
-///             (3, 6),
-///             (1, 7),
-///             (2, 8),
-///             (1, 8),
-///             (5, 5),
-///             (1, 9),
-///             (2, 10),
-///             (1, 10),
-///             (3, 9),
-///             (1, 11),
-///             (2, 12),
-///             (1, 12),
-///             (4, 8),
-///             (1, 13),
-///             (2, 14),
-///             (1, 14),
-///             (3, 12),
-///             (1, 15),
-///             (2, 16),
-///             (1, 16),
-///             (6, 6),
-///             (1, 17),
-///             (2, 18),
-///             (1, 18),
-///             (3, 15),
-///             (1, 19),
-///             (2, 20),
-///             (1, 20),
-///             (4, 12),
-///             (1, 21),
-///             (2, 22),
-///             (1, 22),
-///             (3, 18),
-///             (1, 23),
-///             (2, 24),
-///             (1, 24),
-///             (5, 10),
-///             (1, 25),
-///             (2, 26)
-///         ]
-///     );
+/// // All (x, y) where x is a positive natural and y is a positive multiple of x. It would be
+/// // easier to do
+/// //
+/// // exhaustive_pairs_from_single(exhaustive_positive_primitive_ints::<u64>())
+/// //      .map(|(x, y)| (x, x * y))
+/// //
+/// // in this case.
+/// let xs = exhaustive_positive_primitive_ints::<u64>();
+/// let xss = exhaustive_dependent_pairs(ruler_sequence(), xs.clone(), MultiplesGenerator {})
+///     .take(50)
+///     .collect_vec();
+/// assert_eq!(
+///     xss.as_slice(),
+///     &[
+///         (1, 1),
+///         (2, 2),
+///         (1, 2),
+///         (3, 3),
+///         (1, 3),
+///         (2, 4),
+///         (1, 4),
+///         (4, 4),
+///         (1, 5),
+///         (2, 6),
+///         (1, 6),
+///         (3, 6),
+///         (1, 7),
+///         (2, 8),
+///         (1, 8),
+///         (5, 5),
+///         (1, 9),
+///         (2, 10),
+///         (1, 10),
+///         (3, 9),
+///         (1, 11),
+///         (2, 12),
+///         (1, 12),
+///         (4, 8),
+///         (1, 13),
+///         (2, 14),
+///         (1, 14),
+///         (3, 12),
+///         (1, 15),
+///         (2, 16),
+///         (1, 16),
+///         (6, 6),
+///         (1, 17),
+///         (2, 18),
+///         (1, 18),
+///         (3, 15),
+///         (1, 19),
+///         (2, 20),
+///         (1, 20),
+///         (4, 12),
+///         (1, 21),
+///         (2, 22),
+///         (1, 22),
+///         (3, 18),
+///         (1, 23),
+///         (2, 24),
+///         (1, 24),
+///         (5, 10),
+///         (1, 25),
+///         (2, 26)
+///     ]
+/// );
 ///
-///     let xs = [1, 2, 3, 2, 3, 2, 2].iter().cloned();
-///     let xss = exhaustive_dependent_pairs(
-///         ruler_sequence(),
-///         xs,
-///         DPGeneratorFromMap {
-///             map: hashmap! { 1 => &[100, 101, 102][..], 2 => &[][..], 3 => &[300, 301, 302][..] }
-///         }
-///     ).take(20).collect_vec();
-///     assert_eq!(
-///         xss.as_slice(),
-///         &[
-///             (1, 100),
-///             (3, 300),
-///             (1, 101),
-///             (3, 300),
-///             (1, 102),
-///             (3, 301),
-///             (3, 302),
-///             (3, 301),
-///             (3, 302)
-///         ]
-///     );
-/// }
+/// let xs = [1, 2, 3, 2, 3, 2, 2].iter().cloned();
+/// let xss = exhaustive_dependent_pairs(
+///     ruler_sequence(),
+///     xs,
+///     DPGeneratorFromMap {
+///         map: hashmap! { 1 => &[100, 101, 102][..], 2 => &[][..], 3 => &[300, 301, 302][..] },
+///     },
+/// )
+/// .take(20)
+/// .collect_vec();
+/// assert_eq!(
+///     xss.as_slice(),
+///     &[(1, 100), (3, 300), (1, 101), (3, 300), (1, 102), (3, 301), (3, 302), (3, 301), (3, 302)]
+/// );
 /// ```
 #[inline]
 pub const fn exhaustive_dependent_pairs<
@@ -2044,21 +1995,19 @@ pub const fn exhaustive_dependent_pairs<
 ///
 /// # Examples
 /// ```
-/// #[macro_use]
-/// extern crate maplit;
-///
 /// use itertools::Itertools;
 /// use malachite_base::num::iterators::ruler_sequence;
 /// use malachite_base::tuples::exhaustive::{
 ///     exhaustive_dependent_pairs_stop_after_empty_ys, ExhaustiveDependentPairsYsGenerator,
 /// };
+/// use maplit::hashmap;
 /// use std::collections::HashMap;
 /// use std::hash::Hash;
 /// use std::iter::Cloned;
 /// use std::slice::Iter;
 ///
 /// #[derive(Clone, Debug)]
-/// struct MultiplesGeneratorHelper {
+/// pub struct MultiplesGeneratorHelper {
 ///     u: u64,
 ///     step: u64,
 /// }
@@ -2088,23 +2037,21 @@ pub const fn exhaustive_dependent_pairs<
 ///     }
 /// }
 ///
-/// fn main() {
-///     let xs = [1, 2, 3, 2, 3, 2, 2].iter().cloned();
-///     let xss = exhaustive_dependent_pairs_stop_after_empty_ys(
-///         ruler_sequence(),
-///         xs,
-///         DPGeneratorFromMap {
-///             map: hashmap! {
-///                 1 => &[100, 101, 102][..],
-///                 2 => &[][..],
-///                 3 => &[300, 301, 302][..]
-///             },
+/// let xs = [1, 2, 3, 2, 3, 2, 2].iter().cloned();
+/// let xss = exhaustive_dependent_pairs_stop_after_empty_ys(
+///     ruler_sequence(),
+///     xs,
+///     DPGeneratorFromMap {
+///         map: hashmap! {
+///             1 => &[100, 101, 102][..],
+///             2 => &[][..],
+///             3 => &[300, 301, 302][..]
 ///         },
-///     )
-///     .take(20)
-///     .collect_vec();
-///     assert_eq!(xss.as_slice(), &[(1, 100)]);
-/// }
+///     },
+/// )
+/// .take(20)
+/// .collect_vec();
+/// assert_eq!(xss.as_slice(), &[(1, 100)]);
 /// ```
 #[inline]
 pub const fn exhaustive_dependent_pairs_stop_after_empty_ys<
@@ -2140,10 +2087,8 @@ pub const fn exhaustive_dependent_pairs_stop_after_empty_ys<
 /// See usage examples [here](self#lex_ordered_unique_quadruples).
 ///
 /// ```
-/// # #[macro_use]
-/// # extern crate malachite_base;
-/// # fn main() {
 /// use malachite_base::iterators::iterator_cache::IteratorCache;
+/// use malachite_base::lex_ordered_unique_tuples;
 /// use malachite_base::vecs::exhaustive::fixed_length_ordered_unique_indices_helper;
 /// use std::marker::PhantomData;
 ///
@@ -2212,7 +2157,6 @@ pub const fn exhaustive_dependent_pairs_stop_after_empty_ys<
 ///     lex_ordered_unique_octuples,
 ///     [0, 1, 2, 3, 4, 5, 6, 7]
 /// );
-/// # }
 /// ```
 #[macro_export]
 macro_rules! lex_ordered_unique_tuples {
@@ -2231,8 +2175,8 @@ macro_rules! lex_ordered_unique_tuples {
         /// Generates all $k$-tuples of elements from an iterator, where the tuples have no
         /// repetitions and are ordered the same way as in the iterator.
         ///
-        /// The tuples are generated in lexicographic order with respect to the order of the
-        /// element iterator.
+        /// The tuples are generated in lexicographic order with respect to the order of the element
+        /// iterator.
         #[derive(Clone, Debug)]
         $($vis)* struct $struct<I: Iterator> where I::Item: Clone {
             first: bool,
@@ -2281,8 +2225,8 @@ macro_rules! lex_ordered_unique_tuples {
         /// [`lex_ordered_unique_tuples`] for more information.
         ///
         /// Generates $k$-tuples of elements from a single iterator, such that each tuple has no
-        /// repeated elements, and the elements in each [`Vec`] are ordered the same way as they
-        /// are in the source iterator.
+        /// repeated elements, and the elements in each [`Vec`] are ordered the same way as they are
+        /// in the source iterator.
         ///
         /// The source iterator should not repeat any elements, but this is not enforced.
         ///
@@ -2327,9 +2271,7 @@ lex_ordered_unique_tuples!(
 /// See usage examples [here](self#exhaustive_ordered_unique_quadruples).
 ///
 /// ```
-/// # #[macro_use]
-/// # extern crate malachite_base;
-/// # fn main() {
+/// use malachite_base::exhaustive_ordered_unique_tuples;
 /// use malachite_base::iterators::iterator_cache::IteratorCache;
 /// use malachite_base::vecs::exhaustive::next_bit_pattern;
 ///
@@ -2398,7 +2340,6 @@ lex_ordered_unique_tuples!(
 ///     exhaustive_ordered_unique_octuples,
 ///     [0, 1, 2, 3, 4, 5, 6, 7]
 /// );
-/// # }
 /// ```
 #[macro_export]
 macro_rules! exhaustive_ordered_unique_tuples {
@@ -2462,8 +2403,8 @@ macro_rules! exhaustive_ordered_unique_tuples {
         /// See [`exhaustive_ordered_unique_tuples`] for more information.
         ///
         /// Generates $k$-tuples of elements from a single iterator, such that each tuple has no
-        /// repeated elements, and the elements in each [`Vec`] are ordered the same way as they
-        /// are in the source iterator.
+        /// repeated elements, and the elements in each [`Vec`] are ordered the same way as they are
+        /// in the source iterator.
         ///
         /// The source iterator should not repeat any elements, but this is not enforced.
         ///
@@ -2506,11 +2447,9 @@ exhaustive_ordered_unique_tuples!(
 /// See usage examples [here](self#lex_unique_pairs).
 ///
 /// ```
-/// # #[macro_use]
-/// # extern crate malachite_base;
-/// # fn main() {
 /// use malachite_base::iterators::iterator_cache::IteratorCache;
-/// use malachite_base::vecs::exhaustive::{UniqueIndices, unique_indices};
+/// use malachite_base::lex_unique_tuples;
+/// use malachite_base::vecs::exhaustive::{unique_indices, UniqueIndices};
 ///
 /// lex_unique_tuples!(
 ///     (pub(crate)),
@@ -2577,7 +2516,6 @@ exhaustive_ordered_unique_tuples!(
 ///     lex_unique_octuples,
 ///     [0, 1, 2, 3, 4, 5, 6, 7]
 /// );
-/// # }
 /// ```
 #[macro_export]
 macro_rules! lex_unique_tuples {
@@ -2589,15 +2527,14 @@ macro_rules! lex_unique_tuples {
         $fn: ident,
         [$($i: expr),*]
     ) => {
-        /// This documentation applies not only to `LexUniquePairs`, but also to
-        /// `LexUniqueTriples`, `LexUniqueQuadruples`, and so on. See [`lex_unique_tuples`] for
-        /// more information.
+        /// This documentation applies not only to `LexUniquePairs`, but also to `LexUniqueTriples`,
+        /// `LexUniqueQuadruples`, and so on. See [`lex_unique_tuples`] for more information.
         ///
         /// Generates all $k$-tuples of elements from an iterator, where the tuples have no
         /// repetitions.
         ///
-        /// The tuples are generated in lexicographic order with respect to the order of the
-        /// element iterator.
+        /// The tuples are generated in lexicographic order with respect to the order of the element
+        /// iterator.
         #[derive(Clone)]
         $($vis)* struct $struct<I: Iterator> where I::Item: Clone {
             first: bool,
@@ -2758,17 +2695,16 @@ where
 /// See usage examples [here](self#lex_unique_quadruples).
 ///
 /// ```
-/// # #[macro_use]
-/// # extern crate malachite_base;
-/// # fn main() {
-/// use malachite_base::num::iterators::{RulerSequence, ruler_sequence};
-/// use malachite_base::tuples::exhaustive::{ExhaustiveDependentPairs, exhaustive_dependent_pairs};
-/// use malachite_base::vecs::ExhaustiveVecPermutations;
-/// use malachite_base::vecs::exhaustive::{
-///     ExhaustiveOrderedUniqueCollections,
-///     ExhaustiveUniqueVecsGenerator,
-///     exhaustive_ordered_unique_vecs_fixed_length
+/// use malachite_base::exhaustive_unique_tuples;
+/// use malachite_base::num::iterators::{ruler_sequence, RulerSequence};
+/// use malachite_base::tuples::exhaustive::{
+///     exhaustive_dependent_pairs, ExhaustiveDependentPairs,
 /// };
+/// use malachite_base::vecs::exhaustive::{
+///     exhaustive_ordered_unique_vecs_fixed_length, ExhaustiveOrderedUniqueCollections,
+///     ExhaustiveUniqueVecsGenerator,
+/// };
+/// use malachite_base::vecs::ExhaustiveVecPermutations;
 ///
 /// exhaustive_unique_tuples!(
 ///     (pub(crate)),
@@ -2835,7 +2771,6 @@ where
 ///     exhaustive_unique_octuples,
 ///     [0, 1, 2, 3, 4, 5, 6, 7]
 /// );
-/// # }
 /// ```
 #[macro_export]
 macro_rules! exhaustive_unique_tuples {
