@@ -1,5 +1,13 @@
+// Copyright © 2024 Mikhail Hogrefe
+//
+// This file is part of Malachite.
+//
+// Malachite is free software: you can redistribute it and/or modify it under the terms of the GNU
+// Lesser General Public License (LGPL) as published by the Free Software Foundation; either version
+// 3 of the License, or (at your option) any later version. See <https://www.gnu.org/licenses/>.
+
 use crate::random::Seed;
-use std::cmp::Ordering;
+use std::cmp::Ordering::*;
 use std::iter::{repeat, Repeat};
 
 /// Generates random units; repeats `()`.
@@ -40,11 +48,8 @@ pub fn next_helper<I: Iterator>(x: &mut I, _i: usize) -> Option<I::Item> {
 /// See usage examples [here](self#random_pairs) and [here](self#random_pairs_from_single).
 ///
 /// ```
-/// # #[macro_use]
-/// # extern crate malachite_base;
-/// # fn main() {
-/// use malachite_base::num::random::{random_unsigned_range, RandomUnsignedRange};
 /// use malachite_base::random::Seed;
+/// use malachite_base::random_tuples;
 /// use malachite_base::tuples::random::next_helper;
 ///
 /// random_tuples!(
@@ -145,7 +150,6 @@ pub fn next_helper<I: Iterator>(x: &mut I, _i: usize) -> Option<I::Item> {
 ///     [6, T, O, ts, ts_gen],
 ///     [7, S, P, ss, ss_gen]
 /// );
-/// # }
 /// ```
 #[macro_export]
 macro_rules! random_tuples {
@@ -261,10 +265,8 @@ random_tuples!(
 /// See usage examples [here](self#random_triples_xyx).
 ///
 /// ```
-/// # #[macro_use]
-/// # extern crate malachite_base;
-/// # fn main() {
 /// use malachite_base::random::Seed;
+/// use malachite_base::random_custom_tuples;
 ///
 /// random_custom_tuples!(
 ///     (pub(crate)),
@@ -359,7 +361,6 @@ random_tuples!(
 ///     [Y, J, ys, ys_gen, [y_1, y_1], [y_2, y_2], [y_3, y_3]],
 ///     [Z, K, zs, zs_gen, [z_4, z_4]]
 /// );
-/// # }
 /// ```
 #[macro_export]
 macro_rules! random_custom_tuples {
@@ -374,8 +375,8 @@ macro_rules! random_custom_tuples {
         //
         // The mapping from iterators to tuple slots is indicated by the struct name; for example,
         // in `RandomTriplesXYX` there are two iterators, `X`, and `Y`; `X` generates the elements
-        // in the first and third slots of the output triples, and `Y` generates the elements in
-        // the second slots.
+        // in the first and third slots of the output triples, and `Y` generates the elements in the
+        // second slots.
         #[derive(Clone, Debug)]
         $($vis)* struct $random_struct<$($t: Clone, $it: Iterator<Item = $t>,)*> {
             $($xs: $it,)*
@@ -397,10 +398,10 @@ macro_rules! random_custom_tuples {
 
         // Generates random $n$-tuples with elements from $m$ iterators, where $m \leq n$.
         //
-        // The mapping from iterators to tuple slots is indicated by the function name; for
-        // example, `random_triples_xyx` takes two iterators, `xs`, and `ys`; `xs` generates the
-        // elements in the first and third slots of the output triples, and `ys` generates the
-        // elements in the second slots.
+        // The mapping from iterators to tuple slots is indicated by the function name; for example,
+        // `random_triples_xyx` takes two iterators, `xs`, and `ys`; `xs` generates the elements in
+        // the first and third slots of the output triples, and `ys` generates the elements in the
+        // second slots.
         //
         // The probability of a particular $n$-tuple being generated is the product of the
         // probabilities of each of its elements.
@@ -446,12 +447,12 @@ where
                 out_0 = Some(x);
             } else {
                 match x.cmp(out_0.as_ref().unwrap()) {
-                    Ordering::Equal => {}
-                    Ordering::Greater => {
+                    Equal => {}
+                    Greater => {
                         out_1 = x;
                         break;
                     }
-                    Ordering::Less => {
+                    Less => {
                         out_1 = out_0.unwrap();
                         out_0 = Some(x);
                         break;
@@ -473,8 +474,8 @@ where
 /// P((x\_0, x\_1)) = 2P(x\_0)P(x\_1).
 /// $$
 ///
-/// The above formula assumes that the pair is valid, \emph{i.e.} its first element is less than
-/// its second. The probability of an invalid pair is zero.
+/// The above formula assumes that the pair is valid, \emph{i.e.} its first element is less than its
+/// second. The probability of an invalid pair is zero.
 ///
 /// `xs` must be infinite.
 #[inline]
@@ -494,12 +495,9 @@ where
 /// See usage examples [here](self#random_ordered_unique_quadruples).
 ///
 /// ```
-/// # #[macro_use]
-/// # extern crate malachite_base;
-/// # fn main() {
+/// use malachite_base::random_ordered_unique_tuples;
 /// use malachite_base::sets::random::{
-///     random_b_tree_sets_fixed_length,
-///     RandomBTreeSetsFixedLength
+///     random_b_tree_sets_fixed_length, RandomBTreeSetsFixedLength,
 /// };
 ///
 /// random_ordered_unique_tuples!(
@@ -567,7 +565,6 @@ where
 ///     random_ordered_unique_octuples,
 ///     [0, 1, 2, 3, 4, 5, 6, 7]
 /// );
-/// # }
 /// ```
 #[macro_export]
 macro_rules! random_ordered_unique_tuples {
@@ -579,8 +576,8 @@ macro_rules! random_ordered_unique_tuples {
         $fn: ident,
         [$($i: expr),*]
     ) => {
-        // Generates random $n$-tuples using elements from a single iterator, where the tuples
-        // have no repeated elements, and the elements are in ascending order.
+        // Generates random $n$-tuples using elements from a single iterator, where the tuples have
+        // no repeated elements, and the elements are in ascending order.
         #[derive(Clone, Debug)]
         $($vis)* struct $struct<I: Iterator> where I::Item: Ord {
             xs: RandomBTreeSetsFixedLength<I>,
@@ -596,8 +593,8 @@ macro_rules! random_ordered_unique_tuples {
             }
         }
 
-        // Generates random $n$-tuples using elements from a single iterator, where the tuples
-        // have no repeated elements, and the elements are in ascending order.
+        // Generates random $n$-tuples using elements from a single iterator, where the tuples have
+        // no repeated elements, and the elements are in ascending order.
         //
         // The input iterator must generate at least `len` distinct elements; otherwise, this
         // iterator will hang.
@@ -606,8 +603,8 @@ macro_rules! random_ordered_unique_tuples {
         // P((x\_i)\_{i=0}^{n-1}) = n!\prod\_{i=0}^{n-1}P(x\_i).
         // $$
         //
-        // The above formula assumes that the tuple is valid, \emph{i.e.} its elements are
-        // strictly increasing. The probability of an invalid tuple is zero.
+        // The above formula assumes that the tuple is valid, \emph{i.e.} its elements are strictly
+        // increasing. The probability of an invalid tuple is zero.
         //
         // `xs` must be infinite.
         //
@@ -683,9 +680,7 @@ where
 /// See usage examples [here](self#random_unique_quadruples).
 ///
 /// ```
-/// # #[macro_use]
-/// # extern crate malachite_base;
-/// # fn main() {
+/// use malachite_base::random_unique_tuples;
 /// use std::collections::HashMap;
 /// use std::hash::Hash;
 ///
@@ -754,7 +749,6 @@ where
 ///     random_unique_octuples,
 ///     [0, 1, 2, 3, 4, 5, 6, 7]
 /// );
-/// # }
 /// ```
 #[macro_export]
 macro_rules! random_unique_tuples {
