@@ -1,10 +1,25 @@
+// Copyright © 2024 Mikhail Hogrefe
+//
+// This file is part of Malachite.
+//
+// Malachite is free software: you can redistribute it and/or modify it under the terms of the GNU
+// Lesser General Public License (LGPL) as published by the Free Software Foundation; either version
+// 3 of the License, or (at your option) any later version. See <https://www.gnu.org/licenses/>.
+
+#[cfg(feature = "random")]
 use crate::num::conversion::traits::ExactFrom;
+#[cfg(feature = "random")]
 use crate::num::random::{random_unsigneds_less_than, RandomUnsignedsLessThan};
+#[cfg(feature = "random")]
 use crate::random::Seed;
 use crate::slices::advance_indices;
+use alloc::string::String;
+use alloc::vec::Vec;
+use core::str::FromStr;
+#[cfg(feature = "random")]
 use rand::prelude::SliceRandom;
+#[cfg(feature = "random")]
 use rand_chacha::ChaCha20Rng;
-use std::str::FromStr;
 
 /// Inserts several copies of a value at the left (beginning) of a [`Vec`].
 ///
@@ -71,8 +86,8 @@ pub fn vec_delete_left<T: Copy>(xs: &mut Vec<T>, delete_size: usize) {
 ///
 /// Substrings representing `T`s may contain commas. Sometimes this may lead to ambiguities: for
 /// example, the two `Vec<&str>`s `vec!["a, b"]` and `vec!["a", "b"]` both have the string
-/// representation `"[a, b]"`. The parser is greedy, so it will interpet this string as
-/// `vec!["a", "b"]`.
+/// representation `"[a, b]"`. The parser is greedy, so it will interpet this string as `vec!["a",
+/// "b"]`.
 ///
 /// # Examples
 /// ```
@@ -81,7 +96,10 @@ pub fn vec_delete_left<T: Copy>(xs: &mut Vec<T>, delete_size: usize) {
 ///
 /// assert_eq!(vec_from_str::<Never>("[]"), Some(vec![]));
 /// assert_eq!(vec_from_str("[5, 6, 7]"), Some(vec![5, 6, 7]));
-/// assert_eq!(vec_from_str("[false, false, true]"), Some(vec![false, false, true]));
+/// assert_eq!(
+///     vec_from_str("[false, false, true]"),
+///     Some(vec![false, false, true])
+/// );
 /// assert_eq!(vec_from_str::<bool>("[false, false, true"), None);
 /// ```
 #[inline]
@@ -97,19 +115,19 @@ pub fn vec_from_str<T: FromStr>(src: &str) -> Option<Vec<T>> {
 ///
 /// Substrings representing `T`s may contain commas. Sometimes this may lead to ambiguities: for
 /// example, the two `Vec<&str>`s `vec!["a, b"]` and `vec!["a", "b"]` both have the string
-/// representation `"[a, b]"`. The parser is greedy, so it will interpet this string as
-/// `vec!["a", "b"]`.
+/// representation `"[a, b]"`. The parser is greedy, so it will interpet this string as `vec!["a",
+/// "b"]`.
 ///
 /// # Examples
 /// ```
 /// use malachite_base::options::option_from_str;
 /// use malachite_base::orderings::ordering_from_str;
 /// use malachite_base::vecs::{vec_from_str, vec_from_str_custom};
-/// use std::cmp::Ordering;
+/// use std::cmp::Ordering::*;
 ///
 /// assert_eq!(
 ///     vec_from_str_custom(&ordering_from_str, "[Less, Greater]"),
-///     Some(vec![Ordering::Less, Ordering::Greater]),
+///     Some(vec![Less, Greater]),
 /// );
 /// assert_eq!(
 ///     vec_from_str_custom(&option_from_str, "[Some(false), None]"),
@@ -147,6 +165,7 @@ pub fn vec_from_str_custom<T>(f: &dyn Fn(&str) -> Option<T>, src: &str) -> Optio
     }
 }
 
+#[cfg(feature = "random")]
 /// Uniformly generates a random value from a nonempty [`Vec`].
 ///
 /// This `struct` is created by [`random_values_from_vec`]; see its documentation for more.
@@ -156,6 +175,7 @@ pub struct RandomValuesFromVec<T: Clone> {
     indices: RandomUnsignedsLessThan<u64>,
 }
 
+#[cfg(feature = "random")]
 impl<T: Clone> Iterator for RandomValuesFromVec<T> {
     type Item = T;
 
@@ -165,6 +185,7 @@ impl<T: Clone> Iterator for RandomValuesFromVec<T> {
     }
 }
 
+#[cfg(feature = "random")]
 /// Uniformly generates a random value from a nonempty [`Vec`].
 ///
 /// The iterator owns the data. It may be more convenient for the iterator to return references to a
@@ -186,15 +207,15 @@ impl<T: Clone> Iterator for RandomValuesFromVec<T> {
 ///
 /// let xs = vec![2, 3, 5, 7, 11];
 /// assert_eq!(
-///     random_values_from_vec(EXAMPLE_SEED, xs).take(10).collect_vec(),
+///     random_values_from_vec(EXAMPLE_SEED, xs)
+///         .take(10)
+///         .collect_vec(),
 ///     &[3, 7, 3, 5, 11, 3, 5, 11, 2, 2]
 /// );
 /// ```
 #[inline]
 pub fn random_values_from_vec<T: Clone>(seed: Seed, xs: Vec<T>) -> RandomValuesFromVec<T> {
-    if xs.is_empty() {
-        panic!("empty Vec");
-    }
+    assert!(!xs.is_empty(), "empty Vec");
     let indices = random_unsigneds_less_than(seed, u64::exact_from(xs.len()));
     RandomValuesFromVec { xs, indices }
 }
@@ -227,8 +248,7 @@ impl<T: Clone> Iterator for ExhaustiveVecPermutations<T> {
 ///
 /// The permutations are [`Vec`]s of cloned items. It may be more convenient for the iterator to
 /// return references to a slice, in which case you may use
-/// [`exhaustive_slice_permutations`](crate::slices::exhaustive_slice_permutations).
-/// instead.
+/// [`exhaustive_slice_permutations`](crate::slices::exhaustive_slice_permutations) instead.
 ///
 /// The permutations are generated in lexicographic order with respect to the ordering in the
 /// [`Vec`].
@@ -261,6 +281,7 @@ pub fn exhaustive_vec_permutations<T: Clone>(xs: Vec<T>) -> ExhaustiveVecPermuta
     }
 }
 
+#[cfg(feature = "random")]
 /// Uniformly generates a random [`Vec`] of values cloned from an original [`Vec`].
 ///
 /// This `struct` is created by [`random_vec_permutations`]; see its documentation for more.
@@ -271,6 +292,7 @@ pub struct RandomVecPermutations<T: Clone> {
     rng: ChaCha20Rng,
 }
 
+#[cfg(feature = "random")]
 impl<T: Clone> Iterator for RandomVecPermutations<T> {
     type Item = Vec<T>;
 
@@ -280,6 +302,7 @@ impl<T: Clone> Iterator for RandomVecPermutations<T> {
     }
 }
 
+#[cfg(feature = "random")]
 /// Uniformly generates a random [`Vec`] of values cloned from an original [`Vec`].
 ///
 /// The permutations are [`Vec`]s of cloned items. It may be more convenient for the iterator to
@@ -349,7 +372,6 @@ pub fn random_vec_permutations<T: Clone>(seed: Seed, xs: Vec<T>) -> RandomVecPer
 /// ```
 /// use itertools::Itertools;
 /// use malachite_base::chars::exhaustive::exhaustive_ascii_chars;
-/// use malachite_base::iterators::bit_distributor::BitDistributorOutputType;
 /// use malachite_base::vecs::exhaustive::lex_vecs_fixed_length_2_inputs;
 ///
 /// // We are generating length-3 `Vec`s of `char`s using two input iterators. The first iterator
@@ -475,6 +497,7 @@ pub fn random_vec_permutations<T: Clone>(seed: Seed, xs: Vec<T>) -> RandomVecPer
 /// );
 /// ```
 pub mod exhaustive;
+#[cfg(feature = "random")]
 /// Iterators that generate [`Vec`]s randomly.
 ///
 /// # random_vecs_length_2
@@ -523,7 +546,6 @@ pub mod exhaustive;
 /// use itertools::Itertools;
 /// use malachite_base::chars::random::{random_ascii_chars, random_char_inclusive_range};
 /// use malachite_base::random::EXAMPLE_SEED;
-/// use malachite_base::strings::ToDebugString;
 /// use malachite_base::vecs::random::random_vecs_fixed_length_2_inputs;
 ///
 /// // We are generating length-3 `Vec`s of `char`s using two input iterators. The first iterator
