@@ -1,3 +1,11 @@
+// Copyright © 2024 Mikhail Hogrefe
+//
+// This file is part of Malachite.
+//
+// Malachite is free software: you can redistribute it and/or modify it under the terms of the GNU
+// Lesser General Public License (LGPL) as published by the Free Software Foundation; either version
+// 3 of the License, or (at your option) any later version. See <https://www.gnu.org/licenses/>.
+
 use crate::num::conversion::traits::ExactFrom;
 use crate::num::random::geometric::{
     geometric_random_unsigned_inclusive_range, geometric_random_unsigneds,
@@ -13,7 +21,7 @@ use crate::sets::random::{
     RandomBTreeSetsFixedLength,
 };
 use crate::vecs::exhaustive::validate_oi_map;
-use std::cmp::Ordering;
+use std::cmp::Ordering::*;
 use std::collections::HashMap;
 use std::hash::Hash;
 use std::iter::{repeat, Repeat};
@@ -95,10 +103,8 @@ pub const fn random_vecs_fixed_length_from_single<I: Iterator>(
 /// [here](self#random_vecs_fixed_length_2_inputs).
 ///
 /// ```
-/// # #[macro_use]
-/// # extern crate malachite_base;
-/// # fn main() {
 /// use malachite_base::random::Seed;
+/// use malachite_base::random_vecs_fixed_length;
 /// use malachite_base::vecs::exhaustive::validate_oi_map;
 ///
 /// random_vecs_fixed_length!(
@@ -170,7 +176,6 @@ pub const fn random_vecs_fixed_length_from_single<I: Iterator>(
 ///     [6, O, ts, ts_gen],
 ///     [7, P, ss, ss_gen]
 /// );
-/// # }
 /// ```
 #[macro_export]
 macro_rules! random_vecs_fixed_length {
@@ -218,11 +223,11 @@ macro_rules! random_vecs_fixed_length {
         }
 
         /// This documentation applies not only to `random_vecs_fixed_length_2_inputs`, but also to
-        /// `random_vecs_fixed_length_3_inputs`, `random_vecs_fixed_length_4_inputs`, and so on.
-        /// See [`random_vecs_fixed_length`] for more information.
+        /// `random_vecs_fixed_length_3_inputs`, `random_vecs_fixed_length_4_inputs`, and so on. See
+        /// [`random_vecs_fixed_length`] for more information.
         ///
-        /// Generates random length-$n$ [`Vec`]s using elements from $m$ iterators, where
-        /// $m \leq n$.
+        /// Generates random length-$n$ [`Vec`]s using elements from $m$ iterators, where $m \leq
+        /// n$.
         ///
         /// The `output_to_input_map` parameter defines which iterators are mapped to which slot in
         /// the output [`Vec`]s. The length of the output [`Vec`]s, $n$, is specified by the length
@@ -258,8 +263,8 @@ macro_rules! random_vecs_fixed_length {
         ///
         /// Generates random length-$n$ [`Vec`]s with elements from $n$ iterators.
         ///
-        /// The probability of a particular length-$n$ [`Vec`] being generated is the product of
-        /// the probabilities of each of its elements.
+        /// The probability of a particular length-$n$ [`Vec`] being generated is the product of the
+        /// probabilities of each of its elements.
         ///
         /// `xs`, `ys`, `zs`, ... must be infinite.
         ///
@@ -367,8 +372,8 @@ pub fn random_vecs_from_length_iterator<T, I: Iterator<Item = u64>, J: Iterator<
 
 /// Generates random [`Vec`]s using elements from an iterator.
 ///
-/// The lengths of the [`Vec`]s are sampled from a geometric distribution with a specified mean
-/// $m$, equal to `mean_length_numerator / mean_length_denominator`. $m$ must be greater than 0.
+/// The lengths of the [`Vec`]s are sampled from a geometric distribution with a specified mean $m$,
+/// equal to `mean_length_numerator / mean_length_denominator`. $m$ must be greater than 0.
 ///
 /// $$
 /// P((x\_i)\_{i=0}^{n-1}) = \frac{m^n}{(m+1)^{n+1}}\prod_{i=0}^{n-1}P(x_i).
@@ -433,8 +438,8 @@ pub fn random_vecs<I: Iterator>(
 
 /// Generates random [`Vec`]s with a minimum length, using elements from an iterator.
 ///
-/// The lengths of the [`Vec`]s are sampled from a geometric distribution with a specified mean
-/// $m$, equal to `mean_length_numerator / mean_length_denominator`. $m$ must be greater than
+/// The lengths of the [`Vec`]s are sampled from a geometric distribution with a specified mean $m$,
+/// equal to `mean_length_numerator / mean_length_denominator`. $m$ must be greater than
 /// `min_length`.
 ///
 /// $$
@@ -662,12 +667,12 @@ where
                 out.push(x);
             } else {
                 match x.cmp(&out[0]) {
-                    Ordering::Equal => {}
-                    Ordering::Greater => {
+                    Equal => {}
+                    Greater => {
                         out.push(x);
                         break;
                     }
-                    Ordering::Less => {
+                    Less => {
                         out.insert(0, x);
                         break;
                     }
@@ -732,11 +737,11 @@ where
     }
 }
 
-/// Randomly generates [`Vec`]s of a given length, where the [`Vec`]s have no repeated elements,
-/// and the elements are in ascending order.
+/// Randomly generates [`Vec`]s of a given length, where the [`Vec`]s have no repeated elements, and
+/// the elements are in ascending order.
 ///
-/// The input iterator must generate at least `len` distinct elements; otherwise, this iterator
-/// will hang.
+/// The input iterator must generate at least `len` distinct elements; otherwise, this iterator will
+/// hang.
 ///
 /// $$
 /// P((x\_i)\_{i=0}^{n-1}) = n!\prod\_{i=0}^{n-1}P(x\_i).
@@ -817,8 +822,7 @@ impl<T: Ord, I: Iterator<Item = u64>, J: Iterator<Item = T>> Iterator
 }
 
 /// Generates random [`Vec`]s using elements from an iterator and with lengths from another
-/// iterator, where the [`Vec`]s have no repeated elements, and the elements are in ascending
-/// order.
+/// iterator, where the [`Vec`]s have no repeated elements, and the elements are in ascending order.
 ///
 /// The input iterator must generate at least many distinct elements as any number generated by the
 /// lengths iterator; otherwise, this iterator will hang.
@@ -890,14 +894,14 @@ pub fn random_ordered_unique_vecs_from_length_iterator<
 /// Generates random [`Vec`]s using elements from an iterator, where the [`Vec`]s have no repeated
 /// elements, and the elements are in ascending order.
 ///
-/// The lengths of the [`Vec`]s are sampled from a geometric distribution with a specified mean
-/// $m$, equal to `mean_length_numerator / mean_length_denominator`. $m$ must be greater than 0.
+/// The lengths of the [`Vec`]s are sampled from a geometric distribution with a specified mean $m$,
+/// equal to `mean_length_numerator / mean_length_denominator`. $m$ must be greater than 0.
 ///
 /// Strictly speaking, the input iterator must generate infinitely many distinct elements. In
 /// practice it only needs to generate $k$ distinct elements, where $k$ is the largest length
-/// actually sampled from the geometric distribution. For example, if
-/// `mean_length_numerator / mean_length_denominator` is significantly lower than 256, then it's
-/// ok to use `random_unsigneds::<u8>`.
+/// actually sampled from the geometric distribution. For example, if `mean_length_numerator /
+/// mean_length_denominator` is significantly lower than 256, then it's ok to use
+/// `random_unsigneds::<u8>`.
 ///
 /// $$
 /// P((x\_i)\_{i=0}^{n-1}) = n!P_g(n)\prod\_{i=0}^{n-1}P(x\_i),
@@ -969,9 +973,9 @@ where
 ///
 /// Strictly speaking, the input iterator must generate infinitely many distinct elements. In
 /// practice it only needs to generate $k$ distinct elements, where $k$ is the largest length
-/// actually sampled from the geometric distribution. For example, if
-/// `mean_length_numerator / mean_length_denominator` is significantly lower than 256, then it's
-/// ok to use `random_unsigneds::<u8>`.
+/// actually sampled from the geometric distribution. For example, if `mean_length_numerator /
+/// mean_length_denominator` is significantly lower than 256, then it's ok to use
+/// `random_unsigneds::<u8>`.
 ///
 /// $$
 /// P((x\_i)\_{i=0}^{n-1}) = n!P_g(n)\prod\_{i=0}^{n-1}P(x\_i),
@@ -994,13 +998,8 @@ where
 /// use malachite_base::random::EXAMPLE_SEED;
 /// use malachite_base::vecs::random::random_ordered_unique_vecs_min_length;
 ///
-/// let xs = random_ordered_unique_vecs_min_length(
-///     EXAMPLE_SEED,
-///     2,
-///     &random_primitive_ints::<u8>,
-///     6,
-///     1
-/// );
+/// let xs =
+///     random_ordered_unique_vecs_min_length(EXAMPLE_SEED, 2, &random_primitive_ints::<u8>, 6, 1);
 /// let values = xs.take(20).collect_vec();
 /// assert_eq!(
 ///     values.iter().map(Vec::as_slice).collect_vec().as_slice(),
@@ -1078,12 +1077,8 @@ where
 /// use malachite_base::random::EXAMPLE_SEED;
 /// use malachite_base::vecs::random::random_ordered_unique_vecs_length_range;
 ///
-/// let xs = random_ordered_unique_vecs_length_range(
-///     EXAMPLE_SEED,
-///     2,
-///     5,
-///     &random_primitive_ints::<u8>
-/// );
+/// let xs =
+///     random_ordered_unique_vecs_length_range(EXAMPLE_SEED, 2, 5, &random_primitive_ints::<u8>);
 /// let values = xs.take(20).collect_vec();
 /// assert_eq!(
 ///     values.iter().map(Vec::as_slice).collect_vec().as_slice(),
@@ -1156,7 +1151,7 @@ where
 ///     EXAMPLE_SEED,
 ///     2,
 ///     4,
-///     &random_primitive_ints::<u8>
+///     &random_primitive_ints::<u8>,
 /// );
 /// let values = xs.take(20).collect_vec();
 /// assert_eq!(
@@ -1365,8 +1360,8 @@ where
 
 /// Randomly generates [`Vec`]s of a given length, where the [`Vec`]s have no repeated elements.
 ///
-/// The input iterator must generate at least `len` distinct elements; otherwise, this iterator
-/// will hang.
+/// The input iterator must generate at least `len` distinct elements; otherwise, this iterator will
+/// hang.
 ///
 /// If `len` is 0, the output consists of the empty list, repeated.
 ///
@@ -1423,14 +1418,14 @@ where
 /// Generates random [`Vec`]s using elements from an iterator, where the [`Vec`]s have no repeated
 /// elements.
 ///
-/// The lengths of the [`Vec`]s are sampled from a geometric distribution with a specified mean
-/// $m$, equal to `mean_length_numerator / mean_length_denominator`. $m$ must be greater than 0.
+/// The lengths of the [`Vec`]s are sampled from a geometric distribution with a specified mean $m$,
+/// equal to `mean_length_numerator / mean_length_denominator`. $m$ must be greater than 0.
 ///
 /// Strictly speaking, the input iterator must generate infinitely many distinct elements. In
 /// practice it only needs to generate $k$ distinct elements, where $k$ is the largest length
-/// actually sampled from the geometric distribution. For example, if
-/// `mean_length_numerator / mean_length_denominator` is significantly lower than 256, then it's
-/// ok to use `random_unsigneds::<u8>`.
+/// actually sampled from the geometric distribution. For example, if `mean_length_numerator /
+/// mean_length_denominator` is significantly lower than 256, then it's ok to use
+/// `random_unsigneds::<u8>`.
 ///
 /// `xs_gen` must be infinite.
 ///
@@ -1497,9 +1492,9 @@ where
 ///
 /// Strictly speaking, the input iterator must generate infinitely many distinct elements. In
 /// practice it only needs to generate $k$ distinct elements, where $k$ is the largest length
-/// actually sampled from the geometric distribution. For example, if
-/// `mean_length_numerator / mean_length_denominator` is significantly lower than 256, then it's
-/// ok to use `random_unsigneds::<u8>`.
+/// actually sampled from the geometric distribution. For example, if `mean_length_numerator /
+/// mean_length_denominator` is significantly lower than 256, then it's ok to use
+/// `random_unsigneds::<u8>`.
 ///
 /// `xs_gen` must be infinite.
 ///
@@ -1515,13 +1510,7 @@ where
 /// use malachite_base::random::EXAMPLE_SEED;
 /// use malachite_base::vecs::random::random_unique_vecs_min_length;
 ///
-/// let xs = random_unique_vecs_min_length(
-///     EXAMPLE_SEED,
-///     2,
-///     &random_primitive_ints::<u8>,
-///     6,
-///     1
-/// );
+/// let xs = random_unique_vecs_min_length(EXAMPLE_SEED, 2, &random_primitive_ints::<u8>, 6, 1);
 /// let values = xs.take(20).collect_vec();
 /// assert_eq!(
 ///     values.iter().map(Vec::as_slice).collect_vec().as_slice(),
@@ -1595,12 +1584,7 @@ where
 /// use malachite_base::random::EXAMPLE_SEED;
 /// use malachite_base::vecs::random::random_unique_vecs_length_range;
 ///
-/// let xs = random_unique_vecs_length_range(
-///     EXAMPLE_SEED,
-///     2,
-///     5,
-///     &random_primitive_ints::<u8>
-/// );
+/// let xs = random_unique_vecs_length_range(EXAMPLE_SEED, 2, 5, &random_primitive_ints::<u8>);
 /// let values = xs.take(20).collect_vec();
 /// assert_eq!(
 ///     values.iter().map(Vec::as_slice).collect_vec().as_slice(),
@@ -1665,12 +1649,8 @@ where
 /// use malachite_base::random::EXAMPLE_SEED;
 /// use malachite_base::vecs::random::random_unique_vecs_length_inclusive_range;
 ///
-/// let xs = random_unique_vecs_length_inclusive_range(
-///     EXAMPLE_SEED,
-///     2,
-///     4,
-///     &random_primitive_ints::<u8>
-/// );
+/// let xs =
+///     random_unique_vecs_length_inclusive_range(EXAMPLE_SEED, 2, 4, &random_primitive_ints::<u8>);
 /// let values = xs.take(20).collect_vec();
 /// assert_eq!(
 ///     values.iter().map(Vec::as_slice).collect_vec().as_slice(),
