@@ -1,3 +1,11 @@
+// Copyright © 2024 Mikhail Hogrefe
+//
+// This file is part of Malachite.
+//
+// Malachite is free software: you can redistribute it and/or modify it under the terms of the GNU
+// Lesser General Public License (LGPL) as published by the Free Software Foundation; either version
+// 3 of the License, or (at your option) any later version. See <https://www.gnu.org/licenses/>.
+
 use itertools::__std_iter::empty;
 use malachite_base::iterators::iterator_cache::IteratorCache;
 use malachite_base::nevers::Never;
@@ -11,7 +19,7 @@ fn test_iterator_cache() {
     assert_eq!(xs.known_len(), Some(0));
     assert_eq!(xs.get(0), None);
 
-    let mut xs = IteratorCache::new([1, 2, 3].iter().cloned());
+    let mut xs = IteratorCache::new([1, 2, 3].iter().copied());
     assert_eq!(xs.known_len(), None);
     assert_eq!(xs.get(1), Some(&2));
     assert_eq!(xs.assert_get(1), &2);
