@@ -1,3 +1,11 @@
+// Copyright © 2024 Mikhail Hogrefe
+//
+// This file is part of Malachite.
+//
+// Malachite is free software: you can redistribute it and/or modify it under the terms of the GNU
+// Lesser General Public License (LGPL) as published by the Free Software Foundation; either version
+// 3 of the License, or (at your option) any later version. See <https://www.gnu.org/licenses/>.
+
 #![warn(
     clippy::cast_lossless,
     clippy::explicit_into_iter_loop,
@@ -20,7 +28,17 @@
     clippy::trait_duplication_in_bounds,
     clippy::type_repetition_in_bounds,
     clippy::uninlined_format_args,
-    clippy::unused_self
+    clippy::unused_self,
+    clippy::if_not_else,
+    clippy::manual_assert,
+    clippy::range_plus_one,
+    clippy::redundant_else,
+    clippy::semicolon_if_nothing_returned,
+    clippy::cloned_instead_of_copied,
+    clippy::flat_map_option,
+    clippy::unnecessary_wraps,
+    clippy::unnested_or_patterns,
+    clippy::trivially_copy_pass_by_ref
 )]
 #![allow(
     clippy::assertions_on_constants, // Compile-time asserts still useful
@@ -101,6 +119,7 @@ pub mod bools {
     pub mod exhaustive;
     pub mod not_assign;
     pub mod random {
+        pub mod get_weighted_random_bool;
         pub mod random_bools;
         pub mod weighted_random_bools;
     }
@@ -163,6 +182,7 @@ pub mod iterators {
     pub mod matching_intervals_in_iterator;
     pub mod nonzero_values;
     pub mod prefix_to_string;
+    pub mod thue_morse_sequence;
     pub mod with_special_value;
     pub mod with_special_values;
 }
@@ -174,6 +194,7 @@ pub mod nevers {
 pub mod num {
     pub mod arithmetic {
         pub mod abs;
+        pub mod abs_diff;
         pub mod add_mul;
         pub mod arithmetic_checked_shl;
         pub mod arithmetic_checked_shr;
@@ -241,6 +262,7 @@ pub mod num {
         pub mod pow;
         pub mod power_of_2;
         pub mod primorial;
+        pub mod reciprocal;
         pub mod root;
         pub mod rotate;
         pub mod round_to_multiple;
@@ -373,6 +395,8 @@ pub mod num {
         pub mod primitive_int_increasing_range;
     }
     pub mod factorization {
+        pub mod prime_indicator_sequence;
+        pub mod prime_indicator_sequence_less_than;
         pub mod prime_sieve;
         pub mod primes;
     }
@@ -440,6 +464,7 @@ pub mod num {
             pub mod geometric_random_unsigned_inclusive_range;
             pub mod geometric_random_unsigned_range;
             pub mod geometric_random_unsigneds;
+            pub mod get_geometric_random_signed_from_inclusive_range;
             pub mod mean;
         }
         pub mod random_finite_primitive_floats;
@@ -507,6 +532,7 @@ pub mod num {
         }
         pub mod variable_range_generator {
             pub mod next_bit_chunk;
+            pub mod next_bool;
             pub mod next_in_inclusive_range;
             pub mod next_in_range;
             pub mod next_less_than;
