@@ -1,7 +1,15 @@
+// Copyright © 2024 Mikhail Hogrefe
+//
+// This file is part of Malachite.
+//
+// Malachite is free software: you can redistribute it and/or modify it under the terms of the GNU
+// Lesser General Public License (LGPL) as published by the Free Software Foundation; either version
+// 3 of the License, or (at your option) any later version. See <https://www.gnu.org/licenses/>.
+
 use malachite_base::num::basic::integers::PrimitiveInt;
 use malachite_base::num::basic::signeds::PrimitiveSigned;
 use malachite_base::num::basic::unsigneds::PrimitiveUnsigned;
-use malachite_base::rounding_modes::RoundingMode;
+use malachite_base::rounding_modes::RoundingMode::*;
 use malachite_base::test_util::generators::{
     signed_gen, signed_gen_var_6, signed_pair_gen_var_4, unsigned_gen, unsigned_gen_var_1,
     unsigned_pair_gen_var_12,
@@ -98,6 +106,11 @@ fn div_mod_and_div_rem_properties_helper_unsigned<T: PrimitiveUnsigned>() {
 
     unsigned_gen::<T>().test_properties(|x| {
         assert_eq!(x.div_mod(T::ONE), (x, T::ZERO));
+        assert_panic!(x.div_mod(T::ZERO));
+        assert_panic!({
+            let mut y = x;
+            y.div_assign_mod(T::ZERO)
+        });
     });
 
     unsigned_gen_var_1::<T>().test_properties(|x| {
@@ -386,7 +399,7 @@ fn div_mod_properties_helper_signed<T: PrimitiveSigned>() {
 
         assert_eq!(x.div_mod(y), (q, r));
 
-        let (q_alt, r_alt) = (x.div_round(y, RoundingMode::Floor).0, x.mod_op(y));
+        let (q_alt, r_alt) = (x.div_round(y, Floor).0, x.mod_op(y));
         assert_eq!(q_alt, q);
         assert_eq!(r_alt, r);
 
@@ -419,6 +432,11 @@ fn div_mod_properties_helper_signed<T: PrimitiveSigned>() {
             assert_eq!(q, -x);
             assert_eq!(r, T::ZERO);
         }
+        assert_panic!(x.div_mod(T::ZERO));
+        assert_panic!({
+            let mut y = x;
+            y.div_assign_mod(T::ZERO)
+        });
     });
 
     signed_gen_var_6::<T>().test_properties(|x| {
@@ -728,6 +746,11 @@ fn div_rem_properties_helper_signed<T: PrimitiveSigned>() {
             assert_eq!(q, -x);
             assert_eq!(r, T::ZERO);
         }
+        assert_panic!(x.div_rem(T::ZERO));
+        assert_panic!({
+            let mut y = x;
+            y.div_assign_rem(T::ZERO)
+        });
     });
 
     signed_gen_var_6::<T>().test_properties(|x| {
@@ -838,7 +861,7 @@ fn ceiling_div_neg_mod_properties_helper<T: PrimitiveUnsigned>() {
 
         assert_eq!(x.ceiling_div_neg_mod(y), (q, r));
 
-        let (q_alt, r_alt) = (x.div_round(y, RoundingMode::Ceiling).0, x.neg_mod(y));
+        let (q_alt, r_alt) = (x.div_round(y, Ceiling).0, x.neg_mod(y));
         assert_eq!(q_alt, q);
         assert_eq!(r_alt, r);
 
@@ -852,6 +875,11 @@ fn ceiling_div_neg_mod_properties_helper<T: PrimitiveUnsigned>() {
 
     unsigned_gen::<T>().test_properties(|x| {
         assert_eq!(x.ceiling_div_neg_mod(T::ONE), (x, T::ZERO));
+        assert_panic!(x.ceiling_div_neg_mod(T::ZERO));
+        assert_panic!({
+            let mut y = x;
+            y.ceiling_div_assign_neg_mod(T::ZERO)
+        });
     });
 
     unsigned_gen_var_1::<T>().test_properties(|x| {
@@ -1133,7 +1161,7 @@ fn ceiling_div_mod_properties_helper<T: PrimitiveSigned>() {
 
         assert_eq!(x.ceiling_div_mod(y), (q, r));
 
-        let (q_alt, r_alt) = (x.div_round(y, RoundingMode::Ceiling).0, x.ceiling_mod(y));
+        let (q_alt, r_alt) = (x.div_round(y, Ceiling).0, x.ceiling_mod(y));
         assert_eq!(q_alt, q);
         assert_eq!(r_alt, r);
 
@@ -1167,6 +1195,11 @@ fn ceiling_div_mod_properties_helper<T: PrimitiveSigned>() {
             assert_eq!(q, -x);
             assert_eq!(r, T::ZERO);
         }
+        assert_panic!(x.ceiling_div_mod(T::ZERO));
+        assert_panic!({
+            let mut y = x;
+            y.ceiling_div_assign_mod(T::ZERO)
+        });
     });
 
     signed_gen_var_6::<T>().test_properties(|x| {
