@@ -1,3 +1,11 @@
+// Copyright © 2024 Mikhail Hogrefe
+//
+// This file is part of Malachite.
+//
+// Malachite is free software: you can redistribute it and/or modify it under the terms of the GNU
+// Lesser General Public License (LGPL) as published by the Free Software Foundation; either version
+// 3 of the License, or (at your option) any later version. See <https://www.gnu.org/licenses/>.
+
 use malachite_base::num::arithmetic::root::{
     cbrt_chebyshev_approx_u32, cbrt_chebyshev_approx_u64, fast_ceiling_root_u32,
     fast_ceiling_root_u64, fast_checked_root_u32, fast_checked_root_u64, fast_floor_cbrt_u32,
@@ -489,6 +497,7 @@ fn floor_root_properties_helper_signed<T: PrimitiveSigned>() {
 
 macro_rules! floor_root_approx_and_refine_helper {
     ($t:ty) => {
+        #[allow(clippy::cast_lossless)]
         unsigned_pair_gen_var_32::<$t, u64>().test_properties(|(n, exp)| {
             assert_eq!(
                 floor_root_approx_and_refine(|x| x as f64, |f| f as $t, n, exp),
@@ -496,6 +505,7 @@ macro_rules! floor_root_approx_and_refine_helper {
             );
         });
 
+        #[allow(clippy::cast_lossless)]
         unsigned_gen::<$t>().test_properties(|n| {
             assert_eq!(
                 floor_root_approx_and_refine(|x| x as f64, |f| f as $t, n, 3),
