@@ -1,8 +1,16 @@
+// Copyright © 2024 Mikhail Hogrefe
+//
+// This file is part of Malachite.
+//
+// Malachite is free software: you can redistribute it and/or modify it under the terms of the GNU
+// Lesser General Public License (LGPL) as published by the Free Software Foundation; either version
+// 3 of the License, or (at your option) any later version. See <https://www.gnu.org/licenses/>.
+
 use malachite_base::num::basic::integers::PrimitiveInt;
 use malachite_base::num::basic::signeds::PrimitiveSigned;
 use malachite_base::num::basic::unsigneds::PrimitiveUnsigned;
 use malachite_base::num::conversion::string::options::FromSciStringOptions;
-use malachite_base::rounding_modes::RoundingMode;
+use malachite_base::rounding_modes::RoundingMode::*;
 use malachite_base::test_util::generators::{
     signed_gen, string_from_sci_string_options_pair_gen,
     string_from_sci_string_options_pair_gen_var_1, string_gen, string_gen_var_13, unsigned_gen,
@@ -343,56 +351,56 @@ pub fn test_from_sci_string_with_options() {
     test::<u8>("FF", options, Some(255));
 
     options = FromSciStringOptions::default();
-    options.set_rounding_mode(RoundingMode::Down);
+    options.set_rounding_mode(Down);
     test::<u8>("123.4", options, Some(123));
-    options.set_rounding_mode(RoundingMode::Floor);
+    options.set_rounding_mode(Floor);
     test::<u8>("123.4", options, Some(123));
-    options.set_rounding_mode(RoundingMode::Up);
+    options.set_rounding_mode(Up);
     test::<u8>("123.4", options, Some(124));
-    options.set_rounding_mode(RoundingMode::Ceiling);
+    options.set_rounding_mode(Ceiling);
     test::<u8>("123.4", options, Some(124));
-    options.set_rounding_mode(RoundingMode::Nearest);
+    options.set_rounding_mode(Nearest);
     test::<u8>("123.4", options, Some(123));
-    options.set_rounding_mode(RoundingMode::Exact);
+    options.set_rounding_mode(Exact);
     test::<u8>("123.4", options, None);
 
-    options.set_rounding_mode(RoundingMode::Down);
+    options.set_rounding_mode(Down);
     test::<u8>("123.5", options, Some(123));
-    options.set_rounding_mode(RoundingMode::Floor);
+    options.set_rounding_mode(Floor);
     test::<u8>("123.5", options, Some(123));
-    options.set_rounding_mode(RoundingMode::Up);
+    options.set_rounding_mode(Up);
     test::<u8>("123.5", options, Some(124));
-    options.set_rounding_mode(RoundingMode::Ceiling);
+    options.set_rounding_mode(Ceiling);
     test::<u8>("123.5", options, Some(124));
-    options.set_rounding_mode(RoundingMode::Nearest);
+    options.set_rounding_mode(Nearest);
     test::<u8>("123.5", options, Some(124));
-    options.set_rounding_mode(RoundingMode::Exact);
+    options.set_rounding_mode(Exact);
     test::<u8>("123.5", options, None);
 
-    options.set_rounding_mode(RoundingMode::Down);
+    options.set_rounding_mode(Down);
     test::<u8>("0.4", options, Some(0));
-    options.set_rounding_mode(RoundingMode::Floor);
+    options.set_rounding_mode(Floor);
     test::<u8>("0.4", options, Some(0));
-    options.set_rounding_mode(RoundingMode::Up);
+    options.set_rounding_mode(Up);
     test::<u8>("0.4", options, Some(1));
-    options.set_rounding_mode(RoundingMode::Ceiling);
+    options.set_rounding_mode(Ceiling);
     test::<u8>("0.4", options, Some(1));
-    options.set_rounding_mode(RoundingMode::Nearest);
+    options.set_rounding_mode(Nearest);
     test::<u8>("0.4", options, Some(0));
-    options.set_rounding_mode(RoundingMode::Exact);
+    options.set_rounding_mode(Exact);
     test::<u8>("0.4", options, None);
 
-    options.set_rounding_mode(RoundingMode::Down);
+    options.set_rounding_mode(Down);
     test::<u8>("0.04", options, Some(0));
-    options.set_rounding_mode(RoundingMode::Floor);
+    options.set_rounding_mode(Floor);
     test::<u8>("0.04", options, Some(0));
-    options.set_rounding_mode(RoundingMode::Up);
+    options.set_rounding_mode(Up);
     test::<u8>("0.04", options, Some(1));
-    options.set_rounding_mode(RoundingMode::Ceiling);
+    options.set_rounding_mode(Ceiling);
     test::<u8>("0.04", options, Some(1));
-    options.set_rounding_mode(RoundingMode::Nearest);
+    options.set_rounding_mode(Nearest);
     test::<u8>("0.04", options, Some(0));
-    options.set_rounding_mode(RoundingMode::Exact);
+    options.set_rounding_mode(Exact);
     test::<u8>("0.04", options, None);
 
     options = FromSciStringOptions::default();
@@ -426,7 +434,7 @@ pub fn test_from_sci_string_with_options() {
     test::<u8>("0.002", options, None);
 
     options = FromSciStringOptions::default();
-    options.set_rounding_mode(RoundingMode::Exact);
+    options.set_rounding_mode(Exact);
     test::<u8>("1.5", options, None);
     test::<u8>("1.9999999999999999999999999999", options, None);
 
@@ -564,108 +572,108 @@ pub fn test_from_sci_string_with_options() {
     test::<i16>("-FF", options, Some(-255));
 
     options = FromSciStringOptions::default();
-    options.set_rounding_mode(RoundingMode::Down);
+    options.set_rounding_mode(Down);
     test::<i8>("123.4", options, Some(123));
-    options.set_rounding_mode(RoundingMode::Floor);
+    options.set_rounding_mode(Floor);
     test::<i8>("123.4", options, Some(123));
-    options.set_rounding_mode(RoundingMode::Up);
+    options.set_rounding_mode(Up);
     test::<i8>("123.4", options, Some(124));
-    options.set_rounding_mode(RoundingMode::Ceiling);
+    options.set_rounding_mode(Ceiling);
     test::<i8>("123.4", options, Some(124));
-    options.set_rounding_mode(RoundingMode::Nearest);
+    options.set_rounding_mode(Nearest);
     test::<i8>("123.4", options, Some(123));
-    options.set_rounding_mode(RoundingMode::Exact);
+    options.set_rounding_mode(Exact);
     test::<i8>("123.4", options, None);
 
-    options.set_rounding_mode(RoundingMode::Down);
+    options.set_rounding_mode(Down);
     test::<i8>("-123.4", options, Some(-123));
-    options.set_rounding_mode(RoundingMode::Floor);
+    options.set_rounding_mode(Floor);
     test::<i8>("-123.4", options, Some(-124));
-    options.set_rounding_mode(RoundingMode::Up);
+    options.set_rounding_mode(Up);
     test::<i8>("-123.4", options, Some(-124));
-    options.set_rounding_mode(RoundingMode::Ceiling);
+    options.set_rounding_mode(Ceiling);
     test::<i8>("-123.4", options, Some(-123));
-    options.set_rounding_mode(RoundingMode::Nearest);
+    options.set_rounding_mode(Nearest);
     test::<i8>("-123.4", options, Some(-123));
-    options.set_rounding_mode(RoundingMode::Exact);
+    options.set_rounding_mode(Exact);
     test::<i8>("-123.4", options, None);
 
-    options.set_rounding_mode(RoundingMode::Down);
+    options.set_rounding_mode(Down);
     test::<i8>("123.5", options, Some(123));
-    options.set_rounding_mode(RoundingMode::Floor);
+    options.set_rounding_mode(Floor);
     test::<i8>("123.5", options, Some(123));
-    options.set_rounding_mode(RoundingMode::Up);
+    options.set_rounding_mode(Up);
     test::<i8>("123.5", options, Some(124));
-    options.set_rounding_mode(RoundingMode::Ceiling);
+    options.set_rounding_mode(Ceiling);
     test::<i8>("123.5", options, Some(124));
-    options.set_rounding_mode(RoundingMode::Nearest);
+    options.set_rounding_mode(Nearest);
     test::<i8>("123.5", options, Some(124));
-    options.set_rounding_mode(RoundingMode::Exact);
+    options.set_rounding_mode(Exact);
     test::<i8>("123.5", options, None);
 
-    options.set_rounding_mode(RoundingMode::Down);
+    options.set_rounding_mode(Down);
     test::<i8>("-123.5", options, Some(-123));
-    options.set_rounding_mode(RoundingMode::Floor);
+    options.set_rounding_mode(Floor);
     test::<i8>("-123.5", options, Some(-124));
-    options.set_rounding_mode(RoundingMode::Up);
+    options.set_rounding_mode(Up);
     test::<i8>("-123.5", options, Some(-124));
-    options.set_rounding_mode(RoundingMode::Ceiling);
+    options.set_rounding_mode(Ceiling);
     test::<i8>("-123.5", options, Some(-123));
-    options.set_rounding_mode(RoundingMode::Nearest);
+    options.set_rounding_mode(Nearest);
     test::<i8>("-123.5", options, Some(-124));
-    options.set_rounding_mode(RoundingMode::Exact);
+    options.set_rounding_mode(Exact);
     test::<i8>("-123.5", options, None);
 
-    options.set_rounding_mode(RoundingMode::Down);
+    options.set_rounding_mode(Down);
     test::<i8>("0.4", options, Some(0));
-    options.set_rounding_mode(RoundingMode::Floor);
+    options.set_rounding_mode(Floor);
     test::<i8>("0.4", options, Some(0));
-    options.set_rounding_mode(RoundingMode::Up);
+    options.set_rounding_mode(Up);
     test::<i8>("0.4", options, Some(1));
-    options.set_rounding_mode(RoundingMode::Ceiling);
+    options.set_rounding_mode(Ceiling);
     test::<i8>("0.4", options, Some(1));
-    options.set_rounding_mode(RoundingMode::Nearest);
+    options.set_rounding_mode(Nearest);
     test::<i8>("0.4", options, Some(0));
-    options.set_rounding_mode(RoundingMode::Exact);
+    options.set_rounding_mode(Exact);
     test::<i8>("0.4", options, None);
 
-    options.set_rounding_mode(RoundingMode::Down);
+    options.set_rounding_mode(Down);
     test::<i8>("-0.4", options, Some(0));
-    options.set_rounding_mode(RoundingMode::Floor);
+    options.set_rounding_mode(Floor);
     test::<i8>("-0.4", options, Some(-1));
-    options.set_rounding_mode(RoundingMode::Up);
+    options.set_rounding_mode(Up);
     test::<i8>("-0.4", options, Some(-1));
-    options.set_rounding_mode(RoundingMode::Ceiling);
+    options.set_rounding_mode(Ceiling);
     test::<i8>("-0.4", options, Some(0));
-    options.set_rounding_mode(RoundingMode::Nearest);
+    options.set_rounding_mode(Nearest);
     test::<i8>("-0.4", options, Some(0));
-    options.set_rounding_mode(RoundingMode::Exact);
+    options.set_rounding_mode(Exact);
     test::<i8>("-0.4", options, None);
 
-    options.set_rounding_mode(RoundingMode::Down);
+    options.set_rounding_mode(Down);
     test::<i8>("0.04", options, Some(0));
-    options.set_rounding_mode(RoundingMode::Floor);
+    options.set_rounding_mode(Floor);
     test::<i8>("0.04", options, Some(0));
-    options.set_rounding_mode(RoundingMode::Up);
+    options.set_rounding_mode(Up);
     test::<i8>("0.04", options, Some(1));
-    options.set_rounding_mode(RoundingMode::Ceiling);
+    options.set_rounding_mode(Ceiling);
     test::<i8>("0.04", options, Some(1));
-    options.set_rounding_mode(RoundingMode::Nearest);
+    options.set_rounding_mode(Nearest);
     test::<i8>("0.04", options, Some(0));
-    options.set_rounding_mode(RoundingMode::Exact);
+    options.set_rounding_mode(Exact);
     test::<i8>("0.04", options, None);
 
-    options.set_rounding_mode(RoundingMode::Down);
+    options.set_rounding_mode(Down);
     test::<i8>("-0.04", options, Some(0));
-    options.set_rounding_mode(RoundingMode::Floor);
+    options.set_rounding_mode(Floor);
     test::<i8>("-0.04", options, Some(-1));
-    options.set_rounding_mode(RoundingMode::Up);
+    options.set_rounding_mode(Up);
     test::<i8>("-0.04", options, Some(-1));
-    options.set_rounding_mode(RoundingMode::Ceiling);
+    options.set_rounding_mode(Ceiling);
     test::<i8>("-0.04", options, Some(0));
-    options.set_rounding_mode(RoundingMode::Nearest);
+    options.set_rounding_mode(Nearest);
     test::<i8>("-0.04", options, Some(0));
-    options.set_rounding_mode(RoundingMode::Exact);
+    options.set_rounding_mode(Exact);
     test::<i8>("-0.04", options, None);
 
     options = FromSciStringOptions::default();
@@ -719,11 +727,11 @@ pub fn test_from_sci_string_with_options() {
     test::<i8>("-0.2", options, None);
     test::<i8>("-0.002", options, None);
     options.set_base(25);
-    options.set_rounding_mode(RoundingMode::Ceiling);
+    options.set_rounding_mode(Ceiling);
     test::<i8>(".be.2", options, None);
 
     options = FromSciStringOptions::default();
-    options.set_rounding_mode(RoundingMode::Exact);
+    options.set_rounding_mode(Exact);
     test::<i8>("1.5", options, None);
     test::<i8>("1.9999999999999999999999999999", options, None);
     test::<i8>("-1.5", options, None);
