@@ -1,10 +1,18 @@
+// Copyright © 2024 Mikhail Hogrefe
+//
+// This file is part of Malachite.
+//
+// Malachite is free software: you can redistribute it and/or modify it under the terms of the GNU
+// Lesser General Public License (LGPL) as published by the Free Software Foundation; either version
+// 3 of the License, or (at your option) any later version. See <https://www.gnu.org/licenses/>.
+
 use malachite_base::num::basic::floats::PrimitiveFloat;
 use malachite_base::num::float::NiceFloat;
 use malachite_base::test_util::common::test_cmp_helper;
 use malachite_base::test_util::generators::{
     primitive_float_gen, primitive_float_pair_gen, primitive_float_triple_gen,
 };
-use std::cmp::Ordering;
+use std::cmp::Ordering::*;
 
 const TEST_STRINGS: [&str; 7] = ["-Infinity", "-5.0e5", "-0.0", "NaN", "0.0", "0.123", "Infinity"];
 
@@ -25,7 +33,7 @@ fn cmp_properties_helper<T: PrimitiveFloat>() {
 
     primitive_float_gen::<T>().test_properties(|x| {
         let x = NiceFloat(x);
-        assert_eq!(x.cmp(&x), Ordering::Equal);
+        assert_eq!(x.cmp(&x), Equal);
     });
 
     primitive_float_triple_gen::<T>().test_properties(|(x, y, z)| {
