@@ -1,9 +1,17 @@
+// Copyright © 2024 Mikhail Hogrefe
+//
+// This file is part of Malachite.
+//
+// Malachite is free software: you can redistribute it and/or modify it under the terms of the GNU
+// Lesser General Public License (LGPL) as published by the Free Software Foundation; either version
+// 3 of the License, or (at your option) any later version. See <https://www.gnu.org/licenses/>.
+
 use itertools::Itertools;
 use malachite_base::orderings::random::random_orderings;
 use malachite_base::random::EXAMPLE_SEED;
 use malachite_base::test_util::stats::common_values_map::common_values_map_debug;
 use malachite_base::test_util::stats::median;
-use std::cmp::Ordering::{Equal, Greater, Less};
+use std::cmp::Ordering::*;
 
 #[test]
 fn test_random_orderings() {
